@@ -214,26 +214,30 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
           {
             length=GetQuantumExtent(canvas_image,quantum_info,quantum_type);
             count=ReadBlob(image,length,pixels);
-            if (count != (ssize_t) length)
-              break;
           }
         for (y=0; y < (long) image->extract_info.height; y++)
         {
           register const IndexPacket
-            *__restrict canvas_indexes;
+            *restrict canvas_indexes;
 
           register const PixelPacket
-            *__restrict p;
+            *restrict p;
 
           register IndexPacket
-            *__restrict indexes;
+            *restrict indexes;
 
           register long
             x;
 
           register PixelPacket
-            *__restrict q;
+            *restrict q;
 
+          if (count != (ssize_t) length)
+            {
+              ThrowFileException(exception,CorruptImageError,
+                "UnexpectedEndOfFile",image->filename);
+              break;
+            }
           q=GetAuthenticPixels(canvas_image,0,0,canvas_image->columns,1,
             exception);
           if (q == (PixelPacket *) NULL)
@@ -276,8 +280,6 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
                 break;
             }
           count=ReadBlob(image,length,pixels);
-          if (count != (ssize_t) length)
-            break;
         }
         break;
       }
@@ -300,26 +302,30 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
           {
             length=GetQuantumExtent(canvas_image,quantum_info,CyanQuantum);
             count=ReadBlob(image,length,pixels);
-            if (count != (ssize_t) length)
-              break;
           }
         for (y=0; y < (long) image->extract_info.height; y++)
         {
           register const IndexPacket
-            *__restrict canvas_indexes;
+            *restrict canvas_indexes;
 
           register const PixelPacket
-            *__restrict p;
+            *restrict p;
 
           register IndexPacket
-            *__restrict indexes;
+            *restrict indexes;
 
           register long
             x;
 
           register PixelPacket
-            *__restrict q;
+            *restrict q;
 
+          if (count != (ssize_t) length)
+            {
+              ThrowFileException(exception,CorruptImageError,
+                "UnexpectedEndOfFile",image->filename);
+              break;
+            }
           for (i=0; i < (image->matte != MagickFalse ? 5 : 4); i++)
           {
             quantum_type=quantum_types[i];
@@ -362,8 +368,6 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
                   break;
               }
             count=ReadBlob(image,length,pixels);
-            if (count != (ssize_t) length)
-              break;
           }
           if (image->previous == (Image *) NULL)
             {
@@ -383,20 +387,24 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
           {
             length=GetQuantumExtent(canvas_image,quantum_info,CyanQuantum);
             count=ReadBlob(image,length,pixels);
-            if (count != (ssize_t) length)
-              break;
           }
         for (y=0; y < (long) image->extract_info.height; y++)
         {
           register const PixelPacket
-            *__restrict p;
+            *restrict p;
 
           register long
             x;
 
           register PixelPacket
-            *__restrict q;
+            *restrict q;
 
+          if (count != (ssize_t) length)
+            {
+              ThrowFileException(exception,CorruptImageError,
+                "UnexpectedEndOfFile",image->filename);
+              break;
+            }
           q=GetAuthenticPixels(canvas_image,0,0,canvas_image->columns,1,
             exception);
           if (q == (PixelPacket *) NULL)
@@ -425,8 +433,6 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
                 break;
             }
           count=ReadBlob(image,length,pixels);
-          if (count != (ssize_t) length)
-            break;
         }
         if (image->previous == (Image *) NULL)
           {
@@ -437,14 +443,20 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
         for (y=0; y < (long) image->extract_info.height; y++)
         {
           register const PixelPacket
-            *__restrict p;
+            *restrict p;
 
           register long
             x;
 
           register PixelPacket
-            *__restrict q;
+            *restrict q;
 
+          if (count != (ssize_t) length)
+            {
+              ThrowFileException(exception,CorruptImageError,
+                "UnexpectedEndOfFile",image->filename);
+              break;
+            }
           q=GetAuthenticPixels(canvas_image,0,0,canvas_image->columns,1,
             exception);
           if (q == (PixelPacket *) NULL)
@@ -473,8 +485,6 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
                 break;
            }
           count=ReadBlob(image,length,pixels);
-          if (count != (ssize_t) length)
-            break;
         }
         if (image->previous == (Image *) NULL)
           {
@@ -485,14 +495,20 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
         for (y=0; y < (long) image->extract_info.height; y++)
         {
           register const PixelPacket
-            *__restrict p;
+            *restrict p;
 
           register long
             x;
 
           register PixelPacket
-            *__restrict q;
+            *restrict q;
 
+          if (count != (ssize_t) length)
+            {
+              ThrowFileException(exception,CorruptImageError,
+                "UnexpectedEndOfFile",image->filename);
+              break;
+            }
           q=GetAuthenticPixels(canvas_image,0,0,canvas_image->columns,1,
             exception);
           if (q == (PixelPacket *) NULL)
@@ -521,8 +537,6 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
                 break;
             }
           count=ReadBlob(image,length,pixels);
-          if (count != (ssize_t) length)
-            break;
         }
         if (image->previous == (Image *) NULL)
           {
@@ -533,20 +547,26 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
         for (y=0; y < (long) image->extract_info.height; y++)
         {
           register const IndexPacket
-            *__restrict canvas_indexes;
+            *restrict canvas_indexes;
 
           register const PixelPacket
-            *__restrict p;
+            *restrict p;
 
           register IndexPacket
-            *__restrict indexes;
+            *restrict indexes;
 
           register long
             x;
 
           register PixelPacket
-            *__restrict q;
+            *restrict q;
 
+          if (count != (ssize_t) length)
+            {
+              ThrowFileException(exception,CorruptImageError,
+                "UnexpectedEndOfFile",image->filename);
+              break;
+            }
           q=GetAuthenticPixels(canvas_image,0,0,canvas_image->columns,1,
             exception);
           if (q == (PixelPacket *) NULL)
@@ -577,8 +597,6 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
                 break;
             }
           count=ReadBlob(image,length,pixels);
-          if (count != (ssize_t) length)
-            break;
         }
         if (image->previous == (Image *) NULL)
           {
@@ -591,14 +609,20 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
             for (y=0; y < (long) image->extract_info.height; y++)
             {
               register const PixelPacket
-                *__restrict p;
+                *restrict p;
 
               register long
                 x;
 
               register PixelPacket
-                *__restrict q;
+                *restrict q;
 
+              if (count != (ssize_t) length)
+                {
+                  ThrowFileException(exception,CorruptImageError,
+                    "UnexpectedEndOfFile",image->filename);
+                  break;
+                }
               q=GetAuthenticPixels(canvas_image,0,0,canvas_image->columns,1,
                 exception);
               if (q == (PixelPacket *) NULL)
@@ -628,8 +652,6 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
                     break;
                 }
               count=ReadBlob(image,length,pixels);
-              if (count != (ssize_t) length)
-                break;
             }
             if (image->previous == (Image *) NULL)
               {
@@ -676,19 +698,23 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
                 break;
               }
         count=ReadBlob(image,length,pixels);
-        if (count != (ssize_t) length)
-          break;
         for (y=0; y < (long) image->extract_info.height; y++)
         {
           register const PixelPacket
-            *__restrict p;
+            *restrict p;
 
           register long
             x;
 
           register PixelPacket
-            *__restrict q;
+            *restrict q;
 
+          if (count != (ssize_t) length)
+            {
+              ThrowFileException(exception,CorruptImageError,
+                "UnexpectedEndOfFile",image->filename);
+              break;
+            }
           q=GetAuthenticPixels(canvas_image,0,0,canvas_image->columns,1,
             exception);
           if (q == (PixelPacket *) NULL)
@@ -717,8 +743,6 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
                 break;
             }
           count=ReadBlob(image,length,pixels);
-          if (count != (ssize_t) length)
-            break;
         }
         if (image->previous == (Image *) NULL)
           {
@@ -745,19 +769,23 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
                 break;
               }
         count=ReadBlob(image,length,pixels);
-        if (count != (ssize_t) length)
-          break;
         for (y=0; y < (long) image->extract_info.height; y++)
         {
           register const PixelPacket
-            *__restrict p;
+            *restrict p;
 
           register long
             x;
 
           register PixelPacket
-            *__restrict q;
+            *restrict q;
 
+          if (count != (ssize_t) length)
+            {
+              ThrowFileException(exception,CorruptImageError,
+                "UnexpectedEndOfFile",image->filename);
+              break;
+            }
           q=GetAuthenticPixels(canvas_image,0,0,canvas_image->columns,1,
             exception);
           if (q == (PixelPacket *) NULL)
@@ -786,8 +814,6 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
                 break;
            }
           count=ReadBlob(image,length,pixels);
-          if (count != (ssize_t) length)
-            break;
         }
         if (image->previous == (Image *) NULL)
           {
@@ -814,19 +840,23 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
                 break;
               }
         count=ReadBlob(image,length,pixels);
-        if (count != (ssize_t) length)
-          break;
         for (y=0; y < (long) image->extract_info.height; y++)
         {
           register const PixelPacket
-            *__restrict p;
+            *restrict p;
 
           register long
             x;
 
           register PixelPacket
-            *__restrict q;
+            *restrict q;
 
+          if (count != (ssize_t) length)
+            {
+              ThrowFileException(exception,CorruptImageError,
+                "UnexpectedEndOfFile",image->filename);
+              break;
+            }
           q=GetAuthenticPixels(canvas_image,0,0,canvas_image->columns,1,
             exception);
           if (q == (PixelPacket *) NULL)
@@ -855,8 +885,6 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
                 break;
            }
           count=ReadBlob(image,length,pixels);
-          if (count != (ssize_t) length)
-            break;
         }
         if (image->previous == (Image *) NULL)
           {
@@ -883,25 +911,29 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
                 break;
               }
         count=ReadBlob(image,length,pixels);
-        if (count != (ssize_t) length)
-          break;
         for (y=0; y < (long) image->extract_info.height; y++)
         {
           register const IndexPacket
-            *__restrict canvas_indexes;
+            *restrict canvas_indexes;
 
           register const PixelPacket
-            *__restrict p;
+            *restrict p;
 
           register IndexPacket
-            *__restrict indexes;
+            *restrict indexes;
 
           register long
             x;
 
           register PixelPacket
-            *__restrict q;
+            *restrict q;
 
+          if (count != (ssize_t) length)
+            {
+              ThrowFileException(exception,CorruptImageError,
+                "UnexpectedEndOfFile",image->filename);
+              break;
+            }
           q=GetAuthenticPixels(canvas_image,0,0,canvas_image->columns,1,
             exception);
           if (q == (PixelPacket *) NULL)
@@ -932,8 +964,6 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
                 break;
            }
           count=ReadBlob(image,length,pixels);
-          if (count != (ssize_t) length)
-            break;
         }
         if (image->previous == (Image *) NULL)
           {
@@ -962,19 +992,23 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
                     break;
                   }
             count=ReadBlob(image,length,pixels);
-            if (count != (ssize_t) length)
-              break;
             for (y=0; y < (long) image->extract_info.height; y++)
             {
               register const PixelPacket
-                *__restrict p;
+                *restrict p;
 
               register long
                 x;
 
               register PixelPacket
-                *__restrict q;
+                *restrict q;
 
+              if (count != (ssize_t) length)
+                {
+                  ThrowFileException(exception,CorruptImageError,
+                    "UnexpectedEndOfFile",image->filename);
+                  break;
+                }
               q=GetAuthenticPixels(canvas_image,0,0,canvas_image->columns,1,
                 exception);
               if (q == (PixelPacket *) NULL)
@@ -1003,8 +1037,6 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
                     break;
                }
               count=ReadBlob(image,length,pixels);
-              if (count != (ssize_t) length)
-                break;
             }
             if (image->previous == (Image *) NULL)
               {
@@ -1048,8 +1080,8 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
       }
     scene++;
   } while (count == (ssize_t) length);
-  InheritException(exception,&image->exception);
   quantum_info=DestroyQuantumInfo(quantum_info);
+  InheritException(&image->exception,&canvas_image->exception);
   canvas_image=DestroyImage(canvas_image);
   (void) CloseBlob(image);
   return(GetFirstImageInList(image));
@@ -1234,7 +1266,7 @@ static MagickBooleanType WriteCMYKImage(const ImageInfo *image_info,
         for (y=0; y < (long) image->rows; y++)
         {
           register const PixelPacket
-            *__restrict p;
+            *restrict p;
 
           p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
@@ -1261,7 +1293,7 @@ static MagickBooleanType WriteCMYKImage(const ImageInfo *image_info,
         for (y=0; y < (long) image->rows; y++)
         {
           register const PixelPacket
-            *__restrict p;
+            *restrict p;
 
           p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
@@ -1311,7 +1343,7 @@ static MagickBooleanType WriteCMYKImage(const ImageInfo *image_info,
         for (y=0; y < (long) image->rows; y++)
         {
           register const PixelPacket
-            *__restrict p;
+            *restrict p;
 
           p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
@@ -1331,7 +1363,7 @@ static MagickBooleanType WriteCMYKImage(const ImageInfo *image_info,
         for (y=0; y < (long) image->rows; y++)
         {
           register const PixelPacket
-            *__restrict p;
+            *restrict p;
 
           p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
@@ -1351,7 +1383,7 @@ static MagickBooleanType WriteCMYKImage(const ImageInfo *image_info,
         for (y=0; y < (long) image->rows; y++)
         {
           register const PixelPacket
-            *__restrict p;
+            *restrict p;
 
           p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
@@ -1371,7 +1403,7 @@ static MagickBooleanType WriteCMYKImage(const ImageInfo *image_info,
         for (y=0; y < (long) image->rows; y++)
         {
           register const PixelPacket
-            *__restrict p;
+            *restrict p;
 
           p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
@@ -1393,7 +1425,7 @@ static MagickBooleanType WriteCMYKImage(const ImageInfo *image_info,
             for (y=0; y < (long) image->rows; y++)
             {
               register const PixelPacket
-                *__restrict p;
+                *restrict p;
 
               p=GetVirtualPixels(image,0,y,image->columns,1,
                 &image->exception);
@@ -1436,7 +1468,7 @@ static MagickBooleanType WriteCMYKImage(const ImageInfo *image_info,
         for (y=0; y < (long) image->rows; y++)
         {
           register const PixelPacket
-            *__restrict p;
+            *restrict p;
 
           p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
@@ -1462,7 +1494,7 @@ static MagickBooleanType WriteCMYKImage(const ImageInfo *image_info,
         for (y=0; y < (long) image->rows; y++)
         {
           register const PixelPacket
-            *__restrict p;
+            *restrict p;
 
           p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
@@ -1488,7 +1520,7 @@ static MagickBooleanType WriteCMYKImage(const ImageInfo *image_info,
         for (y=0; y < (long) image->rows; y++)
         {
           register const PixelPacket
-            *__restrict p;
+            *restrict p;
 
           p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
@@ -1514,7 +1546,7 @@ static MagickBooleanType WriteCMYKImage(const ImageInfo *image_info,
         for (y=0; y < (long) image->rows; y++)
         {
           register const PixelPacket
-            *__restrict p;
+            *restrict p;
 
           p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
@@ -1542,7 +1574,7 @@ static MagickBooleanType WriteCMYKImage(const ImageInfo *image_info,
             for (y=0; y < (long) image->rows; y++)
             {
               register const PixelPacket
-                *__restrict p;
+                *restrict p;
 
               p=GetVirtualPixels(image,0,y,image->columns,1,
                 &image->exception);
