@@ -44,6 +44,7 @@
 #include "magick/blob-private.h"
 #include "magick/cache.h"
 #include "magick/color.h"
+#include "magick/colormap-private.h"
 #include "magick/color-private.h"
 #include "magick/colorspace.h"
 #include "magick/exception.h"
@@ -367,7 +368,7 @@ static int ReadBlobLZWByte(LZWInfo *lzw_info)
   return(PopLZWStack(lzw_info->stack));
 }
 
-static MagickBooleanType DecodeImage(Image *image,const Quantum opacity)
+static MagickBooleanType DecodeImage(Image *image,const long opacity)
 {
   ExceptionInfo
     *exception;
@@ -382,15 +383,6 @@ static MagickBooleanType DecodeImage(Image *image,const Quantum opacity)
     offset,
     y;
 
-  register IndexPacket
-    *indexes;
-
-  register long
-    x;
-
-  register PixelPacket
-    *q;
-
   LZWInfo
     *lzw_info;
 
@@ -419,6 +411,15 @@ static MagickBooleanType DecodeImage(Image *image,const Quantum opacity)
   offset=0;
   for (y=0; y < (long) image->rows; y++)
   {
+    register IndexPacket
+      *restrict indexes;
+
+    register long
+      x;
+
+    register PixelPacket
+      *restrict q;
+
     q=GetAuthenticPixels(image,0,offset,image->columns,1,exception);
     if (q == (PixelPacket *) NULL)
       break;
@@ -432,7 +433,7 @@ static MagickBooleanType DecodeImage(Image *image,const Quantum opacity)
       q->red=image->colormap[(long) index].red;
       q->green=image->colormap[(long) index].green;
       q->blue=image->colormap[(long) index].blue;
-      q->opacity=index == opacity ? (Quantum) TransparentOpacity :
+      q->opacity=(long) index == opacity ? (Quantum) TransparentOpacity :
         (Quantum) OpaqueOpacity;
       indexes[x]=index;
       x++;
@@ -569,15 +570,8 @@ static MagickBooleanType EncodeImage(const ImageInfo *image_info,Image *image,
     k,
     y;
 
-  register const IndexPacket
-    *indexes;
-
-  register const PixelPacket
-    *p;
-
   register long
-    i,
-    x;
+    i;
 
   size_t
     length;
@@ -647,6 +641,15 @@ static MagickBooleanType EncodeImage(const ImageInfo *image_info,Image *image,
   waiting_code=0;
   for (y=0; y < (long) image->rows; y++)
   {
+    register const IndexPacket
+      *restrict indexes;
+
+    register const PixelPacket
+      *restrict p;
+
+    register long
+      x;
+
     p=GetVirtualPixels(image,0,offset,image->columns,1,&image->exception);
     if (p == (const PixelPacket *) NULL)
       break;
@@ -816,8 +819,7 @@ static MagickBooleanType EncodeImage(const ImageInfo *image_info,Image *image,
 %
 %  A description of each parameter follows:
 %
-%    o magick: This string is generally the first few bytes of an image file
-%      or blob.
+%    o magick: compare image format pattern against these bytes.
 %
 %    o length: Specifies the length of the magick string.
 %
@@ -1159,15 +1161,15 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 SetStringInfoDatum(profile,(const unsigned char *) info);
                 if (i8bim == MagickTrue)
                   (void) CopyMagickString(name,"8bim",sizeof(name));
-                if (icc == MagickTrue)
+                else if (icc == MagickTrue)
                   (void) CopyMagickString(name,"icc",sizeof(name));
+                else if (iptc == MagickTrue)
+                  (void) CopyMagickString(name,"iptc",sizeof(name));
                 else
-                  if (iptc == MagickTrue)
-                    (void) CopyMagickString(name,"iptc",sizeof(name));
-                  else
-                    (void) FormatMagickString(name,sizeof(name),"gif:%.11s",
-                      header);
+                  (void) FormatMagickString(name,sizeof(name),"gif:%.11s",
+                    header);
                 (void) SetImageProfile(image,name,profile);
+                info=(unsigned char *) RelinquishMagickMemory(info);
                 profile=DestroyStringInfo(profile);
                 (void) LogMagickEvent(CoderEvent,GetMagickModule(),
                   "      profile name=%s",name);
@@ -1308,7 +1310,7 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
     if (image_info->ping != MagickFalse)
       status=PingGIFImage(image);
     else
-      status=DecodeImage(image,(Quantum) opacity);
+      status=DecodeImage(image,opacity);
     if ((image_info->ping == MagickFalse) && (status == MagickFalse))
       {
         global_colormap=(unsigned char *) RelinquishMagickMemory(
@@ -1319,7 +1321,8 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
     opacity=(-1);
-    status=SetImageProgress(image,LoadImageTag,image->scene-1,image->scene);
+    status=SetImageProgress(image,LoadImageTag,(MagickOffsetType) image->scene-
+      1,image->scene);
     if (status == MagickFalse)
       break;
   }
@@ -1540,7 +1543,7 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
   do
   {
     if (image->colorspace != RGBColorspace)
-      (void) SetImageColorspace(image,RGBColorspace);
+      (void) TransformImageColorspace(image,RGBColorspace);
     opacity=(-1);
     if (IsOpaqueImage(image,&image->exception) != MagickFalse)
       {
@@ -1599,6 +1602,8 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
             image->colormap[opacity].blue=image->transparent_color.blue;
           }
       }
+    if ((image->storage_class == DirectClass) || (image->colors > 256))
+      ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
     for (bits_per_pixel=1; bits_per_pixel < 8; bits_per_pixel++)
       if ((1UL << bits_per_pixel) >= image->colors)
         break;
