@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -108,21 +108,17 @@ static Image *ReadGRAYImage(const ImageInfo *image_info,
   MagickBooleanType
     status;
 
+  MagickOffsetType
+    scene;
+
   QuantumInfo
     *quantum_info;
 
   QuantumType
     quantum_type;
 
-  register const PixelPacket
-    *p;
-
   register long
-    i,
-    x;
-
-  register PixelPacket
-    *q;
+    i;
 
   ssize_t
     count;
@@ -162,11 +158,13 @@ static Image *ReadGRAYImage(const ImageInfo *image_info,
   /*
     Create virtual canvas to support cropping (i.e. image.gray[100x100+10+20]).
   */
-  canvas_image=CloneImage(image,image->extract_info.width,1,MagickTrue,
+  canvas_image=CloneImage(image,image->extract_info.width,1,MagickFalse,
     exception);
   (void) SetImageVirtualPixelMethod(canvas_image,BlackVirtualPixelMethod);
   quantum_type=GrayQuantum;
   quantum_info=AcquireQuantumInfo(image_info,canvas_image);
+  if (quantum_info == (QuantumInfo *) NULL)
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
   pixels=GetQuantumPixels(quantum_info);
   if (image_info->number_scenes != 0)
     while (image->scene < image_info->scene)
@@ -183,6 +181,9 @@ static Image *ReadGRAYImage(const ImageInfo *image_info,
           break;
       }
     }
+  scene=0;
+  count=0;
+  length=0;
   do
   {
     /*
@@ -191,23 +192,35 @@ static Image *ReadGRAYImage(const ImageInfo *image_info,
     if ((image_info->ping != MagickFalse) && (image_info->number_scenes != 0))
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
-    if ((SetImageExtent(image,0,0) == MagickFalse) ||
-        (SetImageExtent(canvas_image,0,0) == MagickFalse))
-      break;
-    length=GetQuantumExtent(canvas_image,quantum_info,quantum_type);
-    count=ReadBlob(image,length,pixels);
-    if (count != (ssize_t) length)
-      break;
+    if (scene == 0)
+      {
+        length=GetQuantumExtent(canvas_image,quantum_info,quantum_type);
+        count=ReadBlob(image,length,pixels);
+      }
     for (y=0; y < (long) image->extract_info.height; y++)
     {
+      register const PixelPacket
+        *restrict p;
+
+      register long
+        x;
+
+      register PixelPacket
+        *restrict q;
+
+      if (count != (ssize_t) length)
+        {
+          ThrowFileException(exception,CorruptImageError,
+            "UnexpectedEndOfFile",image->filename);
+          break;
+        }
       q=GetAuthenticPixels(canvas_image,0,0,canvas_image->columns,1,exception);
       if (q == (PixelPacket *) NULL)
         break;
-      length=ImportQuantumPixels(canvas_image,(ViewInfo *) NULL,quantum_info,
+      length=ImportQuantumPixels(canvas_image,(CacheView *) NULL,quantum_info,
         quantum_type,pixels,exception);
       if (SyncAuthenticPixels(canvas_image,exception) == MagickFalse)
         break;
-      count=ReadBlob(image,length,pixels);
       if (((y-image->extract_info.y) >= 0) && 
           ((y-image->extract_info.y) < (long) image->rows))
         {
@@ -234,6 +247,7 @@ static Image *ReadGRAYImage(const ImageInfo *image_info,
           if (status == MagickFalse)
             break;
         }
+      count=ReadBlob(image,length,pixels);
     }
     SetQuantumImageType(image,quantum_type);
     /*
@@ -259,9 +273,10 @@ static Image *ReadGRAYImage(const ImageInfo *image_info,
         if (status == MagickFalse)
           break;
       }
+    scene++;
   } while (count == (ssize_t) length);
   quantum_info=DestroyQuantumInfo(quantum_info);
-  InheritException(exception,&canvas_image->exception);
+  InheritException(&image->exception,&canvas_image->exception);
   canvas_image=DestroyImage(canvas_image);
   (void) CloseBlob(image);
   return(GetFirstImageInList(image));
@@ -375,9 +390,6 @@ static MagickBooleanType WriteGRAYImage(const ImageInfo *image_info,
   QuantumType
     quantum_type;
 
-  register const PixelPacket
-    *p;
-
   ssize_t
     count;
 
@@ -406,16 +418,21 @@ static MagickBooleanType WriteGRAYImage(const ImageInfo *image_info,
       Write grayscale pixels.
     */
     if (image->colorspace != RGBColorspace)
-      (void) SetImageColorspace(image,RGBColorspace);
+      (void) TransformImageColorspace(image,RGBColorspace);
     quantum_type=GrayQuantum;
     quantum_info=AcquireQuantumInfo(image_info,image);
+    if (quantum_info == (QuantumInfo *) NULL)
+      ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
     pixels=GetQuantumPixels(quantum_info);
     for (y=0; y < (long) image->rows; y++)
     {
+      register const PixelPacket
+        *restrict p;
+
       p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
       if (p == (const PixelPacket *) NULL)
         break;
-      length=ExportQuantumPixels(image,(const ViewInfo *) NULL,quantum_info,
+      length=ExportQuantumPixels(image,(const CacheView *) NULL,quantum_info,
         quantum_type,pixels,&image->exception);
       count=WriteBlob(image,length,pixels);
       if (count != (ssize_t) length)
