@@ -48,6 +48,7 @@
 #include "magick/blob-private.h"
 #include "magick/cache.h"
 #include "magick/color.h"
+#include "magick/colormap-private.h"
 #include "magick/color-private.h"
 #include "magick/colorspace.h"
 #include "magick/constitute.h"
@@ -271,6 +272,26 @@ static void InitializeSource(j_decompress_ptr cinfo)
   source->start_of_blob=TRUE;
 }
 
+static MagickBooleanType IsITUFaxImage(const Image *image)
+{
+  const StringInfo
+    *profile;
+
+  const unsigned char
+    *datum;
+
+  profile=GetImageProfile(image,"8bim");
+  if (profile == (const StringInfo *) NULL)
+    return(MagickFalse);
+  if (GetStringInfoLength(profile) < 5)
+    return(MagickFalse);
+  datum=GetStringInfoDatum(profile);
+  if ((datum[0] == 0x47) && (datum[1] == 0x33) && (datum[2] == 0x46) &&
+      (datum[3] == 0x41) && (datum[4] == 0x58))
+    return(MagickTrue);
+  return(MagickFalse);
+}
+
 static void JPEGErrorHandler(j_common_ptr jpeg_info)
 {
   ErrorManager
@@ -848,6 +869,9 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
   char
     value[MaxTextExtent];
 
+  const char
+    *option;
+
   ErrorManager
     error_manager;
 
@@ -920,13 +944,12 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
   if (setjmp(error_manager.error_recovery) != 0)
     {
       jpeg_destroy_decompress(&jpeg_info);
-      InheritException(exception,&image->exception);
       (void) CloseBlob(image);
       number_pixels=(MagickSizeType) image->columns*image->rows;
       if (number_pixels != 0)
         return(GetFirstImageInList(image));
-      image=DestroyImage(image);
-      return(image);
+      InheritException(exception,&image->exception);
+      return(DestroyImage(image));
     }
   jpeg_info.client_data=(void *) &error_manager;
   jpeg_create_decompress(&jpeg_info);
@@ -938,10 +961,18 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
     if ((i != 2) && (i != 13) && (i != 14))
       jpeg_set_marker_processor(&jpeg_info,(int) (JPEG_APP0+i),ReadProfile);
   i=jpeg_read_header(&jpeg_info,MagickTrue);
-  if ((image->colorspace == YCbCrColorspace) ||
-      (image->colorspace == Rec601YCbCrColorspace) ||
-      (image->colorspace == Rec709YCbCrColorspace))
+  if ((image_info->colorspace == YCbCrColorspace) ||
+      (image_info->colorspace == Rec601YCbCrColorspace) ||
+      (image_info->colorspace == Rec709YCbCrColorspace))
     jpeg_info.out_color_space=JCS_YCbCr;
+  if (IsITUFaxImage(image) != MagickFalse)
+    {
+      image->colorspace=LabColorspace;
+      jpeg_info.out_color_space=JCS_YCbCr;
+    }
+  else
+    if (jpeg_info.out_color_space == JCS_CMYK)
+      image->colorspace=CMYKColorspace;
   /*
     Set image resolution.
   */
@@ -958,31 +989,40 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
   if (units == 2)
     image->units=PixelsPerCentimeterResolution;
   number_pixels=(MagickSizeType) image->columns*image->rows;
-  if (image_info->size != (char *) NULL)
+  option=GetImageOption(image_info,"jpeg:size");
+  if (option != (const char *) NULL)
     {
       double
         scale_factor;
 
+      GeometryInfo
+        geometry_info;
+
+      MagickStatusType
+        flags;
+
       /*
-        Let the JPEG library subsample for us.
+        Scale the image.
       */
+      flags=ParseGeometry(option,&geometry_info);
+      if ((flags & SigmaValue) == 0)
+        geometry_info.sigma=geometry_info.rho;
       jpeg_calc_output_dimensions(&jpeg_info);
       image->magick_columns=jpeg_info.output_width;
       image->magick_rows=jpeg_info.output_height;
-      scale_factor=(double) jpeg_info.output_width/image->columns;
-      if (scale_factor > ((double) jpeg_info.output_height/image->rows))
-        scale_factor=(double) jpeg_info.output_height/image->rows;
+      scale_factor=1.0;
+      if (geometry_info.rho != 0.0)
+        scale_factor=jpeg_info.output_width/geometry_info.rho;
+      if ((geometry_info.sigma != 0.0) &&
+          (scale_factor > (jpeg_info.output_height/geometry_info.sigma)))
+        scale_factor=jpeg_info.output_height/geometry_info.sigma;
+      jpeg_info.scale_num=1U;
       jpeg_info.scale_denom=(unsigned int) scale_factor;
       jpeg_calc_output_dimensions(&jpeg_info);
       if (image->debug != MagickFalse)
         (void) LogMagickEvent(CoderEvent,GetMagickModule(),"Scale factor: %ld",
           (long) scale_factor);
     }
-  if (image_info->subrange != 0)
-    {
-      jpeg_info.scale_denom=(unsigned int) image_info->subrange;
-      jpeg_calc_output_dimensions(&jpeg_info);
-    }
   precision=(unsigned long) jpeg_info.data_precision;
 #if (JPEG_LIB_VERSION >= 61) && defined(D_PROGRESSIVE_SUPPORTED)
 #if defined(D_LOSSLESS_SUPPORTED)
@@ -1013,6 +1053,20 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
       jpeg_info.quantize_colors=MagickTrue;
       jpeg_info.desired_number_of_colors=(int) image_info->colors;
     }
+  option=GetImageOption(image_info,"jpeg:block-smoothing");
+  if (option != (const char *) NULL)
+    {
+      jpeg_info.do_block_smoothing=MagickFalse;
+      if (IsMagickTrue(option) != MagickFalse)
+        jpeg_info.do_block_smoothing=MagickTrue;
+    }
+  option=GetImageOption(image_info,"jpeg:fancy-upsampling");
+  if (option != (const char *) NULL)
+    {
+      jpeg_info.do_fancy_upsampling=MagickFalse;
+      if (IsMagickTrue(option) != MagickFalse)
+        jpeg_info.do_fancy_upsampling=MagickTrue;
+    }
   (void) jpeg_start_decompress(&jpeg_info);
   image->columns=jpeg_info.output_width;
   image->rows=jpeg_info.output_height;
@@ -1067,15 +1121,14 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
   */
   if (setjmp(error_manager.error_recovery) != 0)
     {
-      jpeg_pixels=(unsigned char *) RelinquishMagickMemory(jpeg_pixels);
+      if (jpeg_pixels != (unsigned char *) NULL)
+        jpeg_pixels=(unsigned char *) RelinquishMagickMemory(jpeg_pixels);
       jpeg_destroy_decompress(&jpeg_info);
-      InheritException(exception,&image->exception);
       (void) CloseBlob(image);
       number_pixels=(MagickSizeType) image->columns*image->rows;
       if (number_pixels != 0)
         return(GetFirstImageInList(image));
-      image=DestroyImage(image);
-      return(image);
+      return(DestroyImage(image));
     }
   if (jpeg_info.quantize_colors != MagickFalse)
     {
@@ -1101,17 +1154,17 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
   for (y=0; y < (long) image->rows; y++)
   {
     register IndexPacket
-      *__restrict indexes;
+      *restrict indexes;
 
     register long
       x;
 
     register PixelPacket
-      *__restrict q;
+      *restrict q;
 
     if (jpeg_read_scanlines(&jpeg_info,scanline,1) != 1)
       {
-        (void) ThrowMagickException(&image->exception,GetMagickModule(),
+        (void) ThrowMagickException(exception,GetMagickModule(),
           CorruptImageWarning,"SkipToSyncByte","`%s'",image->filename);
         continue;
       }
@@ -1210,7 +1263,6 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
   jpeg_destroy_decompress(&jpeg_info);
   jpeg_pixels=(unsigned char *) RelinquishMagickMemory(jpeg_pixels);
   (void) CloseBlob(image);
-  InheritException(exception,&image->exception);
   return(GetFirstImageInList(image));
 }
 #endif
@@ -1247,7 +1299,7 @@ ModuleExport unsigned long RegisterJPEGImage(void)
     *entry;
 
   static const char
-    *description = "Joint Photographic Experts Group JFIF format";
+    description[] = "Joint Photographic Experts Group JFIF format";
 
   *version='\0';
 #if defined(JPEG_LIB_VERSION)
@@ -1503,9 +1555,9 @@ static void WriteProfile(j_compress_ptr jpeg_info,Image *image)
         /*
           Add namespace to XMP profile.
         */
-        xmp_profile=StringToStringInfo("http://ns.adobe.com/xap/1.0/ ");
+        xmp_profile=StringToStringInfo("http://ns.adobe.com/xap/1.0/");
         ConcatenateStringInfo(xmp_profile,profile);
-        GetStringInfoDatum(xmp_profile)[29]='\0';
+        GetStringInfoDatum(xmp_profile)[28]='\0';
         for (i=0; i < (long) GetStringInfoLength(xmp_profile); i+=65533L)
         {
           length=MagickMin(GetStringInfoLength(xmp_profile)-i,65533L);
@@ -1645,8 +1697,6 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
   if (setjmp(error_manager.error_recovery) != 0)
     {
       jpeg_destroy_compress(&jpeg_info);
-      if (jpeg_pixels != (unsigned char *) NULL)
-        jpeg_pixels=(unsigned char *) RelinquishMagickMemory(jpeg_pixels);
       (void) CloseBlob(image);
       return(MagickFalse);
     }
@@ -1790,6 +1840,49 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
     (void) LogMagickEvent(CoderEvent,GetMagickModule(),
       "Interlace: nonprogressive");
 #endif
+  option=GetImageOption(image_info,"jpeg:extent");
+  if (option != (const char *) NULL)
+    {
+      Image
+        *jpeg_image;
+
+      ImageInfo
+        *jpeg_info;
+
+      jpeg_info=CloneImageInfo(image_info);
+      jpeg_image=CloneImage(image,0,0,MagickTrue,&image->exception);
+      if (jpeg_image != (Image *) NULL)
+        {
+          MagickSizeType
+            extent;
+
+          size_t
+            max,
+            min;
+
+          /*
+            Search for compression quality that does not exceed image extent.
+          */
+          jpeg_info->quality=0;
+          extent=(MagickSizeType) StringToDouble(option,100.0);
+          (void) DeleteImageOption(jpeg_info,"jpeg:extent");
+          (void) AcquireUniqueFilename(jpeg_image->filename);
+          min=0;
+          for (max=100; (max-min) > 1; )
+          {
+            jpeg_image->quality=min+(max-min)/2;
+            status=WriteJPEGImage(jpeg_info,jpeg_image);
+            if (GetBlobSize(jpeg_image) < extent)
+              min=jpeg_image->quality+1;
+            else
+              max=jpeg_image->quality-1;
+          }
+          (void) RelinquishUniqueFileResource(jpeg_image->filename);
+          image->quality=max;
+          jpeg_image=DestroyImage(jpeg_image);
+        }
+      jpeg_info=DestroyImageInfo(jpeg_info);
+    }
   if ((image_info->compression != LosslessJPEGCompression) &&
       (image->quality <= 100))
     {
@@ -1967,7 +2060,7 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
         case RGBColorspace:
         {
           (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-            " Image colorspace is RGB");
+            "Image colorspace is RGB");
           (void) LogMagickEvent(CoderEvent,GetMagickModule(),
             "Sampling factors: %dx%d,%dx%d,%dx%d",
             jpeg_info.comp_info[0].h_samp_factor,
@@ -2029,6 +2122,14 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
     jpeg_info.input_components*sizeof(*jpeg_pixels));
   if (jpeg_pixels == (JSAMPLE *) NULL)
     ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
+  if (setjmp(error_manager.error_recovery) != 0)
+    {
+      jpeg_destroy_compress(&jpeg_info);
+      if (jpeg_pixels != (unsigned char *) NULL)
+        jpeg_pixels=(unsigned char *) RelinquishMagickMemory(jpeg_pixels);
+      (void) CloseBlob(image);
+      return(MagickFalse);
+    }
   scanline[0]=(JSAMPROW) jpeg_pixels;
   if (jpeg_info.data_precision > 8)
     {
