@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -40,14 +40,17 @@
   Include declarations.
 */
 #include "magick/studio.h"
+#include "magick/attribute.h"
 #include "magick/blob.h"
 #include "magick/blob-private.h"
 #include "magick/cache.h"
 #include "magick/color.h"
+#include "magick/colormap-private.h"
 #include "magick/color-private.h"
 #include "magick/colorspace.h"
 #include "magick/exception.h"
 #include "magick/exception-private.h"
+#include "magick/histogram.h"
 #include "magick/image.h"
 #include "magick/image-private.h"
 #include "magick/list.h"
@@ -207,11 +210,6 @@ static Image *ReadMAPImage(const ImageInfo *image_info,ExceptionInfo *exception)
   /*
     Read image pixels.
   */
-  if (SetImageExtent(image,0,0) == MagickFalse)
-    {
-      InheritException(exception,&image->exception);
-      return(DestroyImageList(image));
-    }
   packet_size=(size_t) (depth/8);
   for (y=0; y < (long) image->rows; y++)
   {
@@ -381,7 +379,7 @@ static MagickBooleanType WriteMAPImage(const ImageInfo *image_info,Image *image)
   if (status == MagickFalse)
     return(status);
   if (image->colorspace != RGBColorspace)
-    (void) SetImageColorspace(image,RGBColorspace);
+    (void) TransformImageColorspace(image,RGBColorspace);
   /*
     Allocate colormap.
   */
