@@ -40,10 +40,12 @@
   Include declarations.
 */
 #include "magick/studio.h"
+#include "magick/attribute.h"
 #include "magick/blob.h"
 #include "magick/blob-private.h"
 #include "magick/cache.h"
 #include "magick/color.h"
+#include "magick/colormap-private.h"
 #include "magick/color-private.h"
 #include "magick/colorspace.h"
 #include "magick/constitute.h"
@@ -109,7 +111,7 @@ static MagickBooleanType IsMIFF(const unsigned char *magick,const size_t length)
 {
   if (length < 14)
     return(MagickFalse);
-  if (LocaleNCompare((char *) magick,"id=ImageMagick",14) == 0)
+  if (LocaleNCompare((const char *) magick,"id=ImageMagick",14) == 0)
     return(MagickTrue);
   return(MagickFalse);
 }
@@ -823,7 +825,8 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
                     geometry=DestroyString(geometry);
                     break;
                   }
-                if (LocaleNCompare(keyword,"profile-",8) == 0)
+                if ((LocaleNCompare(keyword,"profile:",8) == 0) ||
+                    (LocaleNCompare(keyword,"profile-",8) == 0))
                   {
                     StringInfo
                       *profile;
@@ -848,7 +851,8 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
                     image->quality=(unsigned long) atol(options);
                     break;
                   }
-                if (LocaleCompare(keyword,"quantum-format") == 0)
+                if ((LocaleCompare(keyword,"quantum-format") == 0) ||
+                    (LocaleCompare(keyword,"quantum:format") == 0))
                   {
                     long
                       format;
@@ -1218,13 +1222,13 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
     for (y=0; y < (long) image->rows; y++)
     {
       register IndexPacket
-        *__restrict indexes;
+        *restrict indexes;
 
       register long
         x;
 
       register PixelPacket
-        *__restrict q;
+        *restrict q;
 
       q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
       if (q == (PixelPacket *) NULL)
@@ -1975,7 +1979,7 @@ static MagickBooleanType WriteMIFFImage(const ImageInfo *image_info,
           profile=GetImageProfile(image,name);
           if (profile != (StringInfo *) NULL)
             {
-              (void) FormatMagickString(buffer,MaxTextExtent,"profile-%s=%lu\n",
+              (void) FormatMagickString(buffer,MaxTextExtent,"profile:%s=%lu\n",
                 name,(unsigned long) GetStringInfoLength(profile));
               (void) WriteBlobString(image,buffer);
             }
@@ -1989,7 +1993,7 @@ static MagickBooleanType WriteMIFFImage(const ImageInfo *image_info,
         (void) WriteBlobString(image,buffer);
       }
     if (quantum_info->format == FloatingPointQuantumFormat)
-      (void) WriteBlobString(image,"quantum-format=floating-point\n");
+      (void) SetImageProperty(image,"quantum:format","floating-point");
     ResetImagePropertyIterator(image);
     property=GetNextImageProperty(image);
     while (property != (const char *) NULL)
@@ -2004,7 +2008,7 @@ static MagickBooleanType WriteMIFFImage(const ImageInfo *image_info,
               break;
           if (i <= (long) strlen(value))
             (void) WriteBlobByte(image,'{');
-          (void) WriteBlob(image,strlen(value),(unsigned char *) value);
+          (void) WriteBlob(image,strlen(value),(const unsigned char *) value);
           if (i <= (long) strlen(value))
             (void) WriteBlobByte(image,'}');
         }
@@ -2124,10 +2128,10 @@ static MagickBooleanType WriteMIFFImage(const ImageInfo *image_info,
     for (y=0; y < (long) image->rows; y++)
     {
       register const IndexPacket
-        *__restrict indexes;
+        *restrict indexes;
 
       register const PixelPacket
-        *__restrict p;
+        *restrict p;
 
       register long
         x;
@@ -2156,8 +2160,8 @@ static MagickBooleanType WriteMIFFImage(const ImageInfo *image_info,
             }
           zip_info.next_in=pixels;
           zip_info.avail_in=(uInt) (packet_size*image->columns);
-          (void) ExportQuantumPixels(image,(const CacheView *) NULL,quantum_info,
-            quantum_type,pixels,&image->exception);
+          (void) ExportQuantumPixels(image,(const CacheView *) NULL,
+            quantum_info,quantum_type,pixels,&image->exception);
           do
           {
             zip_info.next_out=compress_pixels;
@@ -2211,8 +2215,8 @@ static MagickBooleanType WriteMIFFImage(const ImageInfo *image_info,
             }
           bzip_info.next_in=(char *) pixels;
           bzip_info.avail_in=(unsigned int) (packet_size*image->columns);
-          (void) ExportQuantumPixels(image,(const CacheView *) NULL,quantum_info,
-            quantum_type,pixels,&image->exception);
+          (void) ExportQuantumPixels(image,(const CacheView *) NULL,
+            quantum_info,quantum_type,pixels,&image->exception);
           do
           {
             bzip_info.next_out=(char *) compress_pixels;
