@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -295,8 +295,7 @@ static MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
     r=pcd_table[plane];
     for (i=0; ((i < (long) length) && ((sum & r->mask) != r->sequence)); i++)
       r++;
-    if ((row > image->rows) || (r == (PCDTable *) NULL) ||
-        ((size_t) (q-luma) > (image->columns*image->rows)))
+    if ((row > image->rows) || (r == (PCDTable *) NULL))
       {
         (void) ThrowMagickException(&image->exception,GetMagickModule(),
           CorruptImageWarning,"SkipToSyncByte","`%s'",image->filename);
@@ -344,18 +343,16 @@ static MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
 %
 %  A description of each parameter follows:
 %
-%    o magick: This string is generally the first few bytes of an image file
-%      or blob.
+%    o magick: compare image format pattern against these bytes.
 %
 %    o length: Specifies the length of the magick string.
 %
-%
 */
 static MagickBooleanType IsPCD(const unsigned char *magick,const size_t length)
 {
   if (length < 2052)
     return(MagickFalse);
-  if (LocaleNCompare((char *) magick+2048,"PCD_",4) == 0)
+  if (LocaleNCompare((const char *) magick+2048,"PCD_",4) == 0)
     return(MagickTrue);
   return(MagickFalse);
 }
@@ -600,13 +597,8 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
     image->rows<<=1;
   }
   /*
-    Allocate lumand chroma memory.
+    Allocate luma and chroma memory.
   */
-  if (SetImageExtent(image,0,0) == MagickFalse)
-    {
-      InheritException(exception,&image->exception);
-      return(DestroyImageList(image));
-    }
   number_pixels=(MagickSizeType) image->columns*image->rows;
   if (number_pixels != (size_t) number_pixels)
     ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
@@ -1022,7 +1014,7 @@ static MagickBooleanType WritePCDTile(Image *image,const char *page_geometry,
     }
   (void) TransformImage(&tile_image,(char *) NULL,tile_geometry);
   if (image->colorspace != RGBColorspace)
-    (void) SetImageColorspace(tile_image,YCCColorspace);
+    (void) TransformImageColorspace(tile_image,YCCColorspace);
   downsample_image=ResizeImage(tile_image,tile_image->columns/2,
     tile_image->rows/2,TriangleFilter,1.0,&image->exception);
   if (downsample_image == (Image *) NULL)
@@ -1108,7 +1100,7 @@ static MagickBooleanType WritePCDImage(const ImageInfo *image_info,Image *image)
   if (status == MagickFalse)
     return(status);
   if (image->colorspace != RGBColorspace)
-    (void) SetImageColorspace(pcd_image,RGBColorspace);
+    (void) TransformImageColorspace(pcd_image,RGBColorspace);
   /*
     Write PCD image header.
   */
@@ -1132,7 +1124,7 @@ static MagickBooleanType WritePCDImage(const ImageInfo *image_info,Image *image)
     (void) WriteBlobByte(pcd_image,0x01);
   for (i=0; i < 1944; i++)
     (void) WriteBlobByte(pcd_image,'\0');
-  (void) WriteBlob(pcd_image,7,(unsigned char *) "PCD_IPI");
+  (void) WriteBlob(pcd_image,7,(const unsigned char *) "PCD_IPI");
   (void) WriteBlobByte(pcd_image,0x06);
   for (i=0; i < 1530; i++)
     (void) WriteBlobByte(pcd_image,'\0');
