@@ -16,7 +16,7 @@
 %                                 June 2000                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -42,6 +42,7 @@
 #include "magick/blob.h"
 #include "magick/blob-private.h"
 #include "magick/color-private.h"
+#include "magick/colormap-private.h"
 #include "magick/constitute.h"
 #include "magick/exception.h"
 #include "magick/exception-private.h"
@@ -222,12 +223,10 @@ const RGB_Record WPG1_Palette[256]={
 %
 %    o status:  Method IsWPG returns True if the image format type is WPG.
 %
-%    o magick: This string is generally the first few bytes of an image file
-%      or blob.
+%    o magick: compare image format pattern against these bytes.
 %
 %    o length: Specifies the length of the magick string.
 %
-%
 */
 static unsigned int IsWPG(const unsigned char *magick,const size_t length)
 {
@@ -1129,7 +1128,7 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
                     {
                       rotated_image = FlopImage(image, exception);
                       rotated_image->blob = image->blob;
-                      image->blob = NULL;
+                      DuplicateBlob(rotated_image,image);
                       (void) RemoveLastImageFromList(&image);
                       AppendImageToList(&image,rotated_image);
                     }
@@ -1138,7 +1137,7 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
                     {
                       rotated_image = FlipImage(image, exception);
                       rotated_image->blob = image->blob;
-                      image->blob = NULL;
+                      DuplicateBlob(rotated_image,image);
                       (void) RemoveLastImageFromList(&image);
                       AppendImageToList(&image,rotated_image);    
                     }
@@ -1148,7 +1147,7 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
                     {
                       rotated_image = RotateImage(image, (BitmapHeader2.RotAngle & 0x0FFF), exception);
                       rotated_image->blob = image->blob;
-                      image->blob = NULL;
+                      DuplicateBlob(rotated_image,image);
                       (void) RemoveLastImageFromList(&image);
                       AppendImageToList(&image,rotated_image);    
                     }                
@@ -1302,7 +1301,7 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
     {    /*?? RotAngle=360-RotAngle;*/
       rotated_image = FlopImage(image, exception);
       rotated_image->blob = image->blob;
-      image->blob = NULL;
+      DuplicateBlob(rotated_image,image);
       (void) RemoveLastImageFromList(&image);
       AppendImageToList(&image,rotated_image);
                   /* Try to change CTM according to Flip - I am not sure, must be checked.      
@@ -1315,7 +1314,7 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
     {    /*?? RotAngle=360-RotAngle;*/
       rotated_image = FlipImage(image, exception);
       rotated_image->blob = image->blob;
-      image->blob = NULL;
+      DuplicateBlob(rotated_image,image);
       (void) RemoveLastImageFromList(&image);
       AppendImageToList(&image,rotated_image);
                   /* Try to change CTM according to Flip - I am not sure, must be checked.
