@@ -44,6 +44,7 @@
 */
 #include "magick/studio.h"
 #include "magick/annotate.h"
+#include "magick/attribute.h"
 #include "magick/cache-view.h"
 #include "magick/client.h"
 #include "magick/color.h"
@@ -233,7 +234,8 @@ MagickExport MagickBooleanType AnnotateImage(Image *image,
     annotate_info->affine.ty=geometry_info.psi-image->page.y;
     (void) CloneString(&annotate->text,textlist[i]);
     (void) GetTypeMetrics(image,annotate,&metrics);
-    height=(unsigned long) (metrics.ascent-metrics.descent+0.5);
+    height=(long) (metrics.ascent-metrics.descent+
+      draw_info->interline_spacing+0.5);
     switch (annotate->gravity)
     {
       case UndefinedGravity:
@@ -393,7 +395,7 @@ MagickExport MagickBooleanType AnnotateImage(Image *image,
         undercolor_info->affine.tx=offset.x-draw_info->affine.ry*metrics.ascent;
         undercolor_info->affine.ty=offset.y-draw_info->affine.sy*metrics.ascent;
         (void) FormatMagickString(primitive,MaxTextExtent,
-          "rectangle 0,0 %g,%ld",metrics.origin.x,height);
+          "rectangle 0,0 %g,%lu",metrics.origin.x,height);
         (void) CloneString(&undercolor_info->primitive,primitive);
         (void) DrawImage(image,undercolor_info);
         (void) DestroyDrawInfo(undercolor_info);
@@ -612,9 +614,6 @@ MagickExport MagickBooleanType GetMultilineTypeMetrics(Image *image,
   TypeMetric
     extent;
 
-  unsigned long
-    number_lines;
-
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
@@ -648,9 +647,8 @@ MagickExport MagickBooleanType GetMultilineTypeMetrics(Image *image,
     if (extent.width > metrics->width)
       *metrics=extent;
   }
-  number_lines=(unsigned long) i;
-  metrics->height=(double) number_lines*(long) (metrics->ascent-
-    metrics->descent+0.5);
+  metrics->height=(double) (i*(unsigned long) (metrics->ascent-
+    metrics->descent+0.5)+(i-1)*draw_info->interline_spacing);
   /*
     Relinquish resources.
   */
@@ -1240,24 +1238,21 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
     point.y=offset->y-bitmap->top;
     if (draw_info->render != MagickFalse)
       {
+        CacheView
+          *image_view;
+
         ExceptionInfo
           *exception;
 
         MagickBooleanType
           status;
 
-        CacheView
-          *image_view;
-
         /*
           Rasterize the glyph.
         */
         status=MagickTrue;
         exception=(&image->exception);
         image_view=AcquireCacheView(image);
-#if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(dynamic,4) shared(status)
-#endif
         for (y=0; y < (long) bitmap->bitmap.rows; y++)
         {
           long
@@ -1278,7 +1273,7 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
             x;
 
           register PixelPacket
-            *__restrict q;
+            *restrict q;
 
           register unsigned char
             *p;
@@ -1689,7 +1684,7 @@ static MagickBooleanType RenderPostscript(Image *image,
           x;
 
         register PixelPacket
-          *__restrict q;
+          *restrict q;
 
         q=GetCacheViewAuthenticPixels(annotate_view,0,y,annotate_image->columns,
           1,exception);
@@ -1789,6 +1784,9 @@ static MagickBooleanType RenderX11(Image *image,const DrawInfo *draw_info,
 
   if (display == (Display *) NULL)
     {
+      const char
+        *client_name;
+
       ImageInfo
         *image_info;
 
@@ -1807,9 +1805,9 @@ static MagickBooleanType RenderX11(Image *image,const DrawInfo *draw_info,
       */
       (void) XSetErrorHandler(XError);
       image_info=AcquireImageInfo();
-      resource_database=XGetResourceDatabase(display,GetClientName());
-      XGetResourceInfo(image_info,resource_database,GetClientName(),
-        &resource_info);
+      client_name=GetClientName();
+      resource_database=XGetResourceDatabase(display,client_name);
+      XGetResourceInfo(image_info,resource_database,client_name,&resource_info);
       resource_info.close_server=MagickFalse;
       resource_info.colormap=PrivateColormap;
       resource_info.font=AcquireString(draw_info->font);
@@ -1925,7 +1923,8 @@ static MagickBooleanType RenderX11(Image *image,const DrawInfo *draw_info,
     }
   (void) FormatMagickString(annotate_info.geometry,MaxTextExtent,
     "%lux%lu+%ld+%ld",width,height,(long) (offset->x+0.5),
-    (long) (offset->y-metrics->ascent-metrics->descent+0.5));
+    (long) (offset->y-metrics->ascent-metrics->descent+
+    draw_info->interline_spacing+0.5));
   pixel.pen_color.red=ScaleQuantumToShort(draw_info->fill.red);
   pixel.pen_color.green=ScaleQuantumToShort(draw_info->fill.green);
   pixel.pen_color.blue=ScaleQuantumToShort(draw_info->fill.blue);
