@@ -210,21 +210,21 @@ MagickExport Image *CompareImageChannels(Image *image,
       reconstruct_pixel;
 
     register const IndexPacket
-      *__restrict indexes,
-      *__restrict reconstruct_indexes;
+      *restrict indexes,
+      *restrict reconstruct_indexes;
 
     register const PixelPacket
-      *__restrict p,
-      *__restrict q;
+      *restrict p,
+      *restrict q;
 
     register IndexPacket
-      *__restrict highlight_indexes;
+      *restrict highlight_indexes;
 
     register long
       x;
 
     register PixelPacket
-      *__restrict r;
+      *restrict r;
 
     if (status == MagickFalse)
       continue;
@@ -382,12 +382,12 @@ static MagickBooleanType GetAbsoluteError(const Image *image,
       reconstruct_pixel;
 
     register const IndexPacket
-      *__restrict indexes,
-      *__restrict reconstruct_indexes;
+      *restrict indexes,
+      *restrict reconstruct_indexes;
 
     register const PixelPacket
-      *__restrict p,
-      *__restrict q;
+      *restrict p,
+      *restrict q;
 
     register long
       i,
@@ -494,12 +494,12 @@ static MagickBooleanType GetMeanAbsoluteError(const Image *image,
       channel_distortion[AllChannels+1];
 
     register const IndexPacket
-      *__restrict indexes,
-      *__restrict reconstruct_indexes;
+      *restrict indexes,
+      *restrict reconstruct_indexes;
 
     register const PixelPacket
-      *__restrict p,
-      *__restrict q;
+      *restrict p,
+      *restrict q;
 
     register long
       i,
@@ -605,12 +605,12 @@ static MagickBooleanType GetMeanErrorPerPixel(Image *image,
   for (y=0; y < (long) image->rows; y++)
   {
     register const IndexPacket
-      *__restrict indexes,
-      *__restrict reconstruct_indexes;
+      *restrict indexes,
+      *restrict reconstruct_indexes;
 
     register const PixelPacket
-      *__restrict p,
-      *__restrict q;
+      *restrict p,
+      *restrict q;
 
     register long
       x;
@@ -731,12 +731,12 @@ static MagickBooleanType GetMeanSquaredError(const Image *image,
       channel_distortion[AllChannels+1];
 
     register const IndexPacket
-      *__restrict indexes,
-      *__restrict reconstruct_indexes;
+      *restrict indexes,
+      *restrict reconstruct_indexes;
 
     register const PixelPacket
-      *__restrict p,
-      *__restrict q;
+      *restrict p,
+      *restrict q;
 
     register long
       i,
@@ -837,12 +837,12 @@ static MagickBooleanType GetPeakAbsoluteError(const Image *image,
       channel_distortion[AllChannels+1];
 
     register const IndexPacket
-      *__restrict indexes,
-      *__restrict reconstruct_indexes;
+      *restrict indexes,
+      *restrict reconstruct_indexes;
 
     register const PixelPacket
-      *__restrict p,
-      *__restrict q;
+      *restrict p,
+      *restrict q;
 
     register long
       i,
@@ -1273,12 +1273,12 @@ MagickExport MagickBooleanType IsImagesEqual(Image *image,
   for (y=0; y < (long) image->rows; y++)
   {
     register const IndexPacket
-      *__restrict indexes,
-      *__restrict reconstruct_indexes;
+      *restrict indexes,
+      *restrict reconstruct_indexes;
 
     register const PixelPacket
-      *__restrict p,
-      *__restrict q;
+      *restrict p,
+      *restrict q;
 
     register long
       x;
@@ -1394,62 +1394,84 @@ static double GetSimilarityMetric(const Image *image,const Image *reference,
     *image_view,
     *reference_view;
 
+  MagickBooleanType
+    status;
+
   /*
     Compute the similarity in pixels between two images.
   */
+  status=MagickTrue;
   similarity=0.0;
   image_view=AcquireCacheView(image);
   reference_view=AcquireCacheView(reference);
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for schedule(dynamic,4) shared(status)
+#endif
   for (y=0; y < (long) reference->rows; y++)
   {
     register const IndexPacket
-      *__restrict indexes,
-      *__restrict reference_indexes;
+      *restrict indexes,
+      *restrict reference_indexes;
 
     register const PixelPacket
-      *__restrict p,
-      *__restrict q;
+      *restrict p,
+      *restrict q;
 
     register long
       x;
 
+    if (status == MagickFalse)
+      continue;
     p=GetCacheViewVirtualPixels(image_view,x_offset,y_offset+y,
       reference->columns,1,exception);
     q=GetCacheViewVirtualPixels(reference_view,0,y,reference->columns,1,
       exception);
     if ((p == (const PixelPacket *) NULL) || (q == (const PixelPacket *) NULL))
-      continue;
+      {
+        status=MagickFalse;
+        continue;
+      }
     indexes=GetCacheViewVirtualIndexQueue(image_view);
     reference_indexes=GetCacheViewVirtualIndexQueue(reference_view);
     for (x=0; x < (long) reference->columns; x++)
     {
+      double
+        thread_similarity;
+
       MagickRealType
         distance;
 
+      thread_similarity=0.0;
       distance=QuantumScale*(p->red-(MagickRealType) q->red);
-      similarity+=distance*distance;
+      thread_similarity+=distance*distance;
       distance=QuantumScale*(p->green-(MagickRealType) q->green);
-      similarity+=distance*distance;
+      thread_similarity+=distance*distance;
       distance=QuantumScale*(p->blue-(MagickRealType) q->blue);
-      similarity+=distance*distance;
+      thread_similarity+=distance*distance;
       if ((image->matte != MagickFalse) && (reference->matte != MagickFalse))
         {
           distance=QuantumScale*(p->opacity-(MagickRealType) q->opacity);
-          similarity+=distance*distance;
+          thread_similarity+=distance*distance;
         }
       if ((image->colorspace == CMYKColorspace) &&
           (reference->colorspace == CMYKColorspace))
         {
           distance=QuantumScale*(indexes[x]-(MagickRealType)
             reference_indexes[x]);
-          similarity+=distance*distance;
+          thread_similarity+=distance*distance;
         }
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp critical (MagickCore_GetSimilarityMetric)
+#endif
+      similarity+=thread_similarity;
       p++;
       q++;
     }
   }
   reference_view=DestroyCacheView(reference_view);
   image_view=DestroyCacheView(image_view);
+  if (status == MagickFalse)
+    return(0.0);
   similarity/=((double) reference->columns*reference->rows);
   similarity/=(double) GetNumberChannels(reference,AllChannels);
   return(sqrt(similarity));
@@ -1484,7 +1506,7 @@ MagickExport Image *SimilarityImage(Image *image,const Image *reference,
   *similarity_metric=1.0;
   if ((reference->columns > image->columns) ||
       (reference->rows > image->rows))
-    return((Image *) NULL);
+    ThrowImageException(ImageError,"ImageSizeDiffers");
   similarity_image=CloneImage(image,image->columns-reference->columns+1,
     image->rows-reference->rows+1,MagickTrue,exception);
   if (similarity_image == (Image *) NULL)
@@ -1513,7 +1535,7 @@ MagickExport Image *SimilarityImage(Image *image,const Image *reference,
       x;
 
     register PixelPacket
-      *__restrict q;
+      *restrict q;
 
     if (status == MagickFalse)
       continue;
