@@ -54,6 +54,7 @@
 #include "magick/monitor-private.h"
 #include "magick/pixel-private.h"
 #include "magick/quantum.h"
+#include "magick/transform.h"
 
 /*
   Define declarations.
@@ -190,6 +191,7 @@ MagickExport Image *FrameImage(const Image *image,const FrameInfo *frame_info,
     width;
 
   CacheView
+    *image_view,
     *frame_view;
 
   /*
@@ -280,19 +282,20 @@ MagickExport Image *FrameImage(const Image *image,const FrameInfo *frame_info,
     }
   status=MagickTrue;
   progress=0;
+  image_view=AcquireCacheView(image);
   frame_view=AcquireCacheView(frame_image);
   height=(unsigned long) (frame_info->outer_bevel+(frame_info->y-bevel_width)+
     frame_info->inner_bevel);
   if (height != 0)
     {
       register IndexPacket
-        *__restrict frame_indexes;
+        *restrict frame_indexes;
 
       register long
         x;
 
       register PixelPacket
-        *__restrict q;
+        *restrict q;
 
       /*
         Draw top of ornamental border.
@@ -397,18 +400,18 @@ MagickExport Image *FrameImage(const Image *image,const FrameInfo *frame_info,
     Draw sides of ornamental border.
   */
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(dynamic,8) shared(progress,status)
+  #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
 #endif
   for (y=0; y < (long) image->rows; y++)
   {
     register IndexPacket
-      *__restrict frame_indexes;
+      *restrict frame_indexes;
 
     register long
       x;
 
     register PixelPacket
-      *__restrict q;
+      *restrict q;
 
     /*
       Initialize scanline with matte color.
@@ -444,12 +447,39 @@ MagickExport Image *FrameImage(const Image *image,const FrameInfo *frame_info,
     /*
       Set frame interior to interior color.
     */
-    for (x=0; x < (long) image->columns; x++)
-    {
-      SetPixelPacket(frame_image,&interior,q,frame_indexes);
-      q++;
-      frame_indexes++;
-    }
+    if ((image->compose != CopyCompositeOp) &&
+        ((image->compose != OverCompositeOp) || (image->matte != MagickFalse)))
+      for (x=0; x < (long) image->columns; x++)
+      {
+        SetPixelPacket(frame_image,&interior,q,frame_indexes);
+        q++;
+        frame_indexes++;
+      }
+    else
+      {
+        register const IndexPacket
+          *indexes;
+
+        register const PixelPacket
+          *p;
+
+        p=GetCacheViewVirtualPixels(image_view,0,y,image->columns,1,exception);
+        if (p == (const PixelPacket *) NULL)
+          {
+            status=MagickFalse;
+            continue;
+          }
+        indexes=GetCacheViewVirtualIndexQueue(image_view);
+        (void) CopyMagickMemory(q,p,image->columns*sizeof(*p));
+        if ((image->colorspace == CMYKColorspace) &&
+            (frame_image->colorspace == CMYKColorspace))
+          {
+            (void) CopyMagickMemory(frame_indexes,indexes,image->columns*
+              sizeof(*indexes));
+            frame_indexes+=image->columns;
+          }
+        q+=image->columns;
+      }
     for (x=0; x < (long) frame_info->inner_bevel; x++)
     {
       SetPixelPacket(frame_image,&highlight,q,frame_indexes);
@@ -489,13 +519,13 @@ MagickExport Image *FrameImage(const Image *image,const FrameInfo *frame_info,
   if (height != 0)
     {
       register IndexPacket
-        *__restrict frame_indexes;
+        *restrict frame_indexes;
 
       register long
         x;
 
       register PixelPacket
-        *__restrict q;
+        *restrict q;
 
       /*
         Draw bottom of ornamental border.
@@ -596,11 +626,16 @@ MagickExport Image *FrameImage(const Image *image,const FrameInfo *frame_info,
         }
     }
   frame_view=DestroyCacheView(frame_view);
-  x=(long) (frame_info->outer_bevel+(frame_info->x-bevel_width)+
-    frame_info->inner_bevel);
-  y=(long) (frame_info->outer_bevel+(frame_info->y-bevel_width)+
-    frame_info->inner_bevel);
-  (void) CompositeImage(frame_image,image->compose,image,x,y);
+  image_view=DestroyCacheView(image_view);
+  if ((image->compose != CopyCompositeOp) &&
+      ((image->compose != OverCompositeOp) || (image->matte != MagickFalse)))
+    {
+      x=(long) (frame_info->outer_bevel+(frame_info->x-bevel_width)+
+        frame_info->inner_bevel);
+      y=(long) (frame_info->outer_bevel+(frame_info->y-bevel_width)+
+        frame_info->inner_bevel);
+      (void) CompositeImage(frame_image,image->compose,image,x,y);
+    }
   return(frame_image);
 }
 
@@ -695,7 +730,7 @@ MagickExport MagickBooleanType RaiseImage(Image *image,
       x;
 
     register PixelPacket
-      *__restrict q;
+      *restrict q;
 
     if (status == MagickFalse)
       continue;
@@ -762,7 +797,7 @@ MagickExport MagickBooleanType RaiseImage(Image *image,
       x;
 
     register PixelPacket
-      *__restrict q;
+      *restrict q;
 
     if (status == MagickFalse)
       continue;
@@ -818,7 +853,7 @@ MagickExport MagickBooleanType RaiseImage(Image *image,
       x;
 
     register PixelPacket
-      *__restrict q;
+      *restrict q;
 
     if (status == MagickFalse)
       continue;
