@@ -45,6 +45,7 @@
 #include "magick/studio.h"
 #include "magick/annotate.h"
 #include "magick/artifact.h"
+#include "magick/attribute.h"
 #include "magick/blob.h"
 #include "magick/cache.h"
 #include "magick/client.h"
@@ -60,6 +61,7 @@
 #include "magick/exception-private.h"
 #include "magick/gem.h"
 #include "magick/geometry.h"
+#include "magick/histogram.h"
 #include "magick/identify.h"
 #include "magick/image.h"
 #include "magick/image-private.h"
@@ -126,12 +128,14 @@
 %      about the image.
 %
 */
+
 MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
   const MagickBooleanType verbose)
 {
 #define IdentifyFormat "    %s:\n      min: " QuantumFormat  \
   " (%g)\n      max: " QuantumFormat " (%g)\n"  \
-  "      mean: %g (%g)\n      standard deviation: %g (%g)\n"
+  "      mean: %g (%g)\n      standard deviation: %g (%g)\n"  \
+  "      kurtosis: %g\n      skewness: %g\n"
 
   char
     color[MaxTextExtent],
@@ -240,9 +244,9 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
           (void) FormatMagickSize(GetBlobSize(image),format);
           (void) fprintf(file,"%s ",format);
         }
-      if (elapsed_time > 0.06)
-        (void) fprintf(file,"%0.3fu %ld:%02ld",user_time,(long)
-          (elapsed_time/60.0+0.5),(long) ceil(fmod(elapsed_time,60.0)));
+      (void) fprintf(file,"%0.3fu %ld:%02ld.%03ld",user_time,(long)
+        (elapsed_time/60.0),(long) floor(fmod(elapsed_time,60.0)),
+        (long) (1000.0*(elapsed_time-floor(elapsed_time))));
       (void) fprintf(file,"\n");
       (void) fflush(file);
       return(ferror(file) != 0 ? MagickFalse : MagickTrue);
@@ -362,7 +366,7 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
           break;
         }
       }
-      scale=1.0;
+      scale=1;
       if (image->depth <= MAGICKCORE_QUANTUM_DEPTH)
         scale=QuantumRange/((unsigned long) QuantumRange >> ((unsigned long)
           MAGICKCORE_QUANTUM_DEPTH-image->depth));
@@ -381,7 +385,8 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
             channel_statistics[RedChannel].mean/(double) QuantumRange,
             channel_statistics[RedChannel].standard_deviation/(double) scale,
             channel_statistics[RedChannel].standard_deviation/(double)
-            QuantumRange);
+            QuantumRange,channel_statistics[RedChannel].kurtosis,
+            channel_statistics[RedChannel].skewness);
           (void) fprintf(file,IdentifyFormat,"green",(Quantum)
             (channel_statistics[GreenChannel].minima/scale),(double)
             channel_statistics[GreenChannel].minima/(double) QuantumRange,
@@ -391,7 +396,9 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
             channel_statistics[GreenChannel].mean/(double) QuantumRange,
             channel_statistics[GreenChannel].standard_deviation/(double) scale,
             channel_statistics[GreenChannel].standard_deviation/(double)
-            QuantumRange);
+            QuantumRange,
+            channel_statistics[GreenChannel].kurtosis,
+            channel_statistics[GreenChannel].skewness);
           (void) fprintf(file,IdentifyFormat,"blue",(Quantum)
             (channel_statistics[BlueChannel].minima/scale),(double)
             channel_statistics[BlueChannel].minima/(double) QuantumRange,
@@ -401,7 +408,8 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
             channel_statistics[BlueChannel].mean/(double) QuantumRange,
             channel_statistics[BlueChannel].standard_deviation/(double) scale,
             channel_statistics[BlueChannel].standard_deviation/(double)
-            QuantumRange);
+            QuantumRange,channel_statistics[BlueChannel].kurtosis,
+            channel_statistics[BlueChannel].skewness);
           break;
         }
         case CMYKColorspace:
@@ -415,7 +423,8 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
             channel_statistics[CyanChannel].mean/(double) QuantumRange,
             channel_statistics[CyanChannel].standard_deviation/(double) scale,
             channel_statistics[CyanChannel].standard_deviation/(double)
-            QuantumRange);
+            QuantumRange,channel_statistics[CyanChannel].kurtosis,
+            channel_statistics[CyanChannel].skewness);
           (void) fprintf(file,IdentifyFormat,"magenta",(Quantum)
             (channel_statistics[MagentaChannel].minima/scale),(double)
             channel_statistics[MagentaChannel].minima/(double) QuantumRange,
@@ -425,7 +434,8 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
             channel_statistics[MagentaChannel].mean/(double) QuantumRange,
             channel_statistics[MagentaChannel].standard_deviation/(double)
             scale,channel_statistics[MagentaChannel].standard_deviation/(double)
-            QuantumRange);
+            QuantumRange,channel_statistics[MagentaChannel].kurtosis,
+            channel_statistics[MagentaChannel].skewness);
           (void) fprintf(file,IdentifyFormat,"yellow",(Quantum)
             (channel_statistics[YellowChannel].minima/scale),(double)
             channel_statistics[YellowChannel].minima/(double) QuantumRange,
@@ -435,7 +445,8 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
             channel_statistics[YellowChannel].mean/(double) QuantumRange,
             channel_statistics[YellowChannel].standard_deviation/(double) scale,
             channel_statistics[YellowChannel].standard_deviation/(double)
-            QuantumRange);
+            QuantumRange,channel_statistics[YellowChannel].kurtosis,
+            channel_statistics[YellowChannel].skewness);
           (void) fprintf(file,IdentifyFormat,"black",(Quantum)
             (channel_statistics[BlackChannel].minima/scale),(double)
             channel_statistics[BlackChannel].minima/(double) QuantumRange,
@@ -445,7 +456,8 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
             channel_statistics[BlackChannel].mean/(double) QuantumRange,
             channel_statistics[BlackChannel].standard_deviation/(double) scale,
             channel_statistics[BlackChannel].standard_deviation/(double)
-            QuantumRange);
+            QuantumRange,channel_statistics[BlackChannel].kurtosis,
+            channel_statistics[BlackChannel].skewness);
           break;
         }
         case GRAYColorspace:
@@ -459,7 +471,8 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
             channel_statistics[GrayChannel].mean/(double) QuantumRange,
             channel_statistics[GrayChannel].standard_deviation/(double) scale,
             channel_statistics[GrayChannel].standard_deviation/(double)
-            QuantumRange);
+            QuantumRange,channel_statistics[GrayChannel].kurtosis,
+            channel_statistics[GrayChannel].skewness);
           break;
         }
       }
@@ -475,7 +488,23 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
           (double) QuantumRange,
           channel_statistics[AlphaChannel].standard_deviation/(double) scale,
           channel_statistics[AlphaChannel].standard_deviation/(double)
-          QuantumRange);
+          QuantumRange,channel_statistics[AlphaChannel].kurtosis,
+          channel_statistics[AlphaChannel].skewness);
+      if (colorspace != GRAYColorspace)
+        {
+          (void) fprintf(file,"  Image statistics:\n");
+          (void) fprintf(file,IdentifyFormat,"Overall",(Quantum)
+            (channel_statistics[AllChannels].minima/scale),(double)
+            channel_statistics[AllChannels].minima/(double) QuantumRange,
+            (Quantum) (channel_statistics[AllChannels].maxima/scale),(double)
+            channel_statistics[AllChannels].maxima/(double) QuantumRange,
+            channel_statistics[AllChannels].mean/(double) scale,
+            channel_statistics[AllChannels].mean/(double) QuantumRange,
+            channel_statistics[AllChannels].standard_deviation/(double) scale,
+            channel_statistics[AllChannels].standard_deviation/(double)
+            QuantumRange,channel_statistics[AllChannels].kurtosis,
+            channel_statistics[AllChannels].skewness);
+        }
       channel_statistics=(ChannelStatistics *) RelinquishMagickMemory(
         channel_statistics);
       if (image->colorspace == CMYKColorspace)
@@ -552,7 +581,7 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
             pixel;
 
           register PixelPacket
-            *p;
+            *restrict p;
 
           GetMagickPixelPacket(image,&pixel);
           p=image->colormap;
@@ -636,6 +665,8 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
   (void) QueryColorname(image,&image->transparent_color,SVGCompliance,color,
     &image->exception);
   (void) fprintf(file,"  Transparent color: %s\n",color);
+  (void) fprintf(file,"  Compose: %s\n",MagickOptionToMnemonic(
+    MagickComposeOptions,(long) image->compose));
   if ((image->page.width != 0) || (image->page.height != 0) ||
       (image->page.x != 0) || (image->page.y != 0))
     (void) fprintf(file,"  Page geometry: %lux%lu%+ld%+ld\n",image->page.width,
@@ -791,7 +822,7 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
               }
           }
 #endif
-        if (LocaleCompare(name,"8bim") == 0)
+        if (LocaleCompare(name,"iptc") == 0)
           {
             char
               *attribute,
@@ -812,9 +843,6 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
               length,
               profile_length;
 
-            /*
-              Identify IPTC data.
-            */
             profile_length=GetStringInfoLength(profile);
             for (i=0; i < (long) profile_length; i+=(long) length)
             {
@@ -880,13 +908,13 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
                 case 219: tag="Custom Field 20"; break;
                 default: tag="unknown"; break;
               }
-              (void) fprintf(file,"    %s[%ld,%ld]: ",tag,dataset,record);
+              (void) fprintf(file,"      %s[%ld,%ld]: ",tag,dataset,record);
               length=(size_t) (GetStringInfoDatum(profile)[i++] << 8);
               length|=GetStringInfoDatum(profile)[i++];
               attribute=(char *) NULL;
               if (~length >= MaxTextExtent)
-                attribute=(char *) AcquireQuantumMemory(length+MaxTextExtent,
-                  sizeof(*attribute));
+                attribute=(char *) AcquireQuantumMemory(length+
+                  MaxTextExtent,sizeof(*attribute));
               if (attribute != (char *) NULL)
                 {
                   (void) CopyMagickString(attribute,(char *)
@@ -962,17 +990,15 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
   (void) fprintf(file,"  Filesize: %s\n",format);
   (void) FormatMagickSize((MagickSizeType) image->columns*image->rows,format);
   (void) fprintf(file,"  Number pixels: %s\n",format);
-  if (elapsed_time > 0.06)
-    {
-      (void) FormatMagickSize((MagickSizeType) ((double) image->columns*
-        image->rows/elapsed_time+0.5),format);
-      (void) fprintf(file,"  Pixels per second: %s\n",format);
-      (void) fprintf(file,"  User time: %0.3fu\n",user_time);
-      (void) fprintf(file,"  Elapsed time: %ld:%02ld\n",(long) (elapsed_time/
-        60.0+0.5),(long) ceil(fmod(elapsed_time,60.0)));
-    }
-  (void) fprintf(file,"  Version: %s\n",GetMagickVersion(
-    (unsigned long *) NULL));
+  (void) FormatMagickSize((MagickSizeType) ((double) image->columns*image->rows/
+    elapsed_time+0.5),format);
+  (void) fprintf(file,"  Pixels per second: %s\n",format);
+  (void) fprintf(file,"  User time: %0.3fu\n",user_time);
+  (void) fprintf(file,"  Elapsed time: %ld:%02ld.%03ld\n",(long)
+    (elapsed_time/60.0),(long) ceil(fmod(elapsed_time,60.0)),(long)
+    (1000.0*(elapsed_time-floor(elapsed_time))));
+  (void) fprintf(file,"  Version: %s\n",GetMagickVersion((unsigned long *)
+    NULL));
   (void) fflush(file);
   return(ferror(file) != 0 ? MagickFalse : MagickTrue);
 }
