@@ -16,7 +16,7 @@
 %                               January 2006                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -100,18 +100,19 @@ static void ClearBounds(Image *image,RectangleInfo *bounds)
   long
     y;
 
-  register long
-    x;
-
-  register PixelPacket
-    *q;
-
-  if ( bounds->x < 0 ) return;
+  if (bounds->x < 0)
+    return;
   if (image->matte == MagickFalse)
     (void) SetImageAlphaChannel(image,OpaqueAlphaChannel);
   exception=(&image->exception);
   for (y=0; y < (long) bounds->height; y++)
   {
+    register long
+      x;
+
+    register PixelPacket
+      *restrict q;
+
     q=GetAuthenticPixels(image,bounds->x,bounds->y+y,bounds->width,1,exception);
     if (q == (PixelPacket *) NULL)
       break;
@@ -636,17 +637,17 @@ static RectangleInfo CompareImageBounds(const Image *image1,const Image *image2,
     if (y < (long) image1->rows)
       break;
   }
-  if ( x >= (long) image1->columns) {
-    /*
-      Images are identical! - No need to look further
-      Result in the null 'miss' image.
-    */
-    bounds.x=-1;
-    bounds.y=-1;
-    bounds.width=1;
-    bounds.height=1;
-    return(bounds);
-  }
+  if (x >= (long) image1->columns)
+    {
+      /*
+        Images are identical, return a null image.
+      */
+      bounds.x=-1;
+      bounds.y=-1;
+      bounds.width=1;
+      bounds.height=1;
+      return(bounds);
+    }
   bounds.x=x;
   for (x=(long) image1->columns-1; x >= 0; x--)
   {
@@ -877,7 +878,7 @@ MagickExport Image *CompareImageLayers(const Image *image,
 %  DeconstructImages() compares each image with the next in a sequence and
 %  returns the minimum bounding region of all differences from the first image.
 %
-%  This function is depreciated in favor of the more universal
+%  This function is deprecated in favor of the more universal
 %  CompareImageLayers() function.
 %
 %  The format of the DeconstructImages method is:
