@@ -17,7 +17,7 @@
 %                                September 1996                               %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -554,7 +554,6 @@ MagickExport void MACErrorHandler(const ExceptionType error,const char *reason,
 %
 %    o description: Specifies any description to the reason.
 %
-%
 */
 static void MACFatalErrorHandler(const ExceptionType severity,
   const char *reason,const char *description)
@@ -1227,15 +1226,6 @@ MagickExport Image *ReadPICTImage(const ImageInfo *image_info,
   Rect
     rectangle;
 
-  register IndexPacket
-    *indexes;
-
-  register long
-    x;
-
-  register PixelPacket
-    *q;
-
   RGBColor
     Pixel;
 
@@ -1359,6 +1349,15 @@ MagickExport Image *ReadPICTImage(const ImageInfo *image_info,
   */
   for (y=0; y < image->rows; y++)
   {
+    register IndexPacket
+      *restrict indexes;
+
+    register long
+      x;
+
+    register PixelPacket
+      *restrict q;
+
     q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
     if (q == (PixelPacket *) NULL)
       break;
