@@ -1,5 +1,5 @@
 /*
-  Copyright 1999-2007 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
   
   You may not use this file except in compliance with the License.
@@ -24,11 +24,11 @@ extern "C" {
 
 #include "magick/magick-config.h"
 
-#if !defined(QuantumDepth)
-#define QuantumDepth  16
+#if !defined(MAGICKCORE_QUANTUM_DEPTH)
+#define MAGICKCORE_QUANTUM_DEPTH  16
 #endif
 
-#if defined(__WINDOWS__)
+#if defined(__WINDOWS__) && !defined(__MINGW32__)
 #  define MagickLLConstant(c)  (MagickOffsetType) (c ## i64)
 #  define MagickULLConstant(c)  (MagickSizeType) (c ## ui64)
 #else
@@ -36,77 +36,73 @@ extern "C" {
 #  define MagickULLConstant(c)  (MagickSizeType) (c ## ULL)
 #endif
 
-#if (QuantumDepth == 8)
+#if (MAGICKCORE_QUANTUM_DEPTH == 8)
 #define MagickEpsilon  1.0e-6
 #define MagickHuge     1.0e6
 #define MaxColormapSize  256UL
-#define MaxMap  ((Quantum) 255)
-#define QuantumRange  ((Quantum) 255)
+#define MaxMap  255UL
 
-typedef float MagickRealType;
-#if defined(UseHDRI)
+typedef double MagickRealType;
+#if defined(MAGICKCORE_HDRI_SUPPORT)
 typedef float Quantum;
+#define QuantumRange  255.0
 #define QuantumFormat  "%g"
 #else
 typedef unsigned char Quantum;
+#define QuantumRange  255UL
 #define QuantumFormat  "%u"
 #endif
-typedef unsigned long QuantumAny;
-#elif (QuantumDepth == 16)
+#elif (MAGICKCORE_QUANTUM_DEPTH == 16)
 #define MagickEpsilon  1.0e-10
 #define MagickHuge     1.0e12
 #define MaxColormapSize  65536UL
-#define MaxMap  ((Quantum) 65535)
-#define QuantumRange  ((Quantum) 65535)
+#define MaxMap  65535UL
 
 typedef double MagickRealType;
-#if defined(UseHDRI)
+#if defined(MAGICKCORE_HDRI_SUPPORT)
 typedef float Quantum;
+#define QuantumRange  65535.0
 #define QuantumFormat  "%g"
 #else
 typedef unsigned short Quantum;
+#define QuantumRange  65535UL
 #define QuantumFormat  "%u"
 #endif
-typedef unsigned long QuantumAny;
-#elif (QuantumDepth == 32)
+#elif (MAGICKCORE_QUANTUM_DEPTH == 32)
 #define MagickEpsilon  1.0e-10
 #define MagickHuge     1.0e12
 #define MaxColormapSize  65536UL
-#define MaxMap  ((Quantum) 65535)
-#define QuantumRange  ((Quantum) 4294967295UL)
+#define MaxMap  65535UL
 
-#if defined(HAVE_LONG_DOUBLE)
-typedef long double MagickRealType;
-#else
 typedef double MagickRealType;
-#endif
-#if defined(UseHDRI)
+#if defined(MAGICKCORE_HDRI_SUPPORT)
 typedef float Quantum;
+#define QuantumRange  4294967295.0
 #define QuantumFormat  "%g"
 #else
 typedef unsigned int Quantum;
+#define QuantumRange  4294967295UL
 #define QuantumFormat  "%u"
 #endif
-typedef unsigned long QuantumAny;
-#elif (QuantumDepth == 64) && defined(HAVE_LONG_DOUBLE)
+#elif (MAGICKCORE_QUANTUM_DEPTH == 64) && defined(MAGICKCORE_HAVE_LONG_DOUBLE_WIDER)
 #define MagickEpsilon  1.0e-10
 #define MagickHuge     1.0e12
 #define MaxColormapSize  65536UL
-#define MaxMap  ((Quantum) 65535)
-#define QuantumRange  ((Quantum) MagickULLConstant(18446744073709551615))
+#define MaxMap  65535UL
 
 typedef long double MagickRealType;
-#if defined(UseHDRI)
+#if defined(MAGICKCORE_HDRI_SUPPORT)
 typedef double Quantum;
+#define QuantumRange  18446744073709551615.0
 #define QuantumFormat  "%g"
 #else
 typedef unsigned long long Quantum;
+#define QuantumRange  MagickULLConstant(18446744073709551615)
 #define QuantumFormat  "%llu"
 #endif
-typedef unsigned long long QuantumAny;
 #else
 #if !defined(_CH_)
-# error "Specified value of QuantumDepth is not supported"
+# error "Specified value of MAGICKCORE_QUANTUM_DEPTH is not supported"
 #endif
 #endif
 #define MaxRGB  QuantumRange  /* deprecated */
@@ -116,7 +112,7 @@ typedef unsigned long long QuantumAny;
 */
 typedef unsigned int MagickStatusType;
 #if !defined(__WINDOWS__)
-#if (SIZEOF_LONG_LONG == 8)
+#if (MAGICKCORE_SIZEOF_UNSIGNED_LONG_LONG == 8)
 typedef long long MagickOffsetType;
 typedef unsigned long long MagickSizeType;
 #define MagickSizeFormat  "%10llu"
@@ -131,6 +127,12 @@ typedef unsigned __int64 MagickSizeType;
 #define MagickSizeFormat  "%10llu"
 #endif
 
+#if defined(_MSC_VER) && (_MSC_VER == 1200)
+typedef MagickOffsetType QuantumAny;
+#else
+typedef MagickSizeType QuantumAny;
+#endif
+
 #if defined(macintosh)
 #define ExceptionInfo  MagickExceptionInfo
 #endif
@@ -150,8 +152,13 @@ typedef enum
   MatteChannel = 0x0008,  /* deprecated */
   BlackChannel = 0x0020,
   IndexChannel = 0x0020,
-  AllChannels = 0xff,
-  DefaultChannels = (AllChannels &~ OpacityChannel)
+  AllChannels = 0x002F,
+  /* special channel types */
+  TrueAlphaChannel = 0x0040, /* extract actual alpha channel from opacity */
+  RGBChannels = 0x0080,      /* set alpha from  grayscale mask in RGB */
+  GrayChannels = 0x0080,
+  SyncChannels = 0x0100,     /* channels should be modified equally */
+  DefaultChannels = ( (AllChannels | SyncChannels) &~ OpacityChannel)
 } ChannelType;
 
 typedef enum
