@@ -2,6 +2,7 @@
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
 %                                                                             %
+%                                                                             %
 %                      PPPP    AAA   IIIII  N   N  TTTTT                      %
 %                      P   P  A   A    I    NN  N    T                        %
 %                      PPPP   AAAAA    I    N N N    T                        %
@@ -16,7 +17,7 @@
 %                                 July 1998                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -54,6 +55,7 @@
 #include "magick/paint.h"
 #include "magick/pixel-private.h"
 #include "magick/string_.h"
+#include "magick/thread-private.h"
 
 static inline double MagickMax(const double x,const double y)
 {
@@ -136,6 +138,7 @@ MagickExport MagickBooleanType FloodfillPaintImage(Image *image,
   long
     offset,
     start,
+    x,
     x1,
     x2,
     y;
@@ -150,18 +153,6 @@ MagickExport MagickBooleanType FloodfillPaintImage(Image *image,
   PixelPacket
     fill_color;
 
-  register const IndexPacket
-    *indexes;
-
-  register const PixelPacket
-    *p;
-
-  register long
-    x;
-
-  register PixelPacket
-    *q;
-
   register SegmentInfo
     *s;
 
@@ -188,8 +179,7 @@ MagickExport MagickBooleanType FloodfillPaintImage(Image *image,
   /*
     Set floodfill state.
   */
-  floodplane_image=CloneImage(image,image->columns,image->rows,MagickTrue,
-    &image->exception);
+  floodplane_image=CloneImage(image,0,0,MagickTrue,&image->exception);
   if (floodplane_image == (Image *) NULL)
     return(MagickFalse);
   (void) SetImageAlphaChannel(floodplane_image,OpaqueAlphaChannel);
@@ -215,6 +205,18 @@ MagickExport MagickBooleanType FloodfillPaintImage(Image *image,
   GetMagickPixelPacket(image,&pixel);
   while (s > segment_stack)
   {
+    register const IndexPacket
+      *restrict indexes;
+
+    register const PixelPacket
+      *restrict p;
+
+    register long
+      x;
+
+    register PixelPacket
+      *restrict q;
+
     /*
       Pop segment off stack.
     */
@@ -227,7 +229,8 @@ MagickExport MagickBooleanType FloodfillPaintImage(Image *image,
       Recolor neighboring pixels.
     */
     p=GetVirtualPixels(image,0,y,(unsigned long) (x1+1),1,exception);
-    q=GetAuthenticPixels(floodplane_image,0,y,(unsigned long) (x1+1),1,exception);
+    q=GetAuthenticPixels(floodplane_image,0,y,(unsigned long) (x1+1),1,
+      exception);
     if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
       break;
     indexes=GetVirtualIndexQueue(image);
@@ -311,8 +314,17 @@ MagickExport MagickBooleanType FloodfillPaintImage(Image *image,
   }
   for (y=0; y < (long) image->rows; y++)
   {
+    register const PixelPacket
+      *restrict p;
+
     register IndexPacket
-      *indexes;
+      *restrict indexes;
+
+    register long
+      x;
+
+    register PixelPacket
+      *restrict q;
 
     /*
       Tile fill color onto floodplane.
@@ -498,10 +510,11 @@ static unsigned long **DestroyHistogramThreadSet(unsigned long **histogram)
     i;
 
   assert(histogram != (unsigned long **) NULL);
-  for (i=0; i < (long) GetCacheViewMaximumThreads(); i++)
+  for (i=0; i < (long) GetOpenMPMaximumThreads(); i++)
     if (histogram[i] != (unsigned long *) NULL)
       histogram[i]=(unsigned long *) RelinquishMagickMemory(histogram[i]);
-  return((unsigned long **) RelinquishMagickMemory(histogram));
+  histogram=(unsigned long **) RelinquishAlignedMemory(histogram);
+  return(histogram);
 }
 
 static unsigned long **AcquireHistogramThreadSet(const size_t count)
@@ -510,15 +523,16 @@ static unsigned long **AcquireHistogramThreadSet(const size_t count)
     i;
 
   unsigned long
-    **histogram;
+    **histogram,
+    number_threads;
 
-  histogram=(unsigned long **) AcquireQuantumMemory(
-    GetCacheViewMaximumThreads(),sizeof(*histogram));
+  number_threads=GetOpenMPMaximumThreads();
+  histogram=(unsigned long **) AcquireAlignedMemory(number_threads,
+    sizeof(*histogram));
   if (histogram == (unsigned long **) NULL)
     return((unsigned long **) NULL);
-  (void) ResetMagickMemory(histogram,0,GetCacheViewMaximumThreads()*
-    sizeof(*histogram));
-  for (i=0; i < (long) GetCacheViewMaximumThreads(); i++)
+  (void) ResetMagickMemory(histogram,0,number_threads*sizeof(*histogram));
+  for (i=0; i < (long) number_threads; i++)
   {
     histogram[i]=(unsigned long *) AcquireQuantumMemory(count,
       sizeof(**histogram));
@@ -545,12 +559,12 @@ MagickExport Image *OilPaintImage(const Image *image,const double radius,
     status;
 
   unsigned long
-    **histogram,
+    **histograms,
     width;
 
-  ViewInfo
-    **image_view,
-    **paint_view;
+  CacheView
+    *image_view,
+    *paint_view;
 
   /*
     Initialize painted image attributes.
@@ -573,8 +587,8 @@ MagickExport Image *OilPaintImage(const Image *image,const double radius,
       paint_image=DestroyImage(paint_image);
       return((Image *) NULL);
     }
-  histogram=AcquireHistogramThreadSet(NumberPaintBins);
-  if (histogram == (unsigned long **) NULL)
+  histograms=AcquireHistogramThreadSet(NumberPaintBins);
+  if (histograms == (unsigned long **) NULL)
     {
       paint_image=DestroyImage(paint_image);
       ThrowImageException(ResourceLimitError,"MemoryAllocationFailed");
@@ -584,43 +598,45 @@ MagickExport Image *OilPaintImage(const Image *image,const double radius,
   */
   status=MagickTrue;
   progress=0;
-  image_view=AcquireCacheViewThreadSet(image);
-  paint_view=AcquireCacheViewThreadSet(paint_image);
+  image_view=AcquireCacheView(image);
+  paint_view=AcquireCacheView(paint_image);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static,16) shared(progress,status)
+  #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
 #endif
   for (y=0; y < (long) image->rows; y++)
   {
-    const IndexPacket
-      *indexes;
-
-    IndexPacket
-      *paint_indexes;
+    register const IndexPacket
+      *restrict indexes;
 
     register const PixelPacket
-      *p;
+      *restrict p;
+
+    register IndexPacket
+      *restrict paint_indexes;
 
     register long
-      id,
       x;
 
     register PixelPacket
-      *q;
+      *restrict q;
+
+    register unsigned long
+      *histogram;
 
     if (status == MagickFalse)
       continue;
-    id=GetCacheViewThreadId();
-    p=GetCacheViewVirtualPixels(image_view[id],-((long) width/2L),y-(long)
-      (width/2L),image->columns+width,width,exception);
-    q=QueueCacheViewAuthenticPixels(paint_view[id],0,y,paint_image->columns,1,
+    p=GetCacheViewVirtualPixels(image_view,-((long) width/2L),y-(long) (width/
+      2L),image->columns+width,width,exception);
+    q=QueueCacheViewAuthenticPixels(paint_view,0,y,paint_image->columns,1,
       exception);
     if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
       {
         status=MagickFalse;
         continue;
       }
-    indexes=GetCacheViewVirtualIndexes(image_view[id]);
-    paint_indexes=GetCacheViewAuthenticIndexes(paint_view[id]);
+    indexes=GetCacheViewVirtualIndexQueue(image_view);
+    paint_indexes=GetCacheViewAuthenticIndexQueue(paint_view);
+    histogram=histograms[GetOpenMPThreadId()];
     for (x=0; x < (long) image->columns; x++)
     {
       long
@@ -641,18 +657,17 @@ MagickExport Image *OilPaintImage(const Image *image,const double radius,
       i=0;
       j=0;
       count=0;
-      (void) ResetMagickMemory(histogram[id],0,NumberPaintBins*
-        sizeof(**histogram));
+      (void) ResetMagickMemory(histogram,0,NumberPaintBins*sizeof(*histogram));
       for (v=0; v < (long) width; v++)
       {
         for (u=0; u < (long) width; u++)
         {
           k=(long) ScaleQuantumToChar(PixelIntensityToQuantum(p+u+i));
-          histogram[id][k]++;
-          if (histogram[id][k] > count)
+          histogram[k]++;
+          if (histogram[k] > count)
             {
               j=i+u;
-              count=histogram[id][k];
+              count=histogram[k];
             }
         }
         i+=image->columns+width;
@@ -663,7 +678,7 @@ MagickExport Image *OilPaintImage(const Image *image,const double radius,
       p++;
       q++;
     }
-    if (SyncCacheViewAuthenticPixels(paint_view[id],exception) == MagickFalse)
+    if (SyncCacheViewAuthenticPixels(paint_view,exception) == MagickFalse)
       status=MagickFalse;
     if (image->progress_monitor != (MagickProgressMonitor) NULL)
       {
@@ -671,16 +686,16 @@ MagickExport Image *OilPaintImage(const Image *image,const double radius,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-        #pragma omp critical
+  #pragma omp critical (MagickCore_OilPaintImage)
 #endif
         proceed=SetImageProgress(image,OilPaintImageTag,progress++,image->rows);
         if (proceed == MagickFalse)
           status=MagickFalse;
       }
   }
-  paint_view=DestroyCacheViewThreadSet(paint_view);
-  image_view=DestroyCacheViewThreadSet(image_view);
-  histogram=DestroyHistogramThreadSet(histogram);
+  paint_view=DestroyCacheView(paint_view);
+  image_view=DestroyCacheView(image_view);
+  histograms=DestroyHistogramThreadSet(histograms);
   if (status == MagickFalse)
     paint_image=DestroyImage(paint_image);
   return(paint_image);
@@ -755,8 +770,8 @@ MagickExport MagickBooleanType OpaquePaintImageChannel(Image *image,
   MagickPixelPacket
     zero;
 
-  ViewInfo
-    **image_view;
+  CacheView
+    *image_view;
 
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
@@ -773,9 +788,9 @@ MagickExport MagickBooleanType OpaquePaintImageChannel(Image *image,
   progress=0;
   exception=(&image->exception);
   GetMagickPixelPacket(image,&zero);
-  image_view=AcquireCacheViewThreadSet(image);
+  image_view=AcquireCacheView(image);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static,16) shared(progress,status)
+  #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
 #endif
   for (y=0; y < (long) image->rows; y++)
   {
@@ -783,26 +798,23 @@ MagickExport MagickBooleanType OpaquePaintImageChannel(Image *image,
       pixel;
 
     register IndexPacket
-      *indexes;
+      *restrict indexes;
 
     register long
-      id,
       x;
 
     register PixelPacket
-      *q;
+      *restrict q;
 
     if (status == MagickFalse)
       continue;
-    id=GetCacheViewThreadId();
-    q=GetCacheViewAuthenticPixels(image_view[id],0,y,image->columns,1,
-      exception);
+    q=GetCacheViewAuthenticPixels(image_view,0,y,image->columns,1,exception);
     if (q == (PixelPacket *) NULL)
       {
         status=MagickFalse;
         continue;
       }
-    indexes=GetCacheViewAuthenticIndexes(image_view[id]);
+    indexes=GetCacheViewAuthenticIndexQueue(image_view);
     pixel=zero;
     for (x=0; x < (long) image->columns; x++)
     {
@@ -823,7 +835,7 @@ MagickExport MagickBooleanType OpaquePaintImageChannel(Image *image,
         }
       q++;
     }
-    if (SyncCacheViewAuthenticPixels(image_view[id],exception) == MagickFalse)
+    if (SyncCacheViewAuthenticPixels(image_view,exception) == MagickFalse)
       status=MagickFalse;
     if (image->progress_monitor != (MagickProgressMonitor) NULL)
       {
@@ -831,7 +843,7 @@ MagickExport MagickBooleanType OpaquePaintImageChannel(Image *image,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-        #pragma omp critical
+  #pragma omp critical (MagickCore_OpaquePaintImageChannel)
 #endif
         proceed=SetImageProgress(image,OpaquePaintImageTag,progress++,
           image->rows);
@@ -839,7 +851,7 @@ MagickExport MagickBooleanType OpaquePaintImageChannel(Image *image,
           status=MagickFalse;
       }
   }
-  image_view=DestroyCacheViewThreadSet(image_view);
+  image_view=DestroyCacheView(image_view);
   return(status);
 }
 
@@ -899,8 +911,8 @@ MagickExport MagickBooleanType TransparentPaintImage(Image *image,
   MagickPixelPacket
     zero;
 
-  ViewInfo
-    **image_view;
+  CacheView
+    *image_view;
 
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
@@ -918,9 +930,9 @@ MagickExport MagickBooleanType TransparentPaintImage(Image *image,
   progress=0;
   exception=(&image->exception);
   GetMagickPixelPacket(image,&zero);
-  image_view=AcquireCacheViewThreadSet(image);
+  image_view=AcquireCacheView(image);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static,16) shared(progress,status)
+  #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
 #endif
   for (y=0; y < (long) image->rows; y++)
   {
@@ -928,26 +940,23 @@ MagickExport MagickBooleanType TransparentPaintImage(Image *image,
       pixel;
 
     register IndexPacket
-      *indexes;
+      *restrict indexes;
 
     register long
-      id,
       x;
 
     register PixelPacket
-      *q;
+      *restrict q;
 
     if (status == MagickFalse)
       continue;
-    id=GetCacheViewThreadId();
-    q=GetCacheViewAuthenticPixels(image_view[id],0,y,image->columns,1,
-      exception);
+    q=GetCacheViewAuthenticPixels(image_view,0,y,image->columns,1,exception);
     if (q == (PixelPacket *) NULL)
       {
         status=MagickFalse;
         continue;
       }
-    indexes=GetCacheViewAuthenticIndexes(image_view[id]);
+    indexes=GetCacheViewAuthenticIndexQueue(image_view);
     pixel=zero;
     for (x=0; x < (long) image->columns; x++)
     {
@@ -956,7 +965,7 @@ MagickExport MagickBooleanType TransparentPaintImage(Image *image,
         q->opacity=opacity;
       q++;
     }
-    if (SyncCacheViewAuthenticPixels(image_view[id],exception) == MagickFalse)
+    if (SyncCacheViewAuthenticPixels(image_view,exception) == MagickFalse)
       status=MagickFalse;
     if (image->progress_monitor != (MagickProgressMonitor) NULL)
       {
@@ -964,7 +973,7 @@ MagickExport MagickBooleanType TransparentPaintImage(Image *image,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-        #pragma omp critical
+  #pragma omp critical (MagickCore_TransparentPaintImage)
 #endif
         proceed=SetImageProgress(image,TransparentPaintImageTag,progress++,
           image->rows);
@@ -972,10 +981,10 @@ MagickExport MagickBooleanType TransparentPaintImage(Image *image,
           status=MagickFalse;
       }
   }
-  image_view=DestroyCacheViewThreadSet(image_view);
+  image_view=DestroyCacheView(image_view);
   return(status);
 }
-
+
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -1032,8 +1041,8 @@ MagickExport MagickBooleanType TransparentPaintImageChroma(Image *image,
   MagickBooleanType
     status;
 
-  ViewInfo
-    **image_view;
+  CacheView
+    *image_view;
 
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
@@ -1051,9 +1060,9 @@ MagickExport MagickBooleanType TransparentPaintImageChroma(Image *image,
   status=MagickTrue;
   progress=0;
   exception=(&image->exception);
-  image_view=AcquireCacheViewThreadSet(image);
+  image_view=AcquireCacheView(image);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static,16) shared(progress,status)
+  #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
 #endif
   for (y=0; y < (long) image->rows; y++)
   {
@@ -1064,26 +1073,23 @@ MagickExport MagickBooleanType TransparentPaintImageChroma(Image *image,
       pixel;
 
     register IndexPacket
-      *indexes;
+      *restrict indexes;
 
     register long
-      id,
       x;
 
     register PixelPacket
-      *q;
+      *restrict q;
 
     if (status == MagickFalse)
       continue;
-    id=GetCacheViewThreadId();
-    q=GetCacheViewAuthenticPixels(image_view[id],0,y,image->columns,1,
-      exception);
+    q=GetCacheViewAuthenticPixels(image_view,0,y,image->columns,1,exception);
     if (q == (PixelPacket *) NULL)
       {
         status=MagickFalse;
         continue;
       }
-    indexes=GetCacheViewAuthenticIndexes(image_view[id]);
+    indexes=GetCacheViewAuthenticIndexQueue(image_view);
     GetMagickPixelPacket(image,&pixel);
     for (x=0; x < (long) image->columns; x++)
     {
@@ -1096,7 +1102,7 @@ MagickExport MagickBooleanType TransparentPaintImageChroma(Image *image,
         q->opacity=opacity;
       q++;
     }
-    if (SyncCacheViewAuthenticPixels(image_view[id],exception) == MagickFalse)
+    if (SyncCacheViewAuthenticPixels(image_view,exception) == MagickFalse)
       status=MagickFalse;
     if (image->progress_monitor != (MagickProgressMonitor) NULL)
       {
@@ -1104,7 +1110,7 @@ MagickExport MagickBooleanType TransparentPaintImageChroma(Image *image,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp critical
+  #pragma omp critical (MagickCore_TransparentPaintImageChroma)
 #endif
         proceed=SetImageProgress(image,TransparentPaintImageTag,progress++,
           image->rows);
@@ -1112,6 +1118,6 @@ MagickExport MagickBooleanType TransparentPaintImageChroma(Image *image,
           status=MagickFalse;
       }
   }
-  image_view=DestroyCacheViewThreadSet(image_view);
+  image_view=DestroyCacheView(image_view);
   return(status);
 }
