@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -49,6 +49,7 @@
 #include "magick/image.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/monitor-private.h"
 #include "magick/option.h"
 #include "magick/profile.h"
 #include "magick/property.h"
@@ -56,6 +57,8 @@
 #include "magick/quantum-private.h"
 #include "magick/splay-tree.h"
 #include "magick/string_.h"
+#include "magick/thread-private.h"
+#include "magick/token.h"
 #include "magick/utility.h"
 #if defined(MAGICKCORE_LCMS_DELEGATE)
 #if defined(MAGICKCORE_HAVE_LCMS_LCMS_H)
@@ -308,6 +311,93 @@ MagickExport char *GetNextImageProfile(const Image *image)
 %
 */
 
+#if defined(MAGICKCORE_LCMS_DELEGATE)
+
+static unsigned short **DestroyPixelThreadSet(unsigned short **pixels)
+{
+  register long
+    i;
+
+  assert(pixels != (unsigned short **) NULL);
+  for (i=0; i < (long) GetOpenMPMaximumThreads(); i++)
+    if (pixels[i] != (unsigned short *) NULL)
+      pixels[i]=(unsigned short *) RelinquishMagickMemory(pixels[i]);
+  pixels=(unsigned short **) RelinquishAlignedMemory(pixels);
+  return(pixels);
+}
+
+static unsigned short **AcquirePixelThreadSet(const size_t columns,
+  const size_t channels)
+{
+  register long
+    i;
+
+  unsigned short
+    **pixels;
+
+  unsigned long
+    number_threads;
+
+  number_threads=GetOpenMPMaximumThreads();
+  pixels=(unsigned short **) AcquireAlignedMemory(number_threads,
+    sizeof(*pixels));
+  if (pixels == (unsigned short **) NULL)
+    return((unsigned short **) NULL);
+  (void) ResetMagickMemory(pixels,0,number_threads*sizeof(*pixels));
+  for (i=0; i < (long) number_threads; i++)
+  {
+    pixels[i]=(unsigned short *) AcquireQuantumMemory(columns,channels*
+      sizeof(**pixels));
+    if (pixels[i] == (unsigned short *) NULL)
+      return(DestroyPixelThreadSet(pixels));
+  }
+  return(pixels);
+}
+
+static cmsHTRANSFORM *DestroyTransformThreadSet(cmsHTRANSFORM *transform)
+{
+  register long
+    i;
+
+  assert(transform != (cmsHTRANSFORM *) NULL);
+  for (i=0; i < (long) GetOpenMPMaximumThreads(); i++)
+    if (transform[i] != (cmsHTRANSFORM) NULL)
+      cmsDeleteTransform(transform[i]);
+  transform=(cmsHTRANSFORM *) RelinquishAlignedMemory(transform);
+  return(transform);
+}
+
+static cmsHTRANSFORM *AcquireTransformThreadSet(
+  const cmsHPROFILE source_profile,const DWORD source_type,
+  const cmsHPROFILE target_profile,const DWORD target_type,const int intent,
+  const DWORD flags)
+{
+  cmsHTRANSFORM
+    *transform;
+
+  register long
+    i;
+
+  unsigned long
+    number_threads;
+
+  number_threads=GetOpenMPMaximumThreads();
+  transform=(cmsHTRANSFORM *) AcquireAlignedMemory(number_threads,
+    sizeof(*transform));
+  if (transform == (cmsHTRANSFORM *) NULL)
+    return((cmsHTRANSFORM *) NULL);
+  (void) ResetMagickMemory(transform,0,number_threads*sizeof(*transform));
+  for (i=0; i < (long) number_threads; i++)
+  {
+    transform[i]=cmsCreateTransform(source_profile,source_type,target_profile,
+      target_type,intent,flags);
+    if (transform[i] == (cmsHTRANSFORM) NULL)
+      return(DestroyTransformThreadSet(transform));
+  }
+  return(transform);
+}
+#endif
+
 static MagickBooleanType SetAdobeRGB1998ImageProfile(Image *image)
 {
   static unsigned char
@@ -694,11 +784,11 @@ static MagickBooleanType SetsRGBImageProfile(Image *image)
 }
 #if defined(MAGICKCORE_LCMS_DELEGATE)
 #if defined(LCMS_VERSION) && (LCMS_VERSION > 1010)
-static int LCMSErrorHandler(int severity,const char *context)
+static int LCMSErrorHandler(int severity,const char *message)
 {
   (void) LogMagickEvent(TransformEvent,GetMagickModule(),"lcms: #%d, %s",
-    severity,context != (char *) NULL ? context : "no context");
-  return(MagickTrue);
+    severity,message != (char *) NULL ? message : "no message");
+  return(1);
 }
 #endif
 #endif
@@ -707,6 +797,7 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
   const void *datum,const size_t length,
   const MagickBooleanType magick_unused(clone))
 {
+#define ProfileImageTag  "Profile/Image"
 #define ThrowProfileException(severity,tag,context) \
 { \
   (void) cmsCloseProfile(source_profile); \
@@ -727,12 +818,46 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
   assert(name != (const char *) NULL);
   if ((datum == (const void *) NULL) || (length == 0))
     {
-      if (*name == '*')
+      char
+        **arguments,
+        *names;
+
+      int
+        number_arguments;
+
+      register long
+        i;
+
+      /*
+        Delete image profile(s).
+      */
+      names=ConstantString(name);
+      (void) SubstituteString(&names,","," ");
+      arguments=StringToArgv(names,&number_arguments);
+      names=DestroyString(names);
+      if (arguments == (char **) NULL)
+        return(MagickTrue);
+      ResetImageProfileIterator(image);
+      for (name=GetNextImageProfile(image); name != (const char *) NULL; )
+      {
+        for (i=1; i < number_arguments; i++)
         {
-          DestroyImageProfiles(image);
-          return(MagickTrue);
+          if ((*arguments[i] == '!') &&
+              (LocaleCompare(name,arguments[i]+1) == 0))
+            break;
+          if (GlobExpression(name,arguments[i],MagickTrue) != MagickFalse)
+            {
+              (void) DeleteImageProfile(image,name);
+              ResetImageProfileIterator(image);
+              break;
+            }
         }
-      return(DeleteImageProfile(image,name));
+        name=GetNextImageProfile(image);
+      }
+      for (i=0; i < number_arguments; i++)
+        arguments[i]=DestroyString(arguments[i]);
+      arguments=(char **) RelinquishMagickMemory(arguments);
+      return(MagickTrue);
     }
   /*
     Add a ICC, IPTC, or generic profile to the image.
@@ -775,6 +900,9 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
 #else
       if (icc_profile != (StringInfo *) NULL)
         {
+          CacheView
+            *image_view;
+
           ColorspaceType
             source_colorspace,
             target_colorspace;
@@ -784,7 +912,7 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
             target_profile;
 
           cmsHTRANSFORM
-            transform;
+            *transform;
 
           DWORD
             flags,
@@ -794,23 +922,15 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
           ExceptionInfo
             *exception;
 
-          IndexPacket
-            *indexes;
-
           int
             intent;
 
           long
+            progress,
             y;
 
-          register long
-            x;
-
-          register PixelPacket
-            *q;
-
-          register unsigned short
-            *p;
+          MagickBooleanType
+            status;
 
           size_t
             length,
@@ -818,8 +938,8 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
             target_channels;
 
           unsigned short
-            *source_pixels,
-            *target_pixels;
+            **source_pixels,
+            **target_pixels;
 
           /*
             Transform pixel colors as defined by the color profiles.
@@ -846,11 +966,11 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
               source_channels=4;
               break;
             }
-            case icSigYCbCrData:
+            case icSigGrayData:
             {
-              source_colorspace=YCbCrColorspace;
-              source_type=(DWORD) TYPE_YCbCr_16;
-              source_channels=3;
+              source_colorspace=GRAYColorspace;
+              source_type=(DWORD) TYPE_GRAY_16;
+              source_channels=1;
               break;
             }
             case icSigLabData:
@@ -867,13 +987,6 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
               source_channels=3;
               break;
             }
-            case icSigGrayData:
-            {
-              source_colorspace=GRAYColorspace;
-              source_type=(DWORD) TYPE_GRAY_16;
-              source_channels=1;
-              break;
-            }
             case icSigRgbData:
             {
               source_colorspace=RGBColorspace;
@@ -881,6 +994,20 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
               source_channels=3;
               break;
             }
+            case icSigXYZData:
+            {
+              source_colorspace=XYZColorspace;
+              source_type=(DWORD) TYPE_XYZ_16;
+              source_channels=3;
+              break;
+            }
+            case icSigYCbCrData:
+            {
+              source_colorspace=YCbCrColorspace;
+              source_type=(DWORD) TYPE_YCbCr_16;
+              source_channels=3;
+              break;
+            }
             default:
             {
               source_colorspace=UndefinedColorspace;
@@ -898,13 +1025,6 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
               target_channels=4;
               break;
             }
-            case icSigYCbCrData:
-            {
-              target_colorspace=YCbCrColorspace;
-              target_type=(DWORD) TYPE_YCbCr_16;
-              target_channels=3;
-              break;
-            }
             case icSigLabData:
             {
               target_colorspace=LabColorspace;
@@ -912,13 +1032,6 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
               target_channels=3;
               break;
             }
-            case icSigLuvData:
-            {
-              target_colorspace=YUVColorspace;
-              target_type=(DWORD) TYPE_YUV_16;
-              target_channels=3;
-              break;
-            }
             case icSigGrayData:
             {
               target_colorspace=GRAYColorspace;
@@ -926,6 +1039,13 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
               target_channels=1;
               break;
             }
+            case icSigLuvData:
+            {
+              target_colorspace=YUVColorspace;
+              target_type=(DWORD) TYPE_YUV_16;
+              target_channels=3;
+              break;
+            }
             case icSigRgbData:
             {
               target_colorspace=RGBColorspace;
@@ -933,6 +1053,20 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
               target_channels=3;
               break;
             }
+            case icSigXYZData:
+            {
+              target_colorspace=XYZColorspace;
+              target_type=(DWORD) TYPE_XYZ_16;
+              target_channels=3;
+              break;
+            }
+            case icSigYCbCrData:
+            {
+              target_colorspace=YCbCrColorspace;
+              target_type=(DWORD) TYPE_YCbCr_16;
+              target_channels=3;
+              break;
+            }
             default:
             {
               target_colorspace=UndefinedColorspace;
@@ -942,16 +1076,32 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
             }
           }
           exception=(&image->exception);
-          if (((source_colorspace == UndefinedColorspace) ||
-               (target_colorspace == UndefinedColorspace)) ||
-              ((source_colorspace == GRAYColorspace) &&
-               (!IsGrayImage(image,exception))) ||
-              ((source_colorspace == CMYKColorspace) &&
-               (image->colorspace != CMYKColorspace)) ||
-              ((source_colorspace != GRAYColorspace) &&
+          if ((source_colorspace == UndefinedColorspace) ||
+              (target_colorspace == UndefinedColorspace))
+            ThrowProfileException(ImageError,"ColorspaceColorProfileMismatch",
+              name);
+           if ((source_colorspace == GRAYColorspace) &&
+               (IsGrayImage(image,exception) == MagickFalse))
+            ThrowProfileException(ImageError,"ColorspaceColorProfileMismatch",
+              name);
+           if ((source_colorspace == CMYKColorspace) &&
+               (image->colorspace != CMYKColorspace))
+            ThrowProfileException(ImageError,"ColorspaceColorProfileMismatch",
+              name);
+           if ((source_colorspace == XYZColorspace) &&
+               (image->colorspace != XYZColorspace))
+            ThrowProfileException(ImageError,"ColorspaceColorProfileMismatch",
+              name);
+           if ((source_colorspace == YCbCrColorspace) &&
+               (image->colorspace != YCbCrColorspace))
+            ThrowProfileException(ImageError,"ColorspaceColorProfileMismatch",
+              name);
+           if ((source_colorspace != CMYKColorspace) &&
+               (source_colorspace != GRAYColorspace) &&
                (source_colorspace != LabColorspace) &&
-               (source_colorspace != CMYKColorspace) &&
-               (image->colorspace != RGBColorspace)))
+               (source_colorspace != XYZColorspace) &&
+               (source_colorspace != YCbCrColorspace) &&
+               (image->colorspace != RGBColorspace))
             ThrowProfileException(ImageError,"ColorspaceColorProfileMismatch",
               name);
           switch (image->rendering_intent)
@@ -967,39 +1117,72 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
           if (image->black_point_compensation != MagickFalse)
             flags|=cmsFLAGS_BLACKPOINTCOMPENSATION;
 #endif
-          transform=cmsCreateTransform(source_profile,source_type,
+          transform=AcquireTransformThreadSet(source_profile,source_type,
             target_profile,target_type,intent,flags);
           (void) cmsCloseProfile(source_profile);
-          (void) cmsCloseProfile(target_profile);
-          if (transform == (cmsHTRANSFORM) NULL)
+          if (transform == (cmsHTRANSFORM *) NULL)
             ThrowBinaryException(ImageError,"UnableToCreateColorTransform",
               name);
           /*
             Transform image as dictated by the source and target image profiles.
           */
           length=(size_t) image->columns;
-          source_pixels=(unsigned short *) AcquireQuantumMemory(length,
-            source_channels*sizeof(*source_pixels));
-          target_pixels=(unsigned short *) AcquireQuantumMemory(length,
-            target_channels*sizeof(*target_pixels));
-          if ((source_pixels == (unsigned short *) NULL) ||
-              (target_pixels == (unsigned short *) NULL))
+          source_pixels=AcquirePixelThreadSet(image->columns,source_channels);
+          target_pixels=AcquirePixelThreadSet(image->columns,target_channels);
+          if ((source_pixels == (unsigned short **) NULL) ||
+              (target_pixels == (unsigned short **) NULL))
             {
-              cmsDeleteTransform(transform);
+              transform=DestroyTransformThreadSet(transform);
+              (void) cmsCloseProfile(target_profile);
               ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
                 image->filename);
             }
           if (SetImageStorageClass(image,DirectClass) == MagickFalse)
-            return(MagickFalse);
+            {
+              target_pixels=DestroyPixelThreadSet(target_pixels);
+              source_pixels=DestroyPixelThreadSet(source_pixels);
+              transform=DestroyTransformThreadSet(transform);
+              (void) cmsCloseProfile(target_profile);
+              return(MagickFalse);
+            }
           if (target_colorspace == CMYKColorspace)
-            image->colorspace=target_colorspace;
+            (void) SetImageColorspace(image,target_colorspace);
+          status=MagickTrue;
+          progress=0;
+          image_view=AcquireCacheView(image);
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+          #pragma omp parallel for schedule(dynamic,4) shared(status)
+#endif
           for (y=0; y < (long) image->rows; y++)
           {
-            q=GetAuthenticPixels(image,0,y,image->columns,1,exception);
+            MagickBooleanType
+              sync;
+
+            register IndexPacket
+              *restrict indexes;
+
+            register long
+              id,
+              x;
+
+            register PixelPacket
+              *restrict q;
+
+            register unsigned short
+              *p;
+
+            if (status == MagickFalse)
+              continue;
+            q=GetCacheViewAuthenticPixels(image_view,0,y,image->columns,1,
+              exception);
             if (q == (PixelPacket *) NULL)
-              break;
-            p=source_pixels;
-            indexes=GetAuthenticIndexQueue(image);
+              {
+                status=MagickFalse;
+                continue;
+              }
+            indexes=GetCacheViewAuthenticIndexQueue(image_view);
+            id=GetOpenMPThreadId();
+            p=source_pixels[id];
             for (x=0; x < (long) image->columns; x++)
             {
               *p++=ScaleQuantumToShort(q->red);
@@ -1012,9 +1195,9 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
                 *p++=ScaleQuantumToShort(indexes[x]);
               q++;
             }
-            cmsDoTransform(transform,source_pixels,target_pixels,(unsigned int)
-              image->columns);
-            p=target_pixels;
+            cmsDoTransform(transform[id],source_pixels[id],target_pixels[id],
+              (unsigned int) image->columns);
+            p=target_pixels[id];
             q-=image->columns;
             for (x=0; x < (long) image->columns; x++)
             {
@@ -1036,10 +1219,25 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
                 }
               q++;
             }
-            if (SyncAuthenticPixels(image,exception) == MagickFalse)
-              break;
+            sync=SyncCacheViewAuthenticPixels(image_view,exception);
+            if (sync == MagickFalse)
+              status=MagickFalse;
+            if (image->progress_monitor != (MagickProgressMonitor) NULL)
+              {
+                MagickBooleanType
+                  proceed;
+
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp critical (MagickCore_ProfileImage)
+#endif
+                proceed=SetImageProgress(image,ProfileImageTag,progress++,
+                  image->rows);
+                if (proceed == MagickFalse)
+                  status=MagickFalse;
+              }
           }
-          image->colorspace=target_colorspace;
+          image_view=DestroyCacheView(image_view);
+          (void) SetImageColorspace(image,target_colorspace);
           switch (cmsGetColorSpace(target_profile))
           {
             case icSigRgbData:
@@ -1063,11 +1261,10 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
             default:
               break;
           }
-          target_pixels=(unsigned short *) RelinquishMagickMemory(
-            target_pixels);
-          source_pixels=(unsigned short *) RelinquishMagickMemory(
-            source_pixels);
-          cmsDeleteTransform(transform);
+          target_pixels=DestroyPixelThreadSet(target_pixels);
+          source_pixels=DestroyPixelThreadSet(source_pixels);
+          transform=DestroyTransformThreadSet(transform);
+          (void) cmsCloseProfile(target_profile);
         }
 #endif
     }
@@ -1266,7 +1463,7 @@ static MagickBooleanType GetProfilesFromResourceBlock(Image *image,
 
   datum=GetStringInfoDatum(resource_block);
   length=GetStringInfoLength(resource_block);
-  for (p=datum; p < (datum+length-7); )
+  for (p=datum; p < (datum+length-16); )
   {
     if (LocaleNCompare((char *) p,"8BIM",4) != 0)
       break;
@@ -1279,7 +1476,7 @@ static MagickBooleanType GetProfilesFromResourceBlock(Image *image,
     if (p > (datum+length-4))
       break;
     p=ReadResourceLong(p,&count);
-    if (p > (datum+length-count))
+    if ((p > (datum+length-count)) || (count > length))
       break;
     switch (id)
     {
@@ -1297,6 +1494,18 @@ static MagickBooleanType GetProfilesFromResourceBlock(Image *image,
         image->y_resolution=(double) resolution;
         break;
       }
+      case 0x0404:
+      {
+        /*
+          IPTC Profile
+        */
+        profile=AcquireStringInfo(count);
+        SetStringInfoDatum(profile,p);
+        (void) SetImageProfile(image,"iptc",profile);
+        profile=DestroyStringInfo(profile);
+        p+=count;
+        break;
+      }
       case 0x040c:
       {
         /*
@@ -1313,6 +1522,7 @@ static MagickBooleanType GetProfilesFromResourceBlock(Image *image,
         profile=AcquireStringInfo(count);
         SetStringInfoDatum(profile,p);
         (void) SetImageProfile(image,"icc",profile);
+        profile=DestroyStringInfo(profile);
         p+=count;
         break;
       }
@@ -1324,6 +1534,7 @@ static MagickBooleanType GetProfilesFromResourceBlock(Image *image,
         profile=AcquireStringInfo(count);
         SetStringInfoDatum(profile,p);
         (void) SetImageProfile(image,"exif",profile);
+        profile=DestroyStringInfo(profile);
         p+=count;
         break;
       }
@@ -1335,6 +1546,7 @@ static MagickBooleanType GetProfilesFromResourceBlock(Image *image,
         profile=AcquireStringInfo(count);
         SetStringInfoDatum(profile,p);
         (void) SetImageProfile(image,"xmp",profile);
+        profile=DestroyStringInfo(profile);
         p+=count;
         break;
       }
@@ -1492,14 +1704,14 @@ static inline void WriteProfileLong(const EndianType endian,
       buffer[1]=(unsigned char) (value >> 16);
       buffer[2]=(unsigned char) (value >> 8);
       buffer[3]=(unsigned char) value;
-      (void) memcpy(p,buffer,4);
+      (void) CopyMagickMemory(p,buffer,4);
       return;
     }
   buffer[0]=(unsigned char) value;
   buffer[1]=(unsigned char) (value >> 8);
   buffer[2]=(unsigned char) (value >> 16);
   buffer[3]=(unsigned char) (value >> 24);
-  (void) memcpy(p,buffer,4);
+  (void) CopyMagickMemory(p,buffer,4);
 }
 
 static void WriteProfileShort(const EndianType endian,
@@ -1512,12 +1724,12 @@ static void WriteProfileShort(const EndianType endian,
     {
       buffer[0]=(unsigned char) (value >> 8);
       buffer[1]=(unsigned char) value;
-      (void) memcpy(p,buffer,2);
+      (void) CopyMagickMemory(p,buffer,2);
       return;
     }
   buffer[0]=(unsigned char) value;
   buffer[1]=(unsigned char) (value >> 8);
-  (void) memcpy(p,buffer,2);
+  (void) CopyMagickMemory(p,buffer,2);
 }
 
 MagickExport MagickBooleanType SyncImageProfiles(Image *image)
@@ -1667,6 +1879,7 @@ MagickExport MagickBooleanType SyncImageProfiles(Image *image)
         {
           (void) WriteProfileLong(endian,(unsigned long)
             (image->x_resolution+0.5),p);
+          (void) WriteProfileLong(endian,1UL,p+4);
           break;
         }
         case 0x011b:
@@ -1680,7 +1893,6 @@ MagickExport MagickBooleanType SyncImageProfiles(Image *image)
         {
           (void) WriteProfileShort(endian,(unsigned short)
             image->orientation,p);
-          (void) WriteProfileLong(endian,1UL,p+4);
           break;
         }
         case 0x0128:
