@@ -17,7 +17,7 @@
 %                               September 2002                                %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -40,6 +40,7 @@
   Include declarations.
 */
 #include "magick/studio.h"
+#include "magick/cache.h"
 #include "magick/configure.h"
 #include "magick/exception.h"
 #include "magick/exception-private.h"
@@ -48,6 +49,7 @@
 #include "magick/image.h"
 #include "magick/memory_.h"
 #include "magick/option.h"
+#include "magick/policy.h"
 #include "magick/random_.h"
 #include "magick/registry.h"
 #include "magick/resource_.h"
@@ -55,6 +57,7 @@
 #include "magick/signature-private.h"
 #include "magick/string_.h"
 #include "magick/splay-tree.h"
+#include "magick/thread-private.h"
 #include "magick/token.h"
 #include "magick/utility.h"
 
@@ -68,14 +71,18 @@ typedef struct _ResourceInfo
     memory,
     map,
     disk,
-    file;
+    file,
+    thread,
+    time;
 
   MagickSizeType
     area_limit,
     memory_limit,
     map_limit,
     disk_limit,
-    file_limit;
+    file_limit,
+    thread_limit,
+    time_limit;
 } ResourceInfo;
 
 /*
@@ -92,11 +99,15 @@ static ResourceInfo
     MagickULLConstant(0),
     MagickULLConstant(0),
     MagickULLConstant(0),
+    MagickULLConstant(0),
+    MagickULLConstant(0),
     MagickULLConstant(2048)*1024*1024,
     MagickULLConstant(1536)*1024*1024,
     MagickULLConstant(8192)*1024*1024,
     MagickResourceInfinity,
-    MagickULLConstant(768)
+    MagickULLConstant(768),
+    MagickULLConstant(8),
+    MagickResourceInfinity
   };
 
 static SemaphoreInfo
@@ -148,7 +159,9 @@ MagickExport MagickBooleanType AcquireMagickResource(const ResourceType type,
 
   status=MagickFalse;
   (void) FormatMagickSize(size,resource_request);
-  AcquireSemaphoreInfo(&resource_semaphore);
+  if (resource_semaphore == (SemaphoreInfo *) NULL)
+    AcquireSemaphoreInfo(&resource_semaphore);
+  (void) LockSemaphoreInfo(resource_semaphore);
   switch (type)
   {
     case AreaResource:
@@ -213,10 +226,36 @@ MagickExport MagickBooleanType AcquireMagickResource(const ResourceType type,
         resource_limit);
       break;
     }
+    case ThreadResource:
+    {
+      resource_info.thread+=size;
+      limit=resource_info.thread_limit;
+      status=(resource_info.thread_limit == MagickResourceInfinity) ||
+        ((MagickSizeType) resource_info.thread < limit) ?
+        MagickTrue : MagickFalse;
+      (void) FormatMagickSize((MagickSizeType) resource_info.thread,
+        resource_current);
+      (void) FormatMagickSize((MagickSizeType) resource_info.thread_limit,
+        resource_limit);
+      break;
+    }
+    case TimeResource:
+    {
+      resource_info.time+=size;
+      limit=resource_info.time_limit;
+      status=(resource_info.time_limit == MagickResourceInfinity) ||
+        ((MagickSizeType) resource_info.time < limit) ?
+        MagickTrue : MagickFalse;
+      (void) FormatMagickSize((MagickSizeType) resource_info.time,
+        resource_current);
+      (void) FormatMagickSize((MagickSizeType) resource_info.time_limit,
+        resource_limit);
+      break;
+    }
     default:
       break;
   }
-  RelinquishSemaphoreInfo(resource_semaphore);
+  (void) UnlockSemaphoreInfo(resource_semaphore);
   (void) LogMagickEvent(ResourceEvent,GetMagickModule(),"%s: %s/%s/%s",
     MagickOptionToMnemonic(MagickResourceOptions,(long) type),resource_request,
     resource_current,resource_limit);
@@ -228,22 +267,22 @@ MagickExport MagickBooleanType AcquireMagickResource(const ResourceType type,
 %                                                                             %
 %                                                                             %
 %                                                                             %
-+   A s y n c h r o n o u s D e s t r o y M a g i c k R e s o u r c e s       %
++   A s y n c h r o n o u s R e s o u r c e C o m p o n e n t T e r m i n u s %
 %                                                                             %
 %                                                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  AsynchronousDestroyMagickResources() destroys the resource environment.
-%  It differs from DestroyMagickResources() in that it can be called from a
+%  AsynchronousResourceComponentTerminus() destroys the resource environment.
+%  It differs from ResourceComponentTerminus() in that it can be called from a
 %  asynchronous signal handler.
 %
-%  The format of the DestroyMagickResources() method is:
+%  The format of the ResourceComponentTerminus() method is:
 %
-%      DestroyMagickResources(void)
+%      ResourceComponentTerminus(void)
 %
 */
-MagickExport void AsynchronousDestroyMagickResources(void)
+MagickExport void AsynchronousResourceComponentTerminus(void)
 {
   const char
     *path;
@@ -298,6 +337,9 @@ static MagickBooleanType GetPathTemplate(char *path)
   char
     *directory;
 
+  ExceptionInfo
+    *exception;
+
   MagickBooleanType
     status;
 
@@ -308,24 +350,19 @@ static MagickBooleanType GetPathTemplate(char *path)
     attributes;
 
   (void) CopyMagickString(path,"magick-XXXXXXXX",MaxTextExtent);
-  directory=GetEnvironmentValue("MAGICK_TEMPORARY_PATH");
+  exception=AcquireExceptionInfo();
+  directory=(char *) GetImageRegistry(StringRegistryType,"temporary-path",
+    exception);
+  exception=DestroyExceptionInfo(exception);
   if (directory == (char *) NULL)
-    directory=GetConfigureOption("MAGICK_TEMPORARY_PATH");
-  if (directory == (char *) NULL)
-    {
-      ExceptionInfo
-        *exception;
-
-      exception=AcquireExceptionInfo();
-      directory=(char *) GetImageRegistry(StringRegistryType,"temporary-path",
-        exception);
-      exception=DestroyExceptionInfo(exception);
-    }
+    directory=GetEnvironmentValue("MAGICK_TEMPORARY_PATH");
   if (directory == (char *) NULL)
     directory=GetEnvironmentValue("MAGICK_TMPDIR");
+  if (directory == (char *) NULL)
+    directory=GetPolicyValue("temporary-path");
   if (directory == (char *) NULL)
     directory=GetEnvironmentValue("TMPDIR");
-#if defined(__WINDOWS__)
+#if defined(__WINDOWS__) || defined(__OS2__)
   if (directory == (char *) NULL)
     directory=GetEnvironmentValue("TMP");
   if (directory == (char *) NULL)
@@ -410,6 +447,9 @@ MagickExport int AcquireUniqueFileResource(char *path)
     (void) GetPathTemplate(path);
 #if defined(MAGICKCORE_HAVE_MKSTEMP)
     file=mkstemp(path);
+#if defined(__OS2__)
+    setmode(file,O_BINARY);
+#endif
     if (file != -1)
       break;
 #endif
@@ -429,11 +469,13 @@ MagickExport int AcquireUniqueFileResource(char *path)
   (void) LogMagickEvent(ResourceEvent,GetMagickModule(),"%s",path);
   if (file == -1)
     return(file);
-  AcquireSemaphoreInfo(&resource_semaphore);
+  if (resource_semaphore == (SemaphoreInfo *) NULL)
+    AcquireSemaphoreInfo(&resource_semaphore);
+  (void) LockSemaphoreInfo(resource_semaphore);
   if (temporary_resources == (SplayTreeInfo *) NULL)
     temporary_resources=NewSplayTree(CompareSplayTreeString,
       RelinquishMagickMemory,DestroyTemporaryResources);
-  RelinquishSemaphoreInfo(resource_semaphore);
+  (void) UnlockSemaphoreInfo(resource_semaphore);
   resource=ConstantString(path);
   (void) AddValueToSplayTree(temporary_resources,resource,resource);
   return(file);
@@ -444,35 +486,6 @@ MagickExport int AcquireUniqueFileResource(char *path)
 %                                                                             %
 %                                                                             %
 %                                                                             %
-+   D e s t r o y M a g i c k R e s o u r c e s                               %
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%
-%  DestroyMagickResources() destroys the resource environment.
-%
-%  The format of the DestroyMagickResources() method is:
-%
-%      DestroyMagickResources(void)
-%
-*/
-MagickExport void DestroyMagickResources(void)
-{
-  AcquireSemaphoreInfo(&resource_semaphore);
-  if (temporary_resources != (SplayTreeInfo *) NULL)
-    temporary_resources=DestroySplayTree(temporary_resources);
-  if (random_info != (RandomInfo *) NULL)
-    random_info=DestroyRandomInfo(random_info);
-  RelinquishSemaphoreInfo(resource_semaphore);
-  DestroySemaphoreInfo(&resource_semaphore);
-}
-
-/*
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%                                                                             %
-%                                                                             %
-%                                                                             %
 %   G e t M a g i c k R e s o u r c e                                         %
 %                                                                             %
 %                                                                             %
@@ -496,7 +509,7 @@ MagickExport MagickSizeType GetMagickResource(const ResourceType type)
     resource;
 
   resource=0;
-  AcquireSemaphoreInfo(&resource_semaphore);
+  (void) LockSemaphoreInfo(resource_semaphore);
   switch (type)
   {
     case AreaResource:
@@ -524,10 +537,20 @@ MagickExport MagickSizeType GetMagickResource(const ResourceType type)
       resource=(MagickSizeType) resource_info.file;
       break;
     }
+    case ThreadResource:
+    {
+      resource=(MagickSizeType) resource_info.thread;
+      break;
+    }
+    case TimeResource:
+    {
+      resource=(MagickSizeType) resource_info.time;
+      break;
+    }
     default:
       break;
   }
-  RelinquishSemaphoreInfo(resource_semaphore);
+  (void) UnlockSemaphoreInfo(resource_semaphore);
   return(resource);
 }
 
@@ -559,7 +582,9 @@ MagickExport MagickSizeType GetMagickResourceLimit(const ResourceType type)
     resource;
 
   resource=0;
-  AcquireSemaphoreInfo(&resource_semaphore);
+  if (resource_semaphore == (SemaphoreInfo *) NULL)
+    AcquireSemaphoreInfo(&resource_semaphore);
+  (void) LockSemaphoreInfo(resource_semaphore);
   switch (type)
   {
     case AreaResource:
@@ -587,10 +612,20 @@ MagickExport MagickSizeType GetMagickResourceLimit(const ResourceType type)
       resource=resource_info.file_limit;
       break;
     }
+    case ThreadResource:
+    {
+      resource=resource_info.thread_limit;
+      break;
+    }
+    case TimeResource:
+    {
+      resource=resource_info.time_limit;
+      break;
+    }
     default:
       break;
   }
-  RelinquishSemaphoreInfo(resource_semaphore);
+  (void) UnlockSemaphoreInfo(resource_semaphore);
   return(resource);
 }
 
@@ -599,131 +634,6 @@ MagickExport MagickSizeType GetMagickResourceLimit(const ResourceType type)
 %                                                                             %
 %                                                                             %
 %                                                                             %
-+   I n i t i a l i z e M a g i c k R e s o u r c e s                         %
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%
-%  InitializeMagickResources() initializes the resource environment.
-%
-%  The format of the InitializeMagickResources() method is:
-%
-%      InitializeMagickResources(void)
-%
-*/
-
-static inline unsigned long MagickMax(const unsigned long x,
-  const unsigned long y)
-{
-  if (x > y)
-    return(x);
-  return(y);
-}
-
-MagickExport void InitializeMagickResources(void)
-{
-  char
-    *limit;
-
-  long
-    files,
-    pages,
-    pagesize;
-
-  MagickSizeType
-    memory;
-
-  /*
-    Set Magick resource limits.
-  */
-  pagesize=(-1);
-#if defined(MAGICKCORE_HAVE_SYSCONF) && defined(_SC_PAGESIZE)
-  pagesize=sysconf(_SC_PAGESIZE);
-#elif defined(MAGICKCORE_HAVE_GETPAGESIZE) && defined(MAGICKCORE_POSIX_SUPPORT)
-  pagesize=getpagesize();
-#endif
-  pages=(-1);
-#if defined(MAGICKCORE_HAVE_SYSCONF) && defined(_SC_PHYS_PAGES)
-  pages=sysconf(_SC_PHYS_PAGES);
-#endif
-  memory=(MagickSizeType) pages*pagesize;
-  if ((pagesize <= 0) || (pages <= 0))
-    memory=2048UL*1024UL*1024UL;
-#if defined(PixelCacheThreshold)
-  memory=PixelCacheThreshold;
-#endif
-  (void) SetMagickResourceLimit(AreaResource,2UL*memory);
-  (void) SetMagickResourceLimit(MemoryResource,3UL*memory/2UL);
-  (void) SetMagickResourceLimit(MapResource,4UL*memory);
-  limit=GetEnvironmentValue("MAGICK_AREA_LIMIT");
-  if (limit == (char *) NULL)
-    limit=GetConfigureOption("MAGICK_AREA_LIMIT");
-  if (limit != (char *) NULL)
-    {
-      (void) SetMagickResourceLimit(AreaResource,(MagickSizeType)
-        StringToDouble(limit,100.0));
-      limit=DestroyString(limit);
-    }
-  limit=GetEnvironmentValue("MAGICK_MEMORY_LIMIT");
-  if (limit == (char *) NULL)
-    limit=GetConfigureOption("MAGICK_MEMORY_LIMIT");
-  if (limit != (char *) NULL)
-    {
-      (void) SetMagickResourceLimit(MemoryResource,(MagickSizeType)
-        StringToDouble(limit,100.0));
-      limit=DestroyString(limit);
-    }
-  limit=GetEnvironmentValue("MAGICK_MAP_LIMIT");
-  if (limit == (char *) NULL)
-    limit=GetConfigureOption("MAGICK_MAP_LIMIT");
-  if (limit != (char *) NULL)
-    {
-      (void) SetMagickResourceLimit(MapResource,(MagickSizeType)
-        StringToDouble(limit,100.0));
-      limit=DestroyString(limit);
-    }
-  limit=GetEnvironmentValue("MAGICK_DISK_LIMIT");
-  if (limit == (char *) NULL)
-    limit=GetConfigureOption("MAGICK_DISK_LIMIT");
-  if (limit != (char *) NULL)
-    {
-      (void) SetMagickResourceLimit(DiskResource,(MagickSizeType)
-        StringToDouble(limit,100.0));
-      limit=DestroyString(limit);
-    }
-  files=(-1);
-#if defined(MAGICKCORE_HAVE_SYSCONF) && defined(_SC_OPEN_MAX)
-  files=sysconf(_SC_OPEN_MAX);
-#elif defined(MAGICKCORE_HAVE_GETDTABLESIZE) && defined(MAGICKCORE_POSIX_SUPPORT)
-  files=getdtablesize();
-#endif
-  (void) SetMagickResourceLimit(FileResource,MagickMax((unsigned long)
-    (3*files/4),64));
-  limit=GetEnvironmentValue("MAGICK_FILE_LIMIT");
-  if (limit == (char *) NULL)
-    limit=GetConfigureOption("MAGICK_FILE_LIMIT");
-  if (limit != (char *) NULL)
-    {
-      (void) SetMagickResourceLimit(FileResource,(MagickSizeType)
-        StringToDouble(limit,100.0));
-      limit=DestroyString(limit);
-    }
-  limit=GetEnvironmentValue("MAGICK_THREAD_LIMIT");
-  if (limit == (char *) NULL)
-    limit=GetConfigureOption("MAGICK_THREAD_LIMIT");
-  if (limit != (char *) NULL)
-    {
-      SetCacheViewMaximumThreads((unsigned long) atol(limit));
-      limit=DestroyString(limit);
-    }
-}
-
-/*
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%                                                                             %
-%                                                                             %
-%                                                                             %
 %  L i s t M a g i c k R e s o u r c e I n f o                                %
 %                                                                             %
 %                                                                             %
@@ -751,21 +661,31 @@ MagickExport MagickBooleanType ListMagickResourceInfo(FILE *file,
     area_limit[MaxTextExtent],
     disk_limit[MaxTextExtent],
     map_limit[MaxTextExtent],
-    memory_limit[MaxTextExtent];
+    memory_limit[MaxTextExtent],
+    time_limit[MaxTextExtent];
 
   if (file == (const FILE *) NULL)
     file=stdout;
-  AcquireSemaphoreInfo(&resource_semaphore);
+  if (resource_semaphore == (SemaphoreInfo *) NULL)
+    AcquireSemaphoreInfo(&resource_semaphore);
+  (void) LockSemaphoreInfo(resource_semaphore);
   (void) FormatMagickSize(resource_info.area_limit,area_limit);
-  (void) FormatMagickSize(resource_info.disk_limit,disk_limit);
-  (void) FormatMagickSize(resource_info.map_limit,map_limit);
   (void) FormatMagickSize(resource_info.memory_limit,memory_limit);
-  (void) fprintf(file,"File       Area     Memory        Map       Disk\n");
-  (void) fprintf(file,"------------------------------------------------\n");
-  (void) fprintf(file,"%4lu  %9s  %9s  %9s  %9s\n",(unsigned long)
-    resource_info.file_limit,area_limit,memory_limit,map_limit,disk_limit);
+  (void) FormatMagickSize(resource_info.map_limit,map_limit);
+  (void) FormatMagickSize(resource_info.disk_limit,disk_limit);
+  (void) CopyMagickString(time_limit,"unlimited",MaxTextExtent);
+  if (resource_info.time_limit != MagickResourceInfinity)
+    (void) FormatMagickString(time_limit,MaxTextExtent,"%lu",(unsigned long)
+      resource_info.time_limit);
+  (void) fprintf(file,"File        Area      Memory         Map"
+    "        Disk  Thread        Time\n");
+  (void) fprintf(file,"------------------------------------------------------"
+    "------------------\n");
+  (void) fprintf(file,"%4lu  %10s  %10s  %10s  %10s  %6lu  %10s\n",
+    (unsigned long) resource_info.file_limit,area_limit,memory_limit,map_limit,
+    disk_limit,(unsigned long) resource_info.thread_limit,time_limit);
   (void) fflush(file);
-  RelinquishSemaphoreInfo(resource_semaphore);
+  (void) UnlockSemaphoreInfo(resource_semaphore);
   return(MagickTrue);
 }
 
@@ -803,7 +723,9 @@ MagickExport void RelinquishMagickResource(const ResourceType type,
     resource_request[MaxTextExtent];
 
   (void) FormatMagickSize(size,resource_request);
-  AcquireSemaphoreInfo(&resource_semaphore);
+  if (resource_semaphore == (SemaphoreInfo *) NULL)
+    AcquireSemaphoreInfo(&resource_semaphore);
+  (void) LockSemaphoreInfo(resource_semaphore);
   switch (type)
   {
     case AreaResource:
@@ -847,10 +769,28 @@ MagickExport void RelinquishMagickResource(const ResourceType type,
         resource_limit);
       break;
     }
+    case ThreadResource:
+    {
+      resource_info.thread-=size;
+      (void) FormatMagickSize((MagickSizeType) resource_info.thread,
+        resource_current);
+      (void) FormatMagickSize((MagickSizeType) resource_info.thread_limit,
+        resource_limit);
+      break;
+    }
+    case TimeResource:
+    {
+      resource_info.time-=size;
+      (void) FormatMagickSize((MagickSizeType) resource_info.time,
+        resource_current);
+      (void) FormatMagickSize((MagickSizeType) resource_info.time_limit,
+        resource_limit);
+      break;
+    }
     default:
       break;
   }
-  RelinquishSemaphoreInfo(resource_semaphore);
+  (void) UnlockSemaphoreInfo(resource_semaphore);
   (void) LogMagickEvent(ResourceEvent,GetMagickModule(),"%s: %s/%s/%s",
     MagickOptionToMnemonic(MagickResourceOptions,(long) type),resource_request,
     resource_current,resource_limit);
@@ -912,6 +852,192 @@ MagickExport MagickBooleanType RelinquishUniqueFileResource(const char *path)
 %                                                                             %
 %                                                                             %
 %                                                                             %
++   R e s o u r c e C o m p o n e n t G e n e s i s                           %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  ResourceComponentGenesis() instantiates the resource component.
+%
+%  The format of the ResourceComponentGenesis method is:
+%
+%      MagickBooleanType ResourceComponentGenesis(void)
+%
+*/
+
+static inline unsigned long MagickMax(const unsigned long x,
+  const unsigned long y)
+{
+  if (x > y)
+    return(x);
+  return(y);
+}
+
+static inline MagickSizeType StringToSizeType(const char *string,
+  const double interval)
+{
+  double
+    value;
+
+  value=StringToDouble(string,interval);
+  if (value >= (double) MagickULLConstant(~0))
+    return(MagickULLConstant(~0));
+  return((MagickSizeType) value);
+}
+
+MagickExport MagickBooleanType ResourceComponentGenesis(void)
+{
+  char
+    *limit;
+
+  long
+    files,
+    pages,
+    pagesize;
+
+  MagickSizeType
+    memory;
+
+  /*
+    Set Magick resource limits.
+  */
+  AcquireSemaphoreInfo(&resource_semaphore);
+  pagesize=GetMagickPageSize();
+  pages=(-1);
+#if defined(MAGICKCORE_HAVE_SYSCONF) && defined(_SC_PHYS_PAGES)
+  pages=sysconf(_SC_PHYS_PAGES);
+#endif
+  memory=(MagickSizeType) pages*pagesize;
+  if ((pagesize <= 0) || (pages <= 0))
+    memory=2048UL*1024UL*1024UL;
+#if defined(PixelCacheThreshold)
+  memory=PixelCacheThreshold;
+#endif
+  (void) SetMagickResourceLimit(AreaResource,2UL*memory);
+  (void) SetMagickResourceLimit(MemoryResource,3UL*memory/2UL);
+  (void) SetMagickResourceLimit(MapResource,4UL*memory);
+  limit=GetEnvironmentValue("MAGICK_AREA_LIMIT");
+  if (limit == (char *) NULL)
+    limit=GetPolicyValue("area");
+  if (limit != (char *) NULL)
+    {
+      (void) SetMagickResourceLimit(AreaResource,StringToSizeType(limit,100.0));
+      limit=DestroyString(limit);
+    }
+  limit=GetEnvironmentValue("MAGICK_MEMORY_LIMIT");
+  if (limit == (char *) NULL)
+    limit=GetPolicyValue("memory");
+  if (limit != (char *) NULL)
+    {
+      (void) SetMagickResourceLimit(MemoryResource,
+        StringToSizeType(limit,100.0));
+      limit=DestroyString(limit);
+    }
+  limit=GetEnvironmentValue("MAGICK_MAP_LIMIT");
+  if (limit == (char *) NULL)
+    limit=GetPolicyValue("map");
+  if (limit != (char *) NULL)
+    {
+      (void) SetMagickResourceLimit(MapResource,StringToSizeType(limit,100.0));
+      limit=DestroyString(limit);
+    }
+  limit=GetEnvironmentValue("MAGICK_DISK_LIMIT");
+  if (limit == (char *) NULL)
+    limit=GetPolicyValue("disk");
+  if (limit != (char *) NULL)
+    {
+      (void) SetMagickResourceLimit(DiskResource,StringToSizeType(limit,100.0));
+      limit=DestroyString(limit);
+    }
+  files=(-1);
+#if defined(MAGICKCORE_HAVE_SYSCONF) && defined(_SC_OPEN_MAX)
+  files=sysconf(_SC_OPEN_MAX);
+#endif
+#if defined(MAGICKCORE_HAVE_GETRLIMIT) && defined(RLIMIT_NOFILE)
+  if (files < 0)
+    {
+      struct rlimit
+        resources;
+
+      if (getrlimit(RLIMIT_NOFILE,&resources) != -1)
+        files=resources.rlim_cur;
+  }
+#endif
+#if defined(MAGICKCORE_HAVE_GETDTABLESIZE) && defined(MAGICKCORE_POSIX_SUPPORT)
+  if (files < 0)
+    files=getdtablesize();
+#endif
+  if (files < 0)
+    files=64;
+  (void) SetMagickResourceLimit(FileResource,MagickMax((unsigned long)
+    (3*files/4),64));
+  limit=GetEnvironmentValue("MAGICK_FILE_LIMIT");
+  if (limit == (char *) NULL)
+    limit=GetPolicyValue("file");
+  if (limit != (char *) NULL)
+    {
+      (void) SetMagickResourceLimit(FileResource,StringToSizeType(limit,100.0));
+      limit=DestroyString(limit);
+    }
+  (void) SetMagickResourceLimit(ThreadResource,GetOpenMPMaximumThreads());
+  limit=GetEnvironmentValue("MAGICK_THREAD_LIMIT");
+  if (limit == (char *) NULL)
+    limit=GetPolicyValue("thread");
+  if (limit != (char *) NULL)
+    {
+      SetOpenMPMaximumThreads((unsigned long) atol(limit));
+      (void) SetMagickResourceLimit(ThreadResource,StringToSizeType(limit,
+        100.0));
+      limit=DestroyString(limit);
+    }
+  limit=GetEnvironmentValue("MAGICK_TIME_LIMIT");
+  if (limit == (char *) NULL)
+    limit=GetPolicyValue("time");
+  if (limit != (char *) NULL)
+    {
+      (void) SetMagickResourceLimit(TimeResource,StringToSizeType(limit,100.0));
+      limit=DestroyString(limit);
+    }
+  return(MagickTrue);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
++   R e s o u r c e C o m p o n e n t T e r m i n u s                         %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  ResourceComponentTerminus() destroys the resource component.
+%
+%  The format of the ResourceComponentTerminus() method is:
+%
+%      ResourceComponentTerminus(void)
+%
+*/
+MagickExport void ResourceComponentTerminus(void)
+{
+  if (resource_semaphore == (SemaphoreInfo *) NULL)
+    AcquireSemaphoreInfo(&resource_semaphore);
+  (void) LockSemaphoreInfo(resource_semaphore);
+  if (temporary_resources != (SplayTreeInfo *) NULL)
+    temporary_resources=DestroySplayTree(temporary_resources);
+  if (random_info != (RandomInfo *) NULL)
+    random_info=DestroyRandomInfo(random_info);
+  (void) UnlockSemaphoreInfo(resource_semaphore);
+  DestroySemaphoreInfo(&resource_semaphore);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   S e t M a g i c k R e s o u r c e L i m i t                               %
 %                                                                             %
 %                                                                             %
@@ -935,7 +1061,9 @@ MagickExport MagickBooleanType RelinquishUniqueFileResource(const char *path)
 MagickExport MagickBooleanType SetMagickResourceLimit(const ResourceType type,
   const MagickSizeType limit)
 {
-  AcquireSemaphoreInfo(&resource_semaphore);
+  if (resource_semaphore == (SemaphoreInfo *) NULL)
+    AcquireSemaphoreInfo(&resource_semaphore);
+  (void) LockSemaphoreInfo(resource_semaphore);
   switch (type)
   {
     case AreaResource:
@@ -963,9 +1091,20 @@ MagickExport MagickBooleanType SetMagickResourceLimit(const ResourceType type,
       resource_info.file_limit=limit;
       break;
     }
+    case ThreadResource:
+    {
+      SetOpenMPMaximumThreads((unsigned long) limit);
+      resource_info.thread_limit=GetOpenMPMaximumThreads();
+      break;
+    }
+    case TimeResource:
+    {
+      resource_info.time_limit=limit;
+      break;
+    }
     default:
       break;
   }
-  RelinquishSemaphoreInfo(resource_semaphore);
+  (void) UnlockSemaphoreInfo(resource_semaphore);
   return(MagickTrue);
 }
