@@ -17,7 +17,7 @@
 %                                April 1993                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -255,45 +255,29 @@ static MagickBooleanType Classify(Image *image,short **extrema,
     green,
     red;
 
-  IndexPacket
-    index;
-
   long
     count,
-    j,
-    k,
+    progress,
     y;
 
-  MagickBooleanType
-    proceed;
-
   MagickRealType
-    distance_squared,
-    *free_squares,
-    local_minima,
-    numerator,
-    ratio,
-    sum;
+    *free_squares;
 
-  register const PixelPacket
-    *p;
-
-  register IndexPacket
-    *indexes;
+  MagickStatusType
+    status;
 
   register long
-    i,
-    x;
+    i;
 
   register MagickRealType
     *squares;
 
-  register PixelPacket
-    *q;
-
   unsigned long
     number_clusters;
 
+  CacheView
+    *image_view;
+
   /*
     Form clusters.
   */
@@ -360,11 +344,20 @@ static MagickBooleanType Classify(Image *image,short **extrema,
   /*
     Count the pixels for each cluster.
   */
+  status=MagickTrue;
   count=0;
+  progress=0;
   exception=(&image->exception);
+  image_view=AcquireCacheView(image);
   for (y=0; y < (long) image->rows; y++)
   {
-    p=GetVirtualPixels(image,0,y,image->columns,1,exception);
+    register const PixelPacket
+      *p;
+
+    register long
+      x;
+
+    p=GetCacheViewVirtualPixels(image_view,0,y,image->columns,1,exception);
     if (p == (const PixelPacket *) NULL)
       break;
     for (x=0; x < (long) image->columns; x++)
@@ -396,10 +389,21 @@ static MagickBooleanType Classify(Image *image,short **extrema,
           }
       p++;
     }
-    proceed=SetImageProgress(image,SegmentImageTag,y,2*image->rows);
-    if (proceed == MagickFalse)
-      break;
+    if (image->progress_monitor != (MagickProgressMonitor) NULL)
+      {
+        MagickBooleanType
+          proceed;
+
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+        #pragma omp critical (MagickCore_Classify)
+#endif
+        proceed=SetImageProgress(image,SegmentImageTag,progress++,
+          2*image->rows);
+        if (proceed == MagickFalse)
+          status=MagickFalse;
+      }
   }
+  image_view=DestroyCacheView(image_view);
   /*
     Remove clusters that do not meet minimum cluster threshold.
   */
@@ -440,6 +444,10 @@ static MagickBooleanType Classify(Image *image,short **extrema,
       */
       (void) fprintf(stdout,"Fuzzy C-means Statistics\n");
       (void) fprintf(stdout,"===================\n\n");
+      (void) fprintf(stdout,"\tCluster Threshold = %g\n",(double)
+        cluster_threshold);
+      (void) fprintf(stdout,"\tWeighting Exponent = %g\n",(double)
+        weighting_exponent);
       (void) fprintf(stdout,"\tTotal Number of Clusters = %lu\n\n",
         number_clusters);
       /*
@@ -495,32 +503,55 @@ static MagickBooleanType Classify(Image *image,short **extrema,
   if (AcquireImageColormap(image,number_clusters) == MagickFalse)
     ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
       image->filename);
-  cluster=head;
-  for (i=0; i < (long) number_clusters; i++)
+  i=0;
+  for (cluster=head; cluster != (Cluster *) NULL; cluster=cluster->next)
   {
     image->colormap[i].red=ScaleCharToQuantum((unsigned char)
       (cluster->red.center+0.5));
     image->colormap[i].green=ScaleCharToQuantum((unsigned char)
       (cluster->green.center+0.5));
     image->colormap[i].blue=ScaleCharToQuantum((unsigned char)
-       (cluster->blue.center+0.5));
-    cluster=cluster->next;
+      (cluster->blue.center+0.5));
+    i++;
   }
   /*
     Do course grain classes.
   */
   exception=(&image->exception);
+  image_view=AcquireCacheView(image);
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
+#endif
   for (y=0; y < (long) image->rows; y++)
   {
-    q=GetAuthenticPixels(image,0,y,image->columns,1,exception);
+    Cluster
+      *cluster;
+
+    register const PixelPacket
+      *restrict p;
+
+    register IndexPacket
+      *restrict indexes;
+
+    register long
+      x;
+
+    register PixelPacket
+      *restrict q;
+
+    if (status == MagickFalse)
+      continue;
+    q=GetCacheViewAuthenticPixels(image_view,0,y,image->columns,1,exception);
     if (q == (PixelPacket *) NULL)
-      break;
-    indexes=GetAuthenticIndexQueue(image);
+      {
+        status=MagickFalse;
+        continue;
+      }
+    indexes=GetCacheViewAuthenticIndexQueue(image_view);
     for (x=0; x < (long) image->columns; x++)
     {
-      index=(IndexPacket) 0;
-      cluster=head;
-      for (i=0; i < (long) number_clusters; i++)
+      indexes[x]=(IndexPacket) 0;
+      for (cluster=head; cluster != (Cluster *) NULL; cluster=cluster->next)
       {
         if (((long) ScaleQuantumToChar(q->red) >=
              (cluster->red.left-SafeMargin)) &&
@@ -538,13 +569,23 @@ static MagickBooleanType Classify(Image *image,short **extrema,
             /*
               Classify this pixel.
             */
-            index=(IndexPacket) cluster->id;
+            indexes[x]=(IndexPacket) cluster->id;
             break;
           }
-        cluster=cluster->next;
       }
       if (cluster == (Cluster *) NULL)
         {
+          MagickRealType
+            distance_squared,
+            local_minima,
+            numerator,
+            ratio,
+            sum;
+
+          register long
+            j,
+            k;
+
           /*
             Compute fuzzy membership.
           */
@@ -578,20 +619,30 @@ static MagickBooleanType Classify(Image *image,short **extrema,
                   Classify this pixel.
                 */
                 local_minima=1.0/sum;
-                index=(IndexPacket) j;
+                indexes[x]=(IndexPacket) j;
               }
           }
         }
-      indexes[x]=index;
-      *q=image->colormap[(long) index];
       q++;
     }
-    if (SyncAuthenticPixels(image,exception) == MagickFalse)
-      break;
-    proceed=SetImageProgress(image,SegmentImageTag,y,2*image->rows);
-    if (proceed == MagickFalse)
-      break;
+    if (SyncCacheViewAuthenticPixels(image_view,exception) == MagickFalse)
+      status=MagickFalse;
+    if (image->progress_monitor != (MagickProgressMonitor) NULL)
+      {
+        MagickBooleanType
+          proceed;
+
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+        #pragma omp critical (MagickCore_Classify)
+#endif
+        proceed=SetImageProgress(image,SegmentImageTag,progress++,
+          2*image->rows);
+        if (proceed == MagickFalse)
+          status=MagickFalse;
+      }
   }
+  image_view=DestroyCacheView(image_view);
+  status&=SyncImage(image);
   /*
     Relinquish resources.
   */
@@ -1765,7 +1816,7 @@ MagickExport MagickBooleanType SegmentImage(Image *image,
       }
   }
   if (colorspace != RGBColorspace)
-    (void) SetImageColorspace(image,colorspace);
+    (void) TransformImageColorspace(image,colorspace);
   /*
     Initialize histogram.
   */
@@ -1781,7 +1832,7 @@ MagickExport MagickBooleanType SegmentImage(Image *image,
   */
   status=Classify(image,extrema,cluster_threshold,WeightingExponent,verbose);
   if (colorspace != RGBColorspace)
-    (void) SetImageColorspace(image,colorspace);
+    (void) TransformImageColorspace(image,colorspace);
   /*
     Relinquish resources.
   */
