@@ -47,6 +47,7 @@
 */
 #include "magick/studio.h"
 #include "magick/artifact.h"
+#include "magick/attribute.h"
 #include "magick/blob-private.h"
 #include "magick/cache-private.h"
 #include "magick/color-private.h"
@@ -562,7 +563,7 @@ static inline MagickBooleanType SetRadonCell(const RadonInfo *radon_info,
       return(MagickTrue);
     }
   count=WriteRadonCell(radon_info,i*sizeof(*radon_info->cells),
-    sizeof(*radon_info->cells),(unsigned char *) &value);
+    sizeof(*radon_info->cells),(const unsigned char *) &value);
   if (count != (ssize_t) sizeof(*radon_info->cells))
     return(MagickFalse);
   return(MagickTrue);
@@ -626,7 +627,7 @@ static void RadonProjection(RadonInfo *source_cells,
     q=swap;
   }
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for
+  #pragma omp parallel for schedule(dynamic,4)
 #endif
   for (x=0; x < (long) p->width; x++)
   {
@@ -652,6 +653,9 @@ static void RadonProjection(RadonInfo *source_cells,
 static MagickBooleanType RadonTransform(const Image *image,
   const double threshold,unsigned long *projection,ExceptionInfo *exception)
 {
+  CacheView
+    *image_view;
+
   long
     y;
 
@@ -675,9 +679,6 @@ static MagickBooleanType RadonTransform(const Image *image,
   unsigned short
     bits[256];
 
-  CacheView
-    *image_view;
-
   for (width=1; width < ((image->columns+7)/8); width<<=1) ;
   source_cells=AcquireRadonInfo(image,width,image->rows,exception);
   destination_cells=AcquireRadonInfo(image,width,image->rows,exception);
@@ -711,7 +712,7 @@ static MagickBooleanType RadonTransform(const Image *image,
   for (y=0; y < (long) image->rows; y++)
   {
     register const PixelPacket
-      *__restrict p;
+      *restrict p;
 
     register long
       i,
@@ -762,7 +763,7 @@ static MagickBooleanType RadonTransform(const Image *image,
   for (y=0; y < (long) image->rows; y++)
   {
     register const PixelPacket
-      *__restrict p;
+      *restrict p;
 
     register long
       i,
@@ -815,6 +816,9 @@ static MagickBooleanType RadonTransform(const Image *image,
 static void GetImageBackgroundColor(Image *image,const long offset,
   ExceptionInfo *exception)
 {
+  CacheView
+    *image_view;
+
   long
     y;
 
@@ -824,9 +828,6 @@ static void GetImageBackgroundColor(Image *image,const long offset,
   MagickRealType
     count;
 
-  CacheView
-    *image_view;
-
   /*
     Compute average background color.
   */
@@ -838,7 +839,7 @@ static void GetImageBackgroundColor(Image *image,const long offset,
   for (y=0; y < (long) image->rows; y++)
   {
     register const PixelPacket
-      *__restrict p;
+      *restrict p;
 
     register long
       x;
@@ -1009,10 +1010,12 @@ MagickExport Image *DeskewImage(const Image *image,const double threshold,
 static Image *IntegralRotateImage(const Image *image,unsigned long rotations,
   ExceptionInfo *exception)
 {
-#define TileHeight  128
-#define TileWidth  128
 #define RotateImageTag  "Rotate/Image"
 
+  CacheView
+    *image_view,
+    *rotate_view;
+
   Image
     *rotate_image;
 
@@ -1026,16 +1029,14 @@ static Image *IntegralRotateImage(const Image *image,unsigned long rotations,
   RectangleInfo
     page;
 
-  CacheView
-    *image_view,
-    *rotate_view;
-
   /*
     Initialize rotated image attributes.
   */
   assert(image != (Image *) NULL);
   page=image->page;
   rotations%=4;
+  if (rotations == 0)
+    return(CloneImage(image,0,0,MagickTrue,exception));
   if ((rotations == 1) || (rotations == 3))
     rotate_image=CloneImage(image,image->rows,image->columns,MagickTrue,
       exception);
@@ -1058,61 +1059,6 @@ static Image *IntegralRotateImage(const Image *image,unsigned long rotations,
       /*
         Rotate 0 degrees.
       */
-#if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(dynamic,8) shared(progress,status)
-#endif
-      for (y=0; y < (long) image->rows; y++)
-      {
-        MagickBooleanType
-          sync;
-
-        register const IndexPacket
-          *__restrict indexes;
-
-        register const PixelPacket
-          *__restrict p;
-
-        register IndexPacket
-          *__restrict rotate_indexes;
-
-        register PixelPacket
-          *__restrict q;
-
-        if (status == MagickFalse)
-          continue;
-        p=GetCacheViewVirtualPixels(image_view,0,y,image->columns,1,
-          exception);
-        q=QueueCacheViewAuthenticPixels(rotate_view,0,y,
-          rotate_image->columns,1,exception);
-        if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
-          {
-            status=MagickFalse;
-            continue;
-          }
-        indexes=GetCacheViewVirtualIndexQueue(image_view);
-        rotate_indexes=GetCacheViewAuthenticIndexQueue(rotate_view);
-        (void) CopyMagickMemory(q,p,image->columns*sizeof(*p));
-        if ((indexes != (IndexPacket *) NULL) &&
-            (rotate_indexes != (IndexPacket *) NULL))
-          (void) CopyMagickMemory(rotate_indexes,indexes,image->columns*
-            sizeof(*indexes));
-        sync=SyncCacheViewAuthenticPixels(rotate_view,exception);
-        if (sync == MagickFalse)
-          status=MagickFalse;
-        if (image->progress_monitor != (MagickProgressMonitor) NULL)
-          {
-            MagickBooleanType
-              proceed;
-
-#if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp critical (MagickCore_IntegralRotateImage)
-#endif
-            proceed=SetImageProgress(image,RotateImageTag,progress++,
-              image->rows);
-            if (proceed == MagickFalse)
-              status=MagickFalse;
-          }
-      }
       break;
     }
     case 1:
@@ -1120,93 +1066,98 @@ static Image *IntegralRotateImage(const Image *image,unsigned long rotations,
       long
         tile_y;
 
+      unsigned long
+        tile_height,
+        tile_width;
+
       /*
         Rotate 90 degrees.
       */
+      GetPixelCacheTileSize(image,&tile_width,&tile_height);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static,1) shared(progress,status)
+  #pragma omp parallel for schedule(dynamic,4) shared(progress, status)
 #endif
-      for (tile_y=0; tile_y < (long) image->rows; tile_y+=TileHeight)
+      for (tile_y=0; tile_y < (long) image->rows; tile_y+=tile_height)
       {
         register long
           tile_x;
 
         if (status == MagickFalse)
           continue;
-        for (tile_x=0; tile_x < (long) image->columns; tile_x+=TileWidth)
+        for (tile_x=0; tile_x < (long) image->columns; tile_x+=tile_width)
         {
           MagickBooleanType
             sync;
 
           register const IndexPacket
-            *__restrict indexes;
+            *restrict indexes;
 
           register const PixelPacket
-            *__restrict p;
+            *restrict p;
 
           register IndexPacket
-            *__restrict rotate_indexes;
+            *restrict rotate_indexes;
 
           register long
             y;
 
           register PixelPacket
-            *__restrict q;
+            *restrict q;
 
           unsigned long
-            tile_height,
-            tile_width;
+            height,
+            width;
 
-          tile_width=TileWidth;
-          if ((tile_x+TileWidth) > (long) image->columns)
-            tile_width=(unsigned long) (TileWidth-(tile_x+TileWidth-
+          width=tile_width;
+          if ((tile_x+(long) tile_width) > (long) image->columns)
+            width=(unsigned long) (tile_width-(tile_x+tile_width-
               image->columns));
-          tile_height=TileHeight;
-          if ((tile_y+TileHeight) > (long) image->rows)
-            tile_height=(unsigned long) (TileHeight-(tile_y+TileHeight-
+          height=tile_height;
+          if ((tile_y+(long) tile_height) > (long) image->rows)
+            height=(unsigned long) (tile_height-(tile_y+tile_height-
               image->rows));
-          p=GetCacheViewVirtualPixels(image_view,tile_x,tile_y,tile_width,
-            tile_height,exception);
+          p=GetCacheViewVirtualPixels(image_view,tile_x,tile_y,width,height,
+            exception);
           if (p == (const PixelPacket *) NULL)
             {
               status=MagickFalse;
               break;
             }
           indexes=GetCacheViewVirtualIndexQueue(image_view);
-          for (y=0; y < (long) tile_width; y++)
+          for (y=0; y < (long) width; y++)
           {
             register const PixelPacket
-              *__restrict tile_pixels;
+              *restrict tile_pixels;
 
             register long
               x;
 
             q=QueueCacheViewAuthenticPixels(rotate_view,(long)
-              rotate_image->columns-(tile_y+tile_height),y+tile_x,tile_height,
+              rotate_image->columns-(tile_y+height),y+tile_x,height,
               1,exception);
             if (q == (PixelPacket *) NULL)
               {
                 status=MagickFalse;
                 break;
               }
-            tile_pixels=p+(tile_height-1)*tile_width+y;
-            for (x=0; x < (long) tile_height; x++)
+            tile_pixels=p+(height-1)*width+y;
+            for (x=0; x < (long) height; x++)
             {
               *q++=(*tile_pixels);
-              tile_pixels-=tile_width;
+              tile_pixels-=width;
             }
             rotate_indexes=GetCacheViewAuthenticIndexQueue(rotate_view);
             if ((indexes != (IndexPacket *) NULL) &&
                 (rotate_indexes != (IndexPacket *) NULL))
               {
                 register const IndexPacket
-                  *__restrict tile_indexes;
+                  *restrict tile_indexes;
 
-                tile_indexes=indexes+(tile_height-1)*tile_width+y;
-                for (x=0; x < (long) tile_height; x++)
+                tile_indexes=indexes+(height-1)*width+y;
+                for (x=0; x < (long) height; x++)
                 {
                   *rotate_indexes++=(*tile_indexes);
-                  tile_indexes-=tile_width;
+                  tile_indexes-=width;
                 }
               }
             sync=SyncCacheViewAuthenticPixels(rotate_view,exception);
@@ -1219,15 +1170,13 @@ static Image *IntegralRotateImage(const Image *image,unsigned long rotations,
             MagickBooleanType
               proceed;
 
-#if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp critical (MagickCore_IntegralRotateImage)
-#endif
-            proceed=SetImageProgress(image,RotateImageTag,progress+=TileHeight,
+            proceed=SetImageProgress(image,RotateImageTag,progress+=tile_height,
               image->rows);
             if (proceed == MagickFalse)
               status=MagickFalse;
           }
       }
+      (void) SetImageProgress(image,RotateImageTag,image->rows-1,image->rows);
       Swap(page.width,page.height);
       Swap(page.x,page.y);
       if (page.width != 0)
@@ -1240,7 +1189,7 @@ static Image *IntegralRotateImage(const Image *image,unsigned long rotations,
         Rotate 180 degrees.
       */
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static,8) shared(progress,status)
+  #pragma omp parallel for schedule(dynamic,4) shared(progress, status)
 #endif
       for (y=0; y < (long) image->rows; y++)
       {
@@ -1248,19 +1197,19 @@ static Image *IntegralRotateImage(const Image *image,unsigned long rotations,
           sync;
 
         register const IndexPacket
-          *__restrict indexes;
+          *restrict indexes;
 
         register const PixelPacket
-          *__restrict p;
+          *restrict p;
 
         register IndexPacket
-          *__restrict rotate_indexes;
+          *restrict rotate_indexes;
 
         register long
           x;
 
         register PixelPacket
-          *__restrict q;
+          *restrict q;
 
         if (status == MagickFalse)
           continue;
@@ -1290,9 +1239,6 @@ static Image *IntegralRotateImage(const Image *image,unsigned long rotations,
             MagickBooleanType
               proceed;
 
-#if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp critical (MagickCore_IntegralRotateImage)
-#endif
             proceed=SetImageProgress(image,RotateImageTag,progress++,
               image->rows);
             if (proceed == MagickFalse)
@@ -1310,92 +1256,97 @@ static Image *IntegralRotateImage(const Image *image,unsigned long rotations,
       long
         tile_y;
 
+      unsigned long
+        tile_height,
+        tile_width;
+
       /*
         Rotate 270 degrees.
       */
+      GetPixelCacheTileSize(image,&tile_width,&tile_height);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static,1) shared(progress,status)
+  #pragma omp parallel for schedule(dynamic,4) shared(progress, status)
 #endif
-      for (tile_y=0; tile_y < (long) image->rows; tile_y+=TileHeight)
+      for (tile_y=0; tile_y < (long) image->rows; tile_y+=tile_height)
       {
         register long
           tile_x;
 
         if (status == MagickFalse)
           continue;
-        for (tile_x=0; tile_x < (long) image->columns; tile_x+=TileWidth)
+        for (tile_x=0; tile_x < (long) image->columns; tile_x+=tile_width)
         {
           MagickBooleanType
             sync;
 
           register const IndexPacket
-            *__restrict indexes;
+            *restrict indexes;
 
           register const PixelPacket
-            *__restrict p;
+            *restrict p;
 
           register IndexPacket
-            *__restrict rotate_indexes;
+            *restrict rotate_indexes;
 
           register long
             y;
 
           register PixelPacket
-            *__restrict q;
+            *restrict q;
 
           unsigned long
-            tile_height,
-            tile_width;
+            height,
+            width;
 
-          tile_width=TileWidth;
-          if ((tile_x+TileWidth) > (long) image->columns)
-            tile_width=(unsigned long) (TileWidth-(tile_x+TileWidth-
+          width=tile_width;
+          if ((tile_x+(long) tile_width) > (long) image->columns)
+            width=(unsigned long) (tile_width-(tile_x+tile_width-
               image->columns));
-          tile_height=TileHeight;
-          if ((tile_y+TileHeight) > (long) image->rows)
-            tile_height=(unsigned long) (TileHeight-(tile_y+TileHeight-
+          height=tile_height;
+          if ((tile_y+(long) tile_height) > (long) image->rows)
+            height=(unsigned long) (tile_height-(tile_y+tile_height-
               image->rows));
-          p=GetCacheViewVirtualPixels(image_view,tile_x,tile_y,tile_width,
-            tile_height,exception);
+          p=GetCacheViewVirtualPixels(image_view,tile_x,tile_y,width,
+            height,exception);
           if (p == (const PixelPacket *) NULL)
             {
               status=MagickFalse;
               break;
             }
           indexes=GetCacheViewVirtualIndexQueue(image_view);
-          for (y=0; y < (long) tile_width; y++)
+          for (y=0; y < (long) width; y++)
           {
             register const PixelPacket
-              *__restrict tile_pixels;
+              *restrict tile_pixels;
 
             register long
               x;
 
             q=QueueCacheViewAuthenticPixels(rotate_view,tile_y,(long)
-              y+rotate_image->rows-(tile_x+tile_width),tile_height,1,exception);
+              y+rotate_image->rows-(tile_x+width),height,1,exception);
             if (q == (PixelPacket *) NULL)
               {
                 status=MagickFalse;
                 break;
               }
-            tile_pixels=p+(tile_width-1)-y;
-            for (x=0; x < (long) tile_height; x++)
+            tile_pixels=p+(width-1)-y;
+            for (x=0; x < (long) height; x++)
             {
               *q++=(*tile_pixels);
-              tile_pixels+=tile_width;
+              tile_pixels+=width;
             }
             rotate_indexes=GetCacheViewAuthenticIndexQueue(rotate_view);
             if ((indexes != (IndexPacket *) NULL) &&
                 (rotate_indexes != (IndexPacket *) NULL))
               {
                 register const IndexPacket
-                  *__restrict tile_indexes;
+                  *restrict tile_indexes;
 
-                tile_indexes=indexes+(tile_width-1)-y;
-                for (x=0; x < (long) tile_height; x++)
+                tile_indexes=indexes+(width-1)-y;
+                for (x=0; x < (long) height; x++)
                 {
                   *rotate_indexes++=(*tile_indexes);
-                  tile_indexes+=tile_width;
+                  tile_indexes+=width;
                 }
               }
             sync=SyncCacheViewAuthenticPixels(rotate_view,exception);
@@ -1408,15 +1359,13 @@ static Image *IntegralRotateImage(const Image *image,unsigned long rotations,
             MagickBooleanType
               proceed;
 
-#if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp critical (MagickCore_IntegralRotateImage)
-#endif
-            proceed=SetImageProgress(image,RotateImageTag,progress+=TileHeight,
+            proceed=SetImageProgress(image,RotateImageTag,progress+=tile_height,
               image->rows);
             if (proceed == MagickFalse)
               status=MagickFalse;
           }
       }
+      (void) SetImageProgress(image,RotateImageTag,image->rows-1,image->rows);
       Swap(page.width,page.height);
       Swap(page.x,page.y);
       if (page.height != 0)
@@ -1467,50 +1416,6 @@ static Image *IntegralRotateImage(const Image *image,unsigned long rotations,
 %      to shear.
 %
 */
-
-static inline MagickRealType Blend_(const MagickRealType p,
-  const MagickRealType alpha,const MagickRealType q,const MagickRealType beta)
-{
-  return((1.0-QuantumScale*alpha)*p+(1.0-QuantumScale*beta)*q);
-}
-
-static inline void MagickCompositeBlend(const MagickPixelPacket *p,
-  const MagickRealType alpha,const MagickPixelPacket *q,
-  const MagickRealType beta,const MagickRealType area,
-  MagickPixelPacket *composite)
-{
-  MagickRealType
-    gamma;
-
-  if ((alpha == TransparentOpacity) && (beta == TransparentOpacity))
-    {
-      *composite=(*p);
-      return;
-    }
-  gamma=RoundToUnity((1.0-QuantumScale*(QuantumRange-(1.0-area)*
-    (QuantumRange-alpha)))+(1.0-QuantumScale*(QuantumRange-area*
-    (QuantumRange-beta))));
-  composite->opacity=(MagickRealType) QuantumRange*(1.0-gamma);
-  gamma=1.0/(fabs(gamma) <= MagickEpsilon ? 1.0 : gamma);
-  composite->red=gamma*Blend_((MagickRealType) p->red,
-    (MagickRealType) QuantumRange-(1.0-area)*(QuantumRange-alpha),
-    (MagickRealType) q->red,(MagickRealType) (QuantumRange-area*(QuantumRange-
-    beta)));
-  composite->green=gamma*Blend_((MagickRealType) p->green,
-    (MagickRealType) QuantumRange-(1.0-area)*(QuantumRange-alpha),
-    (MagickRealType) q->green,(MagickRealType) (QuantumRange-area*(QuantumRange-
-    beta)));
-  composite->blue=gamma*Blend_((MagickRealType) p->blue,
-    (MagickRealType) QuantumRange-(1.0-area)*(QuantumRange-alpha),
-    (MagickRealType) q->blue,(MagickRealType) (QuantumRange-area*(QuantumRange-
-    beta)));
-  if (q->colorspace == CMYKColorspace)
-    composite->index=gamma*Blend_((MagickRealType) p->index,
-      (MagickRealType) QuantumRange-(1.0-area)*(QuantumRange-alpha),
-      (MagickRealType) q->index,(MagickRealType) (QuantumRange-area*
-      (QuantumRange-beta)));
-}
-
 static MagickBooleanType XShearImage(Image *image,const MagickRealType degrees,
   const unsigned long width,const unsigned long height,const long x_offset,
   const long y_offset)
@@ -1523,6 +1428,9 @@ static MagickBooleanType XShearImage(Image *image,const MagickRealType degrees,
     RIGHT
   } ShearDirection;
 
+  CacheView
+    *image_view;
+
   ExceptionInfo
     *exception;
 
@@ -1536,9 +1444,6 @@ static MagickBooleanType XShearImage(Image *image,const MagickRealType degrees,
   MagickPixelPacket
     background;
 
-  CacheView
-    *image_view;
-
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
@@ -1556,7 +1461,7 @@ static MagickBooleanType XShearImage(Image *image,const MagickRealType degrees,
   exception=(&image->exception);
   image_view=AcquireCacheView(image);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(dynamic,8) shared(progress, status)
+  #pragma omp parallel for schedule(dynamic,4) shared(progress, status)
 #endif
   for (y=0; y < (long) height; y++)
   {
@@ -1576,12 +1481,12 @@ static MagickBooleanType XShearImage(Image *image,const MagickRealType degrees,
       i;
 
     register IndexPacket
-      *__restrict indexes,
-      *__restrict shear_indexes;
+      *restrict indexes,
+      *restrict shear_indexes;
 
     register PixelPacket
-      *__restrict p,
-      *__restrict q;
+      *restrict p,
+      *restrict q;
 
     ShearDirection
       direction;
@@ -1635,13 +1540,13 @@ static MagickBooleanType XShearImage(Image *image,const MagickRealType degrees,
               continue;
             }
           SetMagickPixelPacket(image,p,indexes,&source);
-          MagickCompositeBlend(&pixel,(MagickRealType) pixel.opacity,&source,
-            (MagickRealType) p->opacity,area,&destination);
+          MagickPixelCompositeAreaBlend(&pixel,(MagickRealType) pixel.opacity,
+            &source,(MagickRealType) p->opacity,area,&destination);
           SetPixelPacket(image,&destination,q++,shear_indexes++);
           SetMagickPixelPacket(image,p++,indexes++,&pixel);
         }
-        MagickCompositeBlend(&pixel,(MagickRealType) pixel.opacity,&background,
-          (MagickRealType) background.opacity,area,&destination);
+        MagickPixelCompositeAreaBlend(&pixel,(MagickRealType) pixel.opacity,
+          &background,(MagickRealType) background.opacity,area,&destination);
         SetPixelPacket(image,&destination,q++,shear_indexes++);
         for (i=0; i < (step-1); i++)
           SetPixelPacket(image,&background,q++,shear_indexes++);
@@ -1665,13 +1570,13 @@ static MagickBooleanType XShearImage(Image *image,const MagickRealType degrees,
           if ((unsigned long) (x_offset+width+step-i) >= image->columns)
             continue;
           SetMagickPixelPacket(image,p,indexes,&source);
-          MagickCompositeBlend(&pixel,(MagickRealType) pixel.opacity,&source,
-            (MagickRealType) p->opacity,area,&destination);
+          MagickPixelCompositeAreaBlend(&pixel,(MagickRealType) pixel.opacity,
+            &source,(MagickRealType) p->opacity,area,&destination);
           SetPixelPacket(image,&destination,q,shear_indexes);
           SetMagickPixelPacket(image,p,indexes,&pixel);
         }
-        MagickCompositeBlend(&pixel,(MagickRealType) pixel.opacity,&background,
-          (MagickRealType) background.opacity,area,&destination);
+        MagickPixelCompositeAreaBlend(&pixel,(MagickRealType) pixel.opacity,
+          &background,(MagickRealType) background.opacity,area,&destination);
         SetPixelPacket(image,&destination,--q,--shear_indexes);
         for (i=0; i < (step-1); i++)
           SetPixelPacket(image,&background,--q,--shear_indexes);
@@ -1743,6 +1648,9 @@ static MagickBooleanType YShearImage(Image *image,const MagickRealType degrees,
     DOWN
   } ShearDirection;
 
+  CacheView
+    *image_view;
+
   ExceptionInfo
     *exception;
 
@@ -1756,9 +1664,6 @@ static MagickBooleanType YShearImage(Image *image,const MagickRealType degrees,
   MagickPixelPacket
     background;
 
-  CacheView
-    *image_view;
-
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
@@ -1776,7 +1681,7 @@ static MagickBooleanType YShearImage(Image *image,const MagickRealType degrees,
   exception=(&image->exception);
   image_view=AcquireCacheView(image);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(dynamic,8) shared(progress, status)
+  #pragma omp parallel for schedule(dynamic,4) shared(progress, status)
 #endif
   for (x=0; x < (long) width; x++)
   {
@@ -1793,15 +1698,15 @@ static MagickBooleanType YShearImage(Image *image,const MagickRealType degrees,
       displacement;
 
     register IndexPacket
-      *__restrict indexes,
-      *__restrict shear_indexes;
+      *restrict indexes,
+      *restrict shear_indexes;
 
     register long
       i;
 
     register PixelPacket
-      *__restrict p,
-      *__restrict q;
+      *restrict p,
+      *restrict q;
 
     ShearDirection
       direction;
@@ -1855,13 +1760,13 @@ static MagickBooleanType YShearImage(Image *image,const MagickRealType degrees,
               continue;
             }
           SetMagickPixelPacket(image,p,indexes,&source);
-          MagickCompositeBlend(&pixel,(MagickRealType) pixel.opacity,&source,
-            (MagickRealType) p->opacity,area,&destination);
+          MagickPixelCompositeAreaBlend(&pixel,(MagickRealType) pixel.opacity,
+            &source,(MagickRealType) p->opacity,area,&destination);
           SetPixelPacket(image,&destination,q++,shear_indexes++);
           SetMagickPixelPacket(image,p++,indexes++,&pixel);
         }
-        MagickCompositeBlend(&pixel,(MagickRealType) pixel.opacity,&background,
-          (MagickRealType) background.opacity,area,&destination);
+        MagickPixelCompositeAreaBlend(&pixel,(MagickRealType) pixel.opacity,
+          &background,(MagickRealType) background.opacity,area,&destination);
         SetPixelPacket(image,&destination,q++,shear_indexes++);
         for (i=0; i < (step-1); i++)
           SetPixelPacket(image,&background,q++,shear_indexes++);
@@ -1885,13 +1790,13 @@ static MagickBooleanType YShearImage(Image *image,const MagickRealType degrees,
           if ((unsigned long) (y_offset+height+step-i) >= image->rows)
             continue;
           SetMagickPixelPacket(image,p,indexes,&source);
-          MagickCompositeBlend(&pixel,(MagickRealType) pixel.opacity,&source,
-            (MagickRealType) p->opacity,area,&destination);
+          MagickPixelCompositeAreaBlend(&pixel,(MagickRealType) pixel.opacity,
+            &source,(MagickRealType) p->opacity,area,&destination);
           SetPixelPacket(image,&destination,q,shear_indexes);
           SetMagickPixelPacket(image,p,indexes,&pixel);
         }
-        MagickCompositeBlend(&pixel,(MagickRealType) pixel.opacity,&background,
-          (MagickRealType) background.opacity,area,&destination);
+        MagickPixelCompositeAreaBlend(&pixel,(MagickRealType) pixel.opacity,
+          &background,(MagickRealType) background.opacity,area,&destination);
         SetPixelPacket(image,&destination,--q,--shear_indexes);
         for (i=0; i < (step-1); i++)
           SetPixelPacket(image,&background,--q,--shear_indexes);
