@@ -1,12 +1,12 @@
 /*
-  Copyright 1999-2007 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
-  
+
   You may not use this file except in compliance with the License.
   obtain a copy of the License at
-  
+
     http://www.imagemagick.org/script/license.php
-  
+
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -22,18 +22,18 @@
 extern "C" {
 #endif
 
-#ifdef __CYGWIN32__
-#  ifndef __CYGWIN__
+#if defined(__CYGWIN32__)
+#  if !defined(__CYGWIN__)
 #    define __CYGWIN__ __CYGWIN32__
 #  endif
 #endif
 
 #if defined(_WIN32) || defined(WIN32)
-#  ifndef __WINDOWS__
-#    ifdef _WIN32
+#  if !defined(__WINDOWS__)
+#    if defined(_WIN32)
 #      define __WINDOWS__ _WIN32
 #    else
-#      ifdef WIN32
+#      if defined(WIN32)
 #        define __WINDOWS__ WIN32
 #      endif
 #    endif
@@ -41,11 +41,11 @@ extern "C" {
 #endif
 
 #if defined(_WIN64) || defined(WIN64)
-#  ifndef __WINDOWS__
-#    ifdef _WIN64
+#  if !defined(__WINDOWS__)
+#    if defined(_WIN64)
 #      define __WINDOWS__ _WIN64
 #    else
-#      ifdef WIN64
+#      if defined(WIN64)
 #        define __WINDOWS__ WIN64
 #      endif
 #    endif
@@ -53,7 +53,7 @@ extern "C" {
 #endif
 
 #if !defined(vms) && !defined(macintosh) && !defined(__WINDOWS__)
-# define POSIX
+# define MAGICKCORE_POSIX_SUPPORT
 #endif
 
 #define MAGICKCORE_IMPLEMENTATION  1
@@ -65,12 +65,24 @@ extern "C" {
 # else
 #  include "magick-config.h"
 # endif
+#if defined(MAGICKCORE__FILE_OFFSET_BITS) && !defined(_FILE_OFFSET_BITS)
+# define _FILE_OFFSET_BITS MAGICKCORE__FILE_OFFSET_BITS
+#endif
+#if defined(_magickcore_const) && !defined(const)
+# define const  _magickcore_const
+#endif
+#if defined(_magickcore_inline) && !defined(inline)
+# define inline  _magickcore_inline
+#endif
 # if defined(__cplusplus) || defined(c_plusplus)
 #  undef inline
 # endif
+#if defined(_magickcore_restrict) && !defined(restrict)
+# define restrict  _magickcore_restrict
+#endif
 #endif
 
-#if defined(MagickMethodPrefix)
+#if defined(MAGICKCORE_NAMESPACE_PREFIX)
 # include "magick/methods.h"
 #endif
 
@@ -82,8 +94,8 @@ extern "C" {
 #  pragma message("BCBMagick lib DLL export interface")
 #  define _MAGICKDLL_
 #  define _MAGICKLIB_
-#  define SupportMagickModules
-#  undef BuildMagickModules
+#  define MAGICKCORE_MODULES_SUPPORT
+#  undef MAGICKCORE_BUILD_MODULES
 #endif
 
 #if defined(__WINDOWS__)
@@ -97,30 +109,30 @@ extern "C" {
 #  if !defined(_MAGICKLIB_)
 #   define MagickExport  __declspec(dllimport)
 #   if defined(_VISUALC_)
-#    pragma message( "Magick lib DLL import interface" )
+#    pragma message( "MagickCore lib DLL import interface" )
 #   endif
 #  else
 #   define MagickExport  __declspec(dllexport)
 #   if defined(_VISUALC_)
-#    pragma message( "Magick lib DLL export interface" )
+#    pragma message( "MagickCore lib DLL export interface" )
 #   endif
 #  endif
 # else
 #  define MagickExport
 #  if defined(_VISUALC_)
-#   pragma message( "Magick lib static interface" )
+#   pragma message( "MagickCore lib static interface" )
 #  endif
 # endif
 
 # if defined(_DLL) && !defined(_LIB)
 #  define ModuleExport  __declspec(dllexport)
 #  if defined(_VISUALC_)
-#   pragma message( "Magick module DLL export interface" )
+#   pragma message( "MagickCore module DLL export interface" )
 #  endif
 # else
 #  define ModuleExport
 #  if defined(_VISUALC_)
-#   pragma message( "Magick module static interface" )
+#   pragma message( "MagickCore module static interface" )
 #  endif
 
 # endif
@@ -155,12 +167,12 @@ extern "C" {
 # include <unistd.h>
 #else
 # include <direct.h>
-# if !defined(HAVE_STRERROR)
+# if !defined(MAGICKCORE_HAVE_STRERROR)
 #  define HAVE_STRERROR
 # endif
 #endif
 
-#if defined(HAVE_STRINGS_H)
+#if defined(MAGICKCORE_HAVE_STRINGS_H)
 # include <strings.h>
 #endif
 #include <string.h>
@@ -174,32 +186,60 @@ extern "C" {
 #include <signal.h>
 #include <assert.h>
 
-#if defined(HAVE_PREAD) && defined(HAVE_DECL_PREAD) && !HAVE_DECL_PREAD
+#if defined(MAGICKCORE_HAVE_PTHREAD)
+# include <pthread.h>
+#elif defined(__WINDOWS__)
+#  define MAGICKCORE_HAVE_WINTHREADS  1
+#include <windows.h>
+#endif
+#if defined(MAGICKCORE_HAVE_SYS_SYSLIMITS_H)
+# include <sys/syslimits.h>
+#endif
+#if defined(MAGICKCORE_HAVE_ARM_LIMITS_H)
+# include <arm/limits.h>
+#endif
+
+#if defined(MAGICKCORE_HAVE_CL_CL_H)
+#  include <CL/cl.h>
+#  define MAGICKCORE_OPENCL_SUPPORT  1
+#endif
+
+#if defined(MAGICKCORE_HAVE_OPENCL_CL_H)
+#  include <OpenCL/cl.h>
+#  define MAGICKCORE_OPENCL_SUPPORT  1
+#endif
+
+#if defined(_OPENMP) && (_OPENMP >= 200203)
+#  include <omp.h>
+#  define MAGICKCORE_OPENMP_SUPPORT  1
+#endif
+
+#if defined(MAGICKCORE_HAVE_PREAD) && defined(MAGICKCORE_HAVE_DECL_PREAD) && !MAGICKCORE_HAVE_DECL_PREAD
 ssize_t pread(int,void *,size_t,off_t);
 #endif
 
-#if defined(HAVE_PWRITE) && defined(HAVE_DECL_PWRITE) && !HAVE_DECL_PWRITE
+#if defined(MAGICKCORE_HAVE_PWRITE) && defined(MAGICKCORE_HAVE_DECL_PWRITE) && !MAGICKCORE_HAVE_DECL_PWRITE
 ssize_t pwrite(int,const void *,size_t,off_t);
 #endif
 
-#if defined(HAVE_STRLCPY) && defined(HAVE_DECL_STRLCPY) && !HAVE_DECL_STRLCPY
+#if defined(MAGICKCORE_HAVE_STRLCPY) && defined(MAGICKCORE_HAVE_DECL_STRLCPY) && !MAGICKCORE_HAVE_DECL_STRLCPY
 extern size_t strlcpy(char *,const char *,size_t);
 #endif
 
-#if defined(HAVE_VSNPRINTF) && defined(HAVE_DECL_VSNPRINTF) && !HAVE_DECL_VSNPRINTF
+#if defined(MAGICKCORE_HAVE_VSNPRINTF) && defined(MAGICKCORE_HAVE_DECL_VSNPRINTF) && !MAGICKCORE_HAVE_DECL_VSNPRINTF
 extern int vsnprintf(char *,size_t,const char *,va_list);
 #endif
 
 #if !defined(magick_attribute)
-#  if !defined(__GNUC__)
-#    define magick_attribute(x)  /* nothing */
-#  else
+#  if (defined(__GNUC__) && (__GNUC__ >= 3))
 #    define magick_attribute  __attribute__
+#  else
+#    define magick_attribute(x)  /* nothing */
 #  endif
 #endif
 
 #if !defined(magick_unused)
-#  if defined(__GNUC__)
+#  if (defined(__GNUC__) && (__GNUC__ >= 3))
 #     define magick_unused(x)  magick_unused_ ## x __attribute__((unused))
 #  elif defined(__LCLINT__)
 #    define magick_unused(x) /*@unused@*/ x
@@ -208,23 +248,23 @@ extern int vsnprintf(char *,size_t,const char *,va_list);
 #  endif
 #endif
 
-#if defined(__WINDOWS__) || defined(POSIX)
+#if defined(__WINDOWS__) || defined(MAGICKCORE_POSIX_SUPPORT)
 # include <sys/types.h>
 # include <sys/stat.h>
-# if defined(HAVE_FTIME)
+# if defined(MAGICKCORE_HAVE_FTIME)
 # include <sys/timeb.h>
 # endif
-# if defined(POSIX)
-#  if defined(HAVE_SYS_NDIR_H) || defined(HAVE_SYS_DIR_H) || defined(HAVE_NDIR_H)
+# if defined(MAGICKCORE_POSIX_SUPPORT)
+#  if defined(MAGICKCORE_HAVE_SYS_NDIR_H) || defined(MAGICKCORE_HAVE_SYS_DIR_H) || defined(MAGICKCORE_HAVE_NDIR_H)
 #   define dirent direct
 #   define NAMLEN(dirent) (dirent)->d_namlen
-#   if defined(HAVE_SYS_NDIR_H)
+#   if defined(MAGICKCORE_HAVE_SYS_NDIR_H)
 #    include <sys/ndir.h>
 #   endif
-#   if defined(HAVE_SYS_DIR_H)
+#   if defined(MAGICKCORE_HAVE_SYS_DIR_H)
 #    include <sys/dir.h>
 #   endif
-#   if defined(HAVE_NDIR_H)
+#   if defined(MAGICKCORE_HAVE_NDIR_H)
 #    include <ndir.h>
 #   endif
 #  else
@@ -243,10 +283,10 @@ extern int vsnprintf(char *,size_t,const char *,va_list);
 # include "magick/magick-type.h"
 # if !defined(__WINDOWS__)
 #  include <sys/time.h>
-# if defined(HAVE_SYS_TIMES_H)
+# if defined(MAGICKCORE_HAVE_SYS_TIMES_H)
 #  include <sys/times.h>
 # endif
-# if defined(HAVE_SYS_RESOURCE_H)
+# if defined(MAGICKCORE_HAVE_SYS_RESOURCE_H)
 #  include <sys/resource.h>
 # endif
 #endif
@@ -281,51 +321,66 @@ extern int vsnprintf(char *,size_t,const char *,va_list);
 # include "magick/vms.h"
 #endif
 
+#undef HAVE_CONFIG_H
+#undef gamma
 #undef index
 #undef pipe
+#undef y1
 
 /*
   Review these platform specific definitions.
 */
-#if defined(POSIX)
+#if defined(MAGICKCORE_POSIX_SUPPORT) && !defined(__OS2__)
 # define DirectorySeparator  "/"
 # define DirectoryListSeparator  ':'
 # define EditorOptions  " -title \"Edit Image Comment\" -e vi"
 # define Exit  exit
 # define IsBasenameSeparator(c)  ((c) == '/' ? MagickTrue : MagickFalse)
-# define PreferencesDefaults  "~/."
+# define X11_PREFERENCES_PATH  "~/."
 # define ProcessPendingEvents(text)
 # define ReadCommandlLine(argc,argv)
 # define SetNotifyHandlers
 #else
 # if defined(vms)
-#  define ApplicationDefaults  "decw$system_defaults:"
+#  define X11_APPLICATION_PATH  "decw$system_defaults:"
 #  define DirectorySeparator  ""
 #  define DirectoryListSeparator  ';'
 #  define EditorOptions  ""
 #  define Exit  exit
 #  define IsBasenameSeparator(c) \
   (((c) == ']') || ((c) == ':') || ((c) == '/') ? MagickTrue : MagickFalse)
-#  define MagickLibPath  "sys$login:"
-#  define MagickImageCodersPath  "sys$login:"
-#  define MagickImageFiltersPath  "sys$login:"
-#  define MagickSharePath  "sys$login:"
-#  define PreferencesDefaults  "decw$user_defaults:"
+#  define MAGICKCORE_LIBRARY_PATH  "sys$login:"
+#  define MAGICKCORE_CODER_PATH  "sys$login:"
+#  define MAGICKCORE_FILTER_PATH  "sys$login:"
+#  define MAGICKCORE_SHARE_PATH  "sys$login:"
+#  define X11_PREFERENCES_PATH  "decw$user_defaults:"
 #  define ProcessPendingEvents(text)
 #  define ReadCommandlLine(argc,argv)
 #  define SetNotifyHandlers
 # endif
+# if defined(__OS2__)
+#   define DirectorySeparator  "\\"
+#   define DirectoryListSeparator  ';'
+# define EditorOptions  " -title \"Edit Image Comment\" -e vi"
+# define Exit  exit
+#  define IsBasenameSeparator(c) \
+  (((c) == '/') || ((c) == '\\') ? MagickTrue : MagickFalse)
+# define PreferencesDefaults  "~\."
+# define ProcessPendingEvents(text)
+# define ReadCommandlLine(argc,argv)
+# define SetNotifyHandlers
+#endif
 # if defined(macintosh)
-#  define ApplicationDefaults  "/usr/lib/X11/app-defaults/"
+#  define X11_APPLICATION_PATH  "/usr/lib/X11/app-defaults/"
 #  define DirectorySeparator  ":"
 #  define DirectoryListSeparator  ';'
 #  define EditorOptions ""
 #  define IsBasenameSeparator(c)  ((c) == ':' ? MagickTrue : MagickFalse)
-#  define MagickLibPath  ""
-#  define MagickImageCodersPath  ""
-#  define MagickImageFiltersPath  ""
-#  define MagickSharePath  ""
-#  define PreferencesDefaults  "~/."
+#  define MAGICKCORE_LIBRARY_PATH  ""
+#  define MAGICKCORE_CODER_PATH  ""
+#  define MAGICKCORE_FILTER_PATH  ""
+#  define MAGICKCORE_SHARE_PATH  ""
+#  define X11_PREFERENCES_PATH  "~/."
 #  if defined(DISABLE_SIOUX)
 #   define ReadCommandlLine(argc,argv)
 #   define SetNotifyHandlers \
@@ -346,8 +401,8 @@ extern int vsnprintf(char *,size_t,const char *,va_list);
 #  define IsBasenameSeparator(c) \
   (((c) == '/') || ((c) == '\\') ? MagickTrue : MagickFalse)
 #  define ProcessPendingEvents(text)
-#  if !defined(PreferencesDefaults)
-#    define PreferencesDefaults  "~\\."
+#  if !defined(X11_PREFERENCES_PATH)
+#    define X11_PREFERENCES_PATH  "~\\."
 #  endif
 #  define ReadCommandlLine(argc,argv)
 #  define SetNotifyHandlers \
@@ -355,7 +410,7 @@ extern int vsnprintf(char *,size_t,const char *,va_list);
     SetWarningHandler(NTWarningHandler)
 #  undef sleep
 #  define sleep(seconds)  Sleep(seconds*1000)
-#  if !defined(HAVE_TIFFCONF_H)
+#  if !defined(MAGICKCORE_HAVE_TIFFCONF_H)
 #    define HAVE_TIFFCONF_H
 #  endif
 # endif
@@ -373,13 +428,17 @@ extern int vsnprintf(char *,size_t,const char *,va_list);
 #define O_BINARY  0x00
 #endif
 
-#if defined(HasLTDL) || (defined(__WINDOWS__) && defined(_DLL) && !defined(_LIB))
-#  define SupportMagickModules
+#if !defined(PATH_MAX)
+#define PATH_MAX  4096
+#endif
+
+#if defined(MAGICKCORE_LTDL_DELEGATE) || (defined(__WINDOWS__) && defined(_DLL) && !defined(_LIB))
+#  define MAGICKCORE_MODULES_SUPPORT
 #endif
 
 #if defined(_MAGICKMOD_)
-# undef BuildMagickModules
-# define BuildMagickModules
+# undef MAGICKCORE_BUILD_MODULES
+# define MAGICKCORE_BUILD_MODULES
 #endif
 
 /*
