@@ -200,19 +200,19 @@ MagickExport Image *AdaptiveThresholdImage(const Image *image,
       sync;
 
     register const IndexPacket
-      *__restrict indexes;
+      *restrict indexes;
 
     register const PixelPacket
-      *__restrict p;
+      *restrict p;
 
     register IndexPacket
-      *__restrict threshold_indexes;
+      *restrict threshold_indexes;
 
     register long
       x;
 
     register PixelPacket
-      *__restrict q;
+      *restrict q;
 
     if (status == MagickFalse)
       continue;
@@ -391,13 +391,13 @@ MagickExport MagickBooleanType BilevelImageChannel(Image *image,
   for (y=0; y < (long) image->rows; y++)
   {
     register IndexPacket
-      *__restrict indexes;
+      *restrict indexes;
 
     register long
       x;
 
     register PixelPacket
-      *__restrict q;
+      *restrict q;
 
     if (status == MagickFalse)
       continue;
@@ -478,7 +478,7 @@ MagickExport MagickBooleanType BilevelImageChannel(Image *image,
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %  BlackThresholdImage() is like ThresholdImage() but forces all pixels below
-%  the threshold into black while leaving all pixels above the threshold
+%  the threshold into black while leaving all pixels at or above the threshold
 %  unchanged.
 %
 %  The format of the BlackThresholdImage method is:
@@ -572,18 +572,18 @@ MagickExport MagickBooleanType BlackThresholdImageChannel(Image *image,
   progress=0;
   image_view=AcquireCacheView(image);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(dynamic,8) shared(progress,status)
+  #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
 #endif
   for (y=0; y < (long) image->rows; y++)
   {
     register IndexPacket
-      *__restrict indexes;
+      *restrict indexes;
 
     register long
       x;
 
     register PixelPacket
-      *__restrict q;
+      *restrict q;
 
     if (status == MagickFalse)
       continue;
@@ -596,22 +596,36 @@ MagickExport MagickBooleanType BlackThresholdImageChannel(Image *image,
     indexes=GetCacheViewAuthenticIndexQueue(image_view);
     for (x=0; x < (long) image->columns; x++)
     {
-      if (((channel & RedChannel) != 0) &&
-          ((MagickRealType) q->red < threshold.red))
-        q->red=(Quantum) 0;
-      if (((channel & GreenChannel) != 0) &&
-          ((MagickRealType) q->green < threshold.green))
-        q->green=(Quantum) 0;
-      if (((channel & BlueChannel) != 0) &&
-          ((MagickRealType) q->blue < threshold.blue))
-        q->blue=(Quantum) 0;
-      if (((channel & OpacityChannel) != 0) &&
-          ((MagickRealType) q->opacity < threshold.opacity))
-        q->opacity=(Quantum) 0;
-      if (((channel & IndexChannel) != 0) &&
-          (image->colorspace == CMYKColorspace) &&
-          ((MagickRealType) indexes[x] < threshold.index))
-        indexes[x]=(Quantum) 0;
+      if (channel != DefaultChannels)
+        {
+          if (PixelIntensity(q) < MagickPixelIntensity(&threshold))
+            {
+              q->red=(Quantum) 0;
+              q->green=(Quantum) 0;
+              q->blue=(Quantum) 0;
+              if (image->colorspace == CMYKColorspace)
+                indexes[x]=(Quantum) 0;
+            }
+        }
+      else
+        {
+          if (((channel & RedChannel) != 0) &&
+              ((MagickRealType) q->red < threshold.red))
+            q->red=(Quantum) 0;
+          if (((channel & GreenChannel) != 0) &&
+              ((MagickRealType) q->green < threshold.green))
+            q->green=(Quantum) 0;
+          if (((channel & BlueChannel) != 0) &&
+              ((MagickRealType) q->blue < threshold.blue))
+            q->blue=(Quantum) 0;
+          if (((channel & OpacityChannel) != 0) &&
+              ((MagickRealType) q->opacity < threshold.opacity))
+            q->opacity=(Quantum) 0;
+          if (((channel & IndexChannel) != 0) &&
+              (image->colorspace == CMYKColorspace) &&
+              ((MagickRealType) indexes[x] < threshold.index))
+            indexes[x]=(Quantum) 0;
+        }
       q++;
     }
     if (SyncCacheViewAuthenticPixels(image_view,exception) == MagickFalse)
@@ -639,6 +653,161 @@ MagickExport MagickBooleanType BlackThresholdImageChannel(Image *image,
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%     C l a m p I m a g e                                                     %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  ClampImage() restricts the color range from 0 to the quantum depth.
+%
+%  The format of the ClampImageChannel method is:
+%
+%      MagickBooleanType ClampImage(Image *image)
+%      MagickBooleanType ClampImageChannel(Image *image,
+%        const ChannelType channel)
+%
+%  A description of each parameter follows:
+%
+%    o image: the image.
+%
+%    o channel: the channel type.
+%
+*/
+
+static inline Quantum ClampToQuantum(const Quantum quantum)
+{
+#if defined(MAGICKCORE_HDRI_SUPPORT)
+  if (quantum <= 0)
+    return(0);
+  if (quantum >= QuantumRange)
+    return(QuantumRange);
+  return(quantum);
+#else
+  return(quantum);
+#endif
+}
+
+MagickExport MagickBooleanType ClampImage(Image *image)
+{
+  MagickBooleanType
+    status;
+
+  status=ClampImageChannel(image,DefaultChannels);
+  return(status);
+}
+
+MagickExport MagickBooleanType ClampImageChannel(Image *image,
+  const ChannelType channel)
+{
+#define ClampImageTag  "Clamp/Image"
+
+  ExceptionInfo
+    *exception;
+
+  long
+    progress,
+    y;
+
+  MagickBooleanType
+    status;
+
+  CacheView
+    *image_view;
+
+  assert(image != (Image *) NULL);
+  assert(image->signature == MagickSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
+  if (image->storage_class == PseudoClass)
+    {
+      register long
+        i;
+
+      register PixelPacket
+        *restrict q;
+
+      q=image->colormap;
+      for (i=0; i < (long) image->colors; i++)
+      {
+        q->red=ClampToQuantum(q->red);
+        q->green=ClampToQuantum(q->green);
+        q->blue=ClampToQuantum(q->blue);
+        q->opacity=ClampToQuantum(q->opacity);
+        q++;
+      }
+      return(SyncImage(image));
+    }
+  /*
+    Clamp image.
+  */
+  status=MagickTrue;
+  progress=0;
+  exception=(&image->exception);
+  image_view=AcquireCacheView(image);
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
+#endif
+  for (y=0; y < (long) image->rows; y++)
+  {
+    register IndexPacket
+      *restrict indexes;
+
+    register long
+      x;
+
+    register PixelPacket
+      *restrict q;
+
+    if (status == MagickFalse)
+      continue;
+    q=GetCacheViewAuthenticPixels(image_view,0,y,image->columns,1,exception);
+    if (q == (PixelPacket *) NULL)
+      {
+        status=MagickFalse;
+        continue;
+      }
+    indexes=GetCacheViewAuthenticIndexQueue(image_view);
+    for (x=0; x < (long) image->columns; x++)
+    {
+      if ((channel & RedChannel) != 0)
+        q->red=ClampToQuantum(q->red);
+      if ((channel & GreenChannel) != 0)
+        q->green=ClampToQuantum(q->green);
+      if ((channel & BlueChannel) != 0)
+        q->blue=ClampToQuantum(q->blue);
+      if ((channel & OpacityChannel) != 0)
+        q->opacity=ClampToQuantum(q->opacity);
+      if (((channel & IndexChannel) != 0) &&
+          (image->colorspace == CMYKColorspace))
+        indexes[x]=(IndexPacket) ClampToQuantum(indexes[x]);
+      q++;
+    }
+    if (SyncCacheViewAuthenticPixels(image_view,exception) == MagickFalse)
+      status=MagickFalse;
+    if (image->progress_monitor != (MagickProgressMonitor) NULL)
+      {
+        MagickBooleanType
+          proceed;
+
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp critical (MagickCore_ClampImageChannel)
+#endif
+        proceed=SetImageProgress(image,ClampImageTag,progress++,
+          image->rows);
+        if (proceed == MagickFalse)
+          status=MagickFalse;
+      }
+  }
+  image_view=DestroyCacheView(image_view);
+  return(status);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %  D e s t r o y T h r e s h o l d M a p                                      %
 %                                                                             %
 %                                                                             %
@@ -1312,13 +1481,13 @@ printf("DEBUG levels  r=%ld g=%ld b=%ld a=%ld i=%ld\n",
     for (y=0; y < (long) image->rows; y++)
     {
       register IndexPacket
-        *__restrict indexes;
+        *restrict indexes;
 
       register long
         x;
 
       register PixelPacket
-        *__restrict q;
+        *restrict q;
 
       if (status == MagickFalse)
         continue;
@@ -1534,14 +1703,14 @@ MagickExport MagickBooleanType RandomThresholdImageChannel(Image *image,
           sync;
 
         register IndexPacket
-          *__restrict indexes;
+          *restrict indexes;
 
         register long
           id,
           x;
 
         register PixelPacket
-          *__restrict q;
+          *restrict q;
 
         if (status == MagickFalse)
           continue;
@@ -1608,14 +1777,14 @@ MagickExport MagickBooleanType RandomThresholdImageChannel(Image *image,
   for (y=0; y < (long) image->rows; y++)
   {
     register IndexPacket
-      *__restrict indexes;
+      *restrict indexes;
 
     register long
       id,
       x;
 
     register PixelPacket
-      *__restrict q;
+      *restrict q;
 
     if (status == MagickFalse)
       continue;
@@ -1736,7 +1905,7 @@ MagickExport MagickBooleanType RandomThresholdImageChannel(Image *image,
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %  WhiteThresholdImage() is like ThresholdImage() but forces all pixels above
-%  the threshold into white while leaving all pixels below the threshold
+%  the threshold into white while leaving all pixels at or below the threshold
 %  unchanged.
 %
 %  The format of the WhiteThresholdImage method is:
@@ -1830,18 +1999,18 @@ MagickExport MagickBooleanType WhiteThresholdImageChannel(Image *image,
   progress=0;
   image_view=AcquireCacheView(image);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(dynamic,8) shared(progress,status)
+  #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
 #endif
   for (y=0; y < (long) image->rows; y++)
   {
     register IndexPacket
-      *__restrict indexes;
+      *restrict indexes;
 
     register long
       x;
 
     register PixelPacket
-      *__restrict q;
+      *restrict q;
 
     if (status == MagickFalse)
       continue;
@@ -1854,22 +2023,36 @@ MagickExport MagickBooleanType WhiteThresholdImageChannel(Image *image,
     indexes=GetCacheViewAuthenticIndexQueue(image_view);
     for (x=0; x < (long) image->columns; x++)
     {
-      if (((channel & RedChannel) != 0) &&
-          ((MagickRealType) q->red > threshold.red))
-        q->red=(Quantum) QuantumRange;
-      if (((channel & GreenChannel) != 0) &&
-          ((MagickRealType) q->green > threshold.green))
-        q->green=(Quantum) QuantumRange;
-      if (((channel & BlueChannel) != 0) &&
-          ((MagickRealType) q->blue > threshold.blue))
-        q->blue=(Quantum) QuantumRange;
-      if (((channel & OpacityChannel) != 0) &&
-          ((MagickRealType) q->opacity > threshold.opacity))
-        q->opacity=(Quantum) QuantumRange;
-      if (((channel & IndexChannel) != 0) &&
-          (image->colorspace == CMYKColorspace) &&
-          ((MagickRealType) indexes[x] > threshold.index))
-        indexes[x]=(Quantum) QuantumRange;
+      if (channel != DefaultChannels)
+        {
+          if (PixelIntensity(q) > MagickPixelIntensity(&threshold))
+            {
+              q->red=(Quantum) QuantumRange;
+              q->green=(Quantum) QuantumRange;
+              q->blue=(Quantum) QuantumRange;
+              if (image->colorspace == CMYKColorspace)
+                indexes[x]=(Quantum) QuantumRange;
+            }
+        }
+      else
+        {
+          if (((channel & RedChannel) != 0) &&
+              ((MagickRealType) q->red > threshold.red))
+            q->red=(Quantum) QuantumRange;
+          if (((channel & GreenChannel) != 0) &&
+              ((MagickRealType) q->green > threshold.green))
+            q->green=(Quantum) QuantumRange;
+          if (((channel & BlueChannel) != 0) &&
+              ((MagickRealType) q->blue > threshold.blue))
+            q->blue=(Quantum) QuantumRange;
+          if (((channel & OpacityChannel) != 0) &&
+              ((MagickRealType) q->opacity > threshold.opacity))
+            q->opacity=(Quantum) QuantumRange;
+          if (((channel & IndexChannel) != 0) &&
+              (image->colorspace == CMYKColorspace) &&
+              ((MagickRealType) indexes[x] > threshold.index))
+            indexes[x]=(Quantum) QuantumRange;
+        }
       q++;
     }
     if (SyncCacheViewAuthenticPixels(image_view,exception) == MagickFalse)
