@@ -27,14 +27,14 @@ extern "C" {
 */
 #define MagickPackageName "ImageMagick"
 #define MagickCopyright  "Copyright (C) 1999-2009 ImageMagick Studio LLC"
-#define MagickLibVersion  0x654
-#define MagickLibVersionText  "6.5.4"
-#define MagickLibVersionNumber  2,0,0
-#define MagickLibSubversion  "-5"
-#define MagickReleaseDate  "2009-07-23"
-#define MagickChangeDate   "20090723"
+#define MagickLibVersion  0x658
+#define MagickLibVersionText  "6.5.8"
+#define MagickLibVersionNumber  2,0,1
+#define MagickLibSubversion  "-2"
+#define MagickReleaseDate  "2009-11-28"
+#define MagickChangeDate   "20091128"
 #define MagickAuthoritativeURL  "http://www.imagemagick.org"
-#define MagickHomeURL  "file:///usr/share/doc/ImageMagick-6.5.4/index.html"
+#define MagickHomeURL  "file:///usr/share/doc/ImageMagick-6.5.8/index.html"
 #if (MAGICKCORE_QUANTUM_DEPTH == 8)
 #define MagickQuantumDepth  "Q8"
 #define MagickQuantumRange  "255"
@@ -52,25 +52,32 @@ extern "C" {
 #define MagickQuantumRange  "?"
 #endif
 #if !defined(MAGICKCORE_HDRI_SUPPORT)
-#define MagickHDRISupport ""
+#define MagickHDRIFeature ""
 #else
-#define MagickHDRISupport "HDRI "
+#define MagickHDRIFeature "HDRI "
 #endif
-#if !defined(_OPENMP)
-#define MagickOPENMPSupport ""
+#if !defined(MAGICKCORE_OPENMP_SUPPORT)
+#define MagickOPENMPFeature ""
 #else
-#define MagickOPENMPSupport "OpenMP "
+#define MagickOPENMPFeature "OpenMP "
 #endif
-#define MagickSupport MagickHDRISupport MagickOPENMPSupport
-#define MagickVersion MagickPackageName " " MagickLibVersionText \
-  MagickLibSubversion " " MagickReleaseDate " " MagickQuantumDepth " " \
-  MagickSupport MagickAuthoritativeURL
+#if !defined(MAGICKCORE_OPENCL_SUPPORT)
+#define MagickOPENCLFeature ""
+#else
+#define MagickOPENCLFeature "OpenCL "
+#endif
+#define MagickFeatures \
+  MagickHDRIFeature MagickOPENMPFeature MagickOPENCLFeature
+#define MagickVersion  \
+  MagickPackageName " " MagickLibVersionText MagickLibSubversion " " \
+  MagickReleaseDate " " MagickQuantumDepth " " MagickAuthoritativeURL
 
 extern MagickExport char
   *GetMagickHomeURL(void);
 
 extern MagickExport const char
   *GetMagickCopyright(void),
+  *GetMagickFeatures(void),
   *GetMagickPackageName(void),
   *GetMagickQuantumDepth(unsigned long *),
   *GetMagickQuantumRange(unsigned long *),
