@@ -17,7 +17,7 @@
 %                               December 2001                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2007 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -60,14 +60,8 @@
 /*
   Include declarations.
 */
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <time.h>
+#include "wand/studio.h"
 #include "wand/MagickWand.h"
-#if defined(__WINDOWS__)
-#include <windows.h>
-#endif
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -84,9 +78,6 @@
 */
 int main(int argc,char **argv)
 {
-  char
-    *option;
-
   ExceptionInfo
     *exception;
 
@@ -94,36 +85,15 @@ int main(int argc,char **argv)
     *image_info;
 
   MagickBooleanType
-    regard_warnings,
     status;
 
-  register long
-    i;
-
   MagickCoreGenesis(*argv,MagickTrue);
   exception=AcquireExceptionInfo();
-  regard_warnings=MagickFalse;
-  for (i=1; i < (long) argc; i++)
-  {
-    option=argv[i];
-    if ((strlen(option) == 1) || ((*option != '-') && (*option != '+')))
-      continue;
-    if (LocaleCompare("debug",option+1) == 0)
-      (void) SetLogEventMask(argv[++i]);
-    if (LocaleCompare("regard-warnings",option+1) == 0)
-      regard_warnings=MagickTrue;
-  }
   image_info=AcquireImageInfo();
-  status=ConjureImageCommand(image_info,argc,argv,(char **) NULL,exception);
-  if ((status == MagickFalse) || (exception->severity != UndefinedException))
-    {
-      if ((exception->severity < ErrorException) &&
-          (regard_warnings == MagickFalse))
-        status=MagickTrue;
-      CatchException(exception);
-    }
+  status=MagickCommandGenesis(image_info,ConjureImageCommand,argc,argv,
+    (char **) NULL,exception);
   image_info=DestroyImageInfo(image_info);
   exception=DestroyExceptionInfo(exception);
   MagickCoreTerminus();
-  return(status == MagickFalse ? 1 : 0);
+  return(status);
 }
