@@ -17,7 +17,7 @@
 %                               December 2003                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -33,7 +33,8 @@
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%
+%  Use the compare program to mathematically and visually annotate the
+%  difference between an image and its reconstruction.
 %
 */
 
@@ -78,7 +79,7 @@
 %
 */
 
-static void CompareUsage(void)
+static MagickBooleanType CompareUsage(void)
 {
   const char
     **p;
@@ -94,7 +95,8 @@ static void CompareUsage(void)
     },
     *settings[]=
     {
-      "-alpha option        activate, deactivate, reset, or set the alpha channel",
+      "-alpha option        on, activate, off, deactivate, set, opaque, copy",
+      "                     transparent, extract, background, or shape",
       "-authenticate password",
       "                     decipher image with this password",
       "-channel type        apply option to select image channels",
@@ -106,6 +108,8 @@ static void CompareUsage(void)
       "                     define one or more image format options",
       "-density geometry    horizontal and vertical density of the image",
       "-depth value         image depth",
+      "-dissimilarity-threshold value",
+      "                     maximum RMSE for (sub)image match",
       "-encipher filename   convert plain pixels to cipher pixels",
       "-extract geometry    extract area from image",
       "-format \"string\"     output formatted image characteristics",
@@ -125,11 +129,12 @@ static void CompareUsage(void)
       "-quiet               suppress all warning messages",
       "-quantize colorspace reduce colors in this colorspace",
       "-regard-warnings     pay attention to warning messages",
-      "-respect-parenthesis settings remain in effect until parenthesis boundary",
+      "-respect-parentheses settings remain in effect until parenthesis boundary",
       "-sampling-factor geometry",
       "                     horizontal and vertical sampling factor",
       "-seed value          seed a new sequence of pseudo-random numbers",
       "-set attribute value set an image attribute",
+      "-quality value       JPEG/MIFF/PNG compression level",
       "-size geometry       width and height of image",
       "-transparent-color color",
       "                     transparent color",
@@ -142,7 +147,8 @@ static void CompareUsage(void)
     };
 
   (void) printf("Version: %s\n",GetMagickVersion((unsigned long *) NULL));
-  (void) printf("Copyright: %s\n\n",GetMagickCopyright());
+  (void) printf("Copyright: %s\n",GetMagickCopyright());
+  (void) printf("Features: %s\n\n",GetMagickFeatures());
   (void) printf("Usage: %s [options ...] image reconstruct difference\n",
     GetClientName());
   (void) printf("\nImage Settings:\n");
@@ -160,14 +166,19 @@ static void CompareUsage(void)
   (void) printf(
     "image type as the filename suffix (i.e. image.ps).  Specify 'file' as\n");
   (void) printf("'-' for standard input or output.\n");
-  exit(0);
+  return(MagickFalse);
 }
 
 WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
   int argc,char **argv,char **metadata,ExceptionInfo *exception)
 {
+#define DefaultDissimilarityThreshold  0.31830988618379067154
 #define DestroyCompare() \
 { \
+  if (similarity_image != (Image *) NULL) \
+    similarity_image=DestroyImageList(similarity_image); \
+  if (difference_image != (Image *) NULL) \
+    difference_image=DestroyImageList(difference_image); \
   DestroyImageStack(); \
   for (i=0; i < (long) argc; i++) \
     argv[i]=DestroyString(argv[i]); \
@@ -184,7 +195,7 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
 #define ThrowCompareInvalidArgumentException(option,argument) \
 { \
   (void) ThrowMagickException(exception,GetMagickModule(),OptionError, \
-    "InvalidArgument","`%s': %s",argument,option); \
+    "InvalidArgument","`%s': %s",option,argument); \
   DestroyCompare(); \
   return(MagickFalse); \
 }
@@ -200,12 +211,15 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
     channels;
 
   double
-    distortion;
+    dissimilarity_threshold,
+    distortion,
+    similarity_metric;
 
   Image
     *difference_image,
     *image,
-    *reconstruct_image;
+    *reconstruct_image,
+    *similarity_image;
 
   ImageStack
     image_stack[MaxImageStackDepth+1];
@@ -224,6 +238,9 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
   MetricType
     metric;
 
+  RectangleInfo
+    offset;
+
   register long
     i;
 
@@ -243,14 +260,20 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
         {
           (void) fprintf(stdout,"Version: %s\n",
             GetMagickVersion((unsigned long *) NULL));
-          (void) fprintf(stdout,"Copyright: %s\n\n",GetMagickCopyright());
+          (void) fprintf(stdout,"Copyright: %s\n",GetMagickCopyright());
+          (void) fprintf(stdout,"Features: %s\n\n",GetMagickFeatures());
           return(MagickFalse);
         }
     }
   if (argc < 3)
-    CompareUsage();
+    {
+      (void) CompareUsage();
+      return(MagickTrue);
+    }
   channels=AllChannels;
   difference_image=NewImageList();
+  similarity_image=NewImageList();
+  dissimilarity_threshold=DefaultDissimilarityThreshold;
   distortion=0.0;
   format=(char *) NULL;
   j=1;
@@ -268,7 +291,7 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
   status=ExpandFilenames(&argc,&argv);
   if (status == MagickFalse)
     ThrowCompareException(ResourceLimitError,"MemoryAllocationFailed",
-      strerror(errno));
+      GetExceptionMessage(errno));
   for (i=1; i < (long) (argc-1); i++)
   {
     option=argv[i];
@@ -297,7 +320,7 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
         /*
           Read input image.
         */
-        FireImageStack(MagickTrue,MagickTrue,pend);
+        FireImageStack(MagickFalse,MagickFalse,pend);
         filename=argv[i];
         if ((LocaleCompare(filename,"--") == 0) && (i < (argc-1)))
           filename=argv[++i];
@@ -423,6 +446,8 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
                 argv[i]);
             break;
           }
+        if (LocaleCompare("concurrent",option+1) == 0)
+          break;
         ThrowCompareException(OptionError,"UnrecognizedOption",option)
       }
       case 'd':
@@ -491,6 +516,32 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
               ThrowCompareInvalidArgumentException(option,argv[i]);
             break;
           }
+        if (LocaleCompare("dissimilarity-threshold",option+1) == 0)
+          {
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) argc)
+              ThrowCompareException(OptionError,"MissingArgument",option);
+            if (IsGeometry(argv[i]) == MagickFalse)
+              ThrowCompareInvalidArgumentException(option,argv[i]);
+            if (*option == '+')
+              dissimilarity_threshold=DefaultDissimilarityThreshold;
+            else
+              dissimilarity_threshold=atof(argv[i]);
+            break;
+          }
+        if (LocaleCompare("duration",option+1) == 0)
+          {
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) (argc-1))
+              ThrowCompareException(OptionError,"MissingArgument",option);
+            if (IsGeometry(argv[i]) == MagickFalse)
+              ThrowCompareInvalidArgumentException(option,argv[i]);
+            break;
+          }
         ThrowCompareException(OptionError,"UnrecognizedOption",option)
       }
       case 'e':
@@ -546,7 +597,7 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
       {
         if ((LocaleCompare("help",option+1) == 0) ||
             (LocaleCompare("-help",option+1) == 0))
-          CompareUsage();
+          return(CompareUsage());
         if (LocaleCompare("highlight-color",option+1) == 0)
           {
             if (*option == '+')
@@ -588,6 +639,9 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
             char
               *p;
 
+            double
+              value;
+
             long
               resource;
 
@@ -604,11 +658,29 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
             i++;
             if (i == (long) argc)
               ThrowCompareException(OptionError,"MissingArgument",option);
-            (void) strtod(argv[i],&p);
+            value=strtod(argv[i],&p);
             if ((p == argv[i]) && (LocaleCompare("unlimited",argv[i]) != 0))
               ThrowCompareInvalidArgumentException(option,argv[i]);
             break;
           }
+        if (LocaleCompare("list",option+1) == 0)
+          {
+            long
+              list;
+
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) argc)
+              ThrowCompareException(OptionError,"MissingArgument",option);
+            list=ParseMagickOption(MagickListOptions,MagickFalse,argv[i]);
+            if (list < 0)
+              ThrowCompareException(OptionError,"UnrecognizedListType",argv[i]);
+            (void) MogrifyImageInfo(image_info,(int) (i-j+1),(const char **)
+              argv+j,exception);
+            DestroyCompare();
+            return(MagickTrue);
+          }
         if (LocaleCompare("log",option+1) == 0)
           {
             if (*option == '+')
@@ -712,7 +784,7 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
       {
         if (LocaleCompare("regard-warnings",option+1) == 0)
           break;
-        if (LocaleCompare("respect-parenthesis",option+1) == 0)
+        if (LocaleNCompare("respect-parentheses",option+1,17) == 0)
           {
             respect_parenthesis=(*option == '-') ? MagickTrue : MagickFalse;
             break;
@@ -806,7 +878,8 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
           {
             (void) fprintf(stdout,"Version: %s\n",
               GetMagickVersion((unsigned long *) NULL));
-            (void) fprintf(stdout,"Copyright: %s\n\n",GetMagickCopyright());
+            (void) fprintf(stdout,"Copyright: %s\n",GetMagickCopyright());
+            (void) fprintf(stdout,"Features: %s\n\n",GetMagickFeatures());
             break;
           }
         if (LocaleCompare("virtual-pixel",option+1) == 0)
@@ -833,7 +906,7 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
       default:
         ThrowCompareException(OptionError,"UnrecognizedOption",option)
     }
-    fire=ParseMagickOption(MagickImageListOptions,MagickFalse,option+1) < 0 ? 
+    fire=ParseMagickOption(MagickImageListOptions,MagickFalse,option+1) < 0 ?
       MagickFalse : MagickTrue;
     if (fire != MagickFalse)
       FireImageStack(MagickTrue,MagickTrue,MagickTrue);
@@ -842,72 +915,121 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
     ThrowCompareException(OptionError,"UnbalancedParenthesis",argv[i]);
   if (i-- != (long) (argc-1))
     ThrowCompareException(OptionError,"MissingAnImageFilename",argv[i]);
-  if ((image == (Image *) NULL) ||
-      (GetImageListLength(image) < 2))
+  if ((image == (Image *) NULL) || (GetImageListLength(image) < 2))
+    ThrowCompareException(OptionError,"MissingAnImageFilename",argv[i]);
+  FinalizeImageSettings(image_info,image,MagickTrue);
+  if ((image == (Image *) NULL) || (GetImageListLength(image) < 2))
     ThrowCompareException(OptionError,"MissingAnImageFilename",argv[i]);
-  FinalizeImageSettings(image_info,image);
   image=GetImageFromList(image,0);
   reconstruct_image=GetImageFromList(image,1);
-  difference_image=CompareImageChannels(image,reconstruct_image,channels,
-    metric,&distortion,exception);
+  similarity_image=SimilarityImage(image,reconstruct_image,&offset,
+    &similarity_metric,exception);
+  if (similarity_metric > dissimilarity_threshold)
+    ThrowCompareException(ImageError,"ImagesTooDissimilar",image->filename);
+  if ((reconstruct_image->columns == image->columns) &&
+      (reconstruct_image->rows == image->rows))
+    difference_image=CompareImageChannels(image,reconstruct_image,channels,
+      metric,&distortion,exception);
+  else
+    if (similarity_image != (Image *) NULL)
+      {
+        Image
+          *composite_image;
+
+        /*
+          Determine if reconstructed image is a subimage of the image.
+        */
+        composite_image=CloneImage(image,0,0,MagickTrue,exception);
+        if (composite_image == (Image *) NULL)
+          difference_image=CompareImageChannels(image,reconstruct_image,
+            channels,metric,&distortion,exception);
+        else
+          {
+            (void) CompositeImage(composite_image,CopyCompositeOp,
+              reconstruct_image,offset.x,offset.y);
+            difference_image=CompareImageChannels(image,composite_image,
+              channels,metric,&distortion,exception);
+            if (difference_image != (Image *) NULL)
+              {
+                difference_image->page.x=offset.x;
+                difference_image->page.y=offset.y;
+              }
+            composite_image=DestroyImage(composite_image);
+          }
+        if (difference_image == (Image *) NULL)
+          similarity_image=DestroyImage(similarity_image);
+        else
+          {
+            AppendImageToList(&difference_image,similarity_image);
+            similarity_image=(Image *) NULL;
+          }
+      }
   if (difference_image == (Image *) NULL)
     status=0;
   else
     {
       if (image_info->verbose != MagickFalse)
         (void) IsImagesEqual(image,reconstruct_image);
-      status&=WriteImages(image_info,difference_image,argv[argc-1],exception);
-      if ((metadata != (char **) NULL) && (format != (char *) NULL))
-        {
-          char
-            *text;
-
-          text=InterpretImageProperties(image_info,difference_image,format);
-          if (text == (char *) NULL)
-            ThrowCompareException(ResourceLimitError,"MemoryAllocationFailed",
-              strerror(errno));
-          (void) ConcatenateString(&(*metadata),text);
-          (void) ConcatenateString(&(*metadata),"\n");
-          text=DestroyString(text);
-        }
-      difference_image=DestroyImageList(difference_image);
+      if (*difference_image->magick == '\0')
+        (void) CopyMagickString(difference_image->magick,image->magick,
+          MaxTextExtent);
       if (image_info->verbose == MagickFalse)
-        switch (metric)
         {
-          case MeanAbsoluteErrorMetric:
-          case MeanSquaredErrorMetric:
-          case RootMeanSquaredErrorMetric:
-          case PeakAbsoluteErrorMetric:
+          switch (metric)
           {
-            (void) fprintf(stderr,"%g (%g)\n",distortion,(double) (QuantumScale*
-              distortion));
-            break;
+            case MeanAbsoluteErrorMetric:
+            case MeanSquaredErrorMetric:
+            case RootMeanSquaredErrorMetric:
+            case PeakAbsoluteErrorMetric:
+            {
+              (void) fprintf(stderr,"%g (%g)",QuantumRange*distortion,(double)
+                distortion);
+              if ((reconstruct_image->columns != image->columns) ||
+                  (reconstruct_image->rows != image->rows))
+                (void) fprintf(stderr," @ %ld,%ld",difference_image->page.x,
+                  difference_image->page.y);
+              (void) fprintf(stderr,"\n");
+              break;
+            }
+            case AbsoluteErrorMetric:
+            case PeakSignalToNoiseRatioMetric:
+            {
+              (void) fprintf(stderr,"%g",distortion);
+              if ((reconstruct_image->columns != image->columns) ||
+                  (reconstruct_image->rows != image->rows))
+                (void) fprintf(stderr," @ %ld,%ld",difference_image->page.x,
+                  difference_image->page.y);
+              (void) fprintf(stderr,"\n");
+              break;
+            }
+            case MeanErrorPerPixelMetric:
+            {
+              (void) fprintf(stderr,"%g (%g, %g)",distortion,
+                image->error.normalized_mean_error,
+                image->error.normalized_maximum_error);
+              if ((reconstruct_image->columns != image->columns) ||
+                  (reconstruct_image->rows != image->rows))
+                (void) fprintf(stderr," @ %ld,%ld",difference_image->page.x,
+                  difference_image->page.y);
+              (void) fprintf(stderr,"\n");
+              break;
+            }
+            case UndefinedMetric:
+              break;
           }
-          case AbsoluteErrorMetric:
-          case PeakSignalToNoiseRatioMetric:
-          {
-            (void) fprintf(stderr,"%g\n",distortion);
-            break;
-          }
-          case MeanErrorPerPixelMetric:
-          {
-            (void) fprintf(stderr,"%g (%g, %g)\n",distortion,
-              image->error.normalized_mean_error,
-              image->error.normalized_maximum_error);
-            break;
-          }
-          case UndefinedMetric:
-            break;
         }
       else
         {
           double
             *channel_distortion;
 
- 
           channel_distortion=GetImageChannelDistortions(image,reconstruct_image,
             metric,&image->exception);
           (void) fprintf(stderr,"Image: %s\n",image->filename);
+          if ((reconstruct_image->columns != image->columns) ||
+              (reconstruct_image->rows != image->rows))
+            (void) fprintf(stderr,"Offset: %ld,%ld\n",difference_image->page.x,
+              difference_image->page.y);
           (void) fprintf(stderr,"  Channel distortion: %s\n",
             MagickOptionToMnemonic(MagickMetricOptions,(long) metric));
           switch (metric)
@@ -923,54 +1045,54 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
                 default:
                 {
                   (void) fprintf(stderr,"    red: %g (%g)\n",
-                    channel_distortion[RedChannel],(double) QuantumScale*
+                    QuantumRange*channel_distortion[RedChannel],
                     channel_distortion[RedChannel]);
                   (void) fprintf(stderr,"    green: %g (%g)\n",
-                    channel_distortion[GreenChannel],(double) QuantumScale*
+                    QuantumRange*channel_distortion[GreenChannel],
                     channel_distortion[GreenChannel]);
                   (void) fprintf(stderr,"    blue: %g (%g)\n",
-                    channel_distortion[BlueChannel],(double) QuantumScale*
+                    QuantumRange*channel_distortion[BlueChannel],
                     channel_distortion[BlueChannel]);
                   if (image->matte != MagickFalse)
                     (void) fprintf(stderr,"    alpha: %g (%g)\n",
-                      channel_distortion[OpacityChannel],(double) QuantumScale*
+                      QuantumRange*channel_distortion[OpacityChannel],
                       channel_distortion[OpacityChannel]);
                   break;
                 }
                 case CMYKColorspace:
                 {
                   (void) fprintf(stderr,"    cyan: %g (%g)\n",
-                    channel_distortion[CyanChannel],(double) QuantumScale*
+                    QuantumRange*channel_distortion[CyanChannel],
                     channel_distortion[CyanChannel]);
                   (void) fprintf(stderr,"    magenta: %g (%g)\n",
-                    channel_distortion[MagentaChannel],(double) QuantumScale*
+                    QuantumRange*channel_distortion[MagentaChannel],
                     channel_distortion[MagentaChannel]);
                   (void) fprintf(stderr,"    yellow: %g (%g)\n",
-                    channel_distortion[YellowChannel],(double) QuantumScale*
+                    QuantumRange*channel_distortion[YellowChannel],
                     channel_distortion[YellowChannel]);
                   (void) fprintf(stderr,"    black: %g (%g)\n",
-                    channel_distortion[BlackChannel],(double) QuantumScale*
+                    QuantumRange*channel_distortion[BlackChannel],
                     channel_distortion[BlackChannel]);
                   if (image->matte != MagickFalse)
                     (void) fprintf(stderr,"    alpha: %g (%g)\n",
-                      channel_distortion[OpacityChannel],(double) QuantumScale*
+                      QuantumRange*channel_distortion[OpacityChannel],
                       channel_distortion[OpacityChannel]);
                   break;
                 }
                 case GRAYColorspace:
                 {
                   (void) fprintf(stderr,"    gray: %g (%g)\n",
-                    channel_distortion[GrayChannel],(double) QuantumScale*
+                    QuantumRange*channel_distortion[GrayChannel],
                     channel_distortion[GrayChannel]);
                   if (image->matte != MagickFalse)
                     (void) fprintf(stderr,"    alpha: %g (%g)\n",
-                      channel_distortion[OpacityChannel],(double) QuantumScale*
+                      QuantumRange*channel_distortion[OpacityChannel],
                       channel_distortion[OpacityChannel]);
                   break;
                 }
               }
               (void) fprintf(stderr,"    all: %g (%g)\n",
-                channel_distortion[AllChannels],(double) QuantumScale*
+                QuantumRange*channel_distortion[AllChannels],
                 channel_distortion[AllChannels]);
               break;
             }
@@ -1036,6 +1158,21 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
           channel_distortion=(double *) RelinquishMagickMemory(
             channel_distortion);
         }
+      status&=WriteImages(image_info,difference_image,argv[argc-1],exception);
+      if ((metadata != (char **) NULL) && (format != (char *) NULL))
+        {
+          char
+            *text;
+
+          text=InterpretImageProperties(image_info,difference_image,format);
+          if (text == (char *) NULL)
+            ThrowCompareException(ResourceLimitError,"MemoryAllocationFailed",
+              GetExceptionMessage(errno));
+          (void) ConcatenateString(&(*metadata),text);
+          (void) ConcatenateString(&(*metadata),"\n");
+          text=DestroyString(text);
+        }
+      difference_image=DestroyImageList(difference_image);
     }
   DestroyCompare();
   return(status != 0 ? MagickTrue : MagickFalse);
