@@ -114,7 +114,7 @@ static MagickBooleanType DisplayUsage(void)
     },
     *operators[]=
     {
-      "-auto-orient         automatically orient image",
+      "-auto-orient         automagically orient image",
       "-border geometry     surround image with a border of color",
       "-clip                clip along the first path from the 8BIM profile",
       "-clip-path id        clip along a named path from the 8BIM profile",
@@ -220,7 +220,8 @@ static MagickBooleanType DisplayUsage(void)
     };
 
   (void) printf("Version: %s\n",GetMagickVersion((unsigned long *) NULL));
-  (void) printf("Copyright: %s\n\n",GetMagickCopyright());
+  (void) printf("Copyright: %s\n",GetMagickCopyright());
+  (void) printf("Features: %s\n\n",GetMagickFeatures());
   (void) printf("Usage: %s [options ...] file [ [options ...] file ...]\n",
     GetClientName());
   (void) printf("\nImage Settings:\n");
@@ -357,7 +358,8 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
         {
           (void) fprintf(stdout,"Version: %s\n",
             GetMagickVersion((unsigned long *) NULL));
-          (void) fprintf(stdout,"Copyright: %s\n\n",GetMagickCopyright());
+          (void) fprintf(stdout,"Copyright: %s\n",GetMagickCopyright());
+          (void) fprintf(stdout,"Features: %s\n\n",GetMagickFeatures());
           return(MagickFalse);
         }
     }
@@ -397,8 +399,6 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
       Check command line for server name.
     */
     option=argv[i];
-    if (IsMagickOption(option) == MagickFalse)
-      continue;
     if (LocaleCompare("display",option+1) == 0)
       {
         /*
@@ -448,7 +448,6 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
     Parse command line.
   */
   iteration=0;
-  iterations=0;
   for (i=1; ((i <= (long) argc) && ((state & ExitState) == 0)); i++)
   {
     if (i < (long) argc)
@@ -517,6 +516,9 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
           continue;
         AppendImageStack(images);
         FinalizeImageSettings(image_info,image,MagickFalse);
+        iterations=0;
+        if (i == (argc-1))
+          iterations=image->iterations;
         do
         {
           /*
@@ -530,7 +532,10 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
               */
               status=XDisplayBackgroundImage(display,&resource_info,image);
               if (status != MagickFalse)
-                state|=RetainColorsState;
+                {
+                  state|=RetainColorsState;
+                  status=MagickFalse;
+                }
             }
           else
             do
@@ -647,11 +652,10 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
           }
         if (resource_info.window_id != (char *) NULL)
           state|=ExitState;
-        if ((iterations != 0) && (iteration == (long) iterations))
+        if ((iterations != 0) && (++iteration == (long) iterations))
           state|=ExitState;
         if (LocaleCompare(filename,"-") == 0)
           state|=ExitState;
-        iteration++;
         continue;
       }
     pend=image != (Image *) NULL ? MagickTrue : MagickFalse;
@@ -857,6 +861,8 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
                 argv[i]);
             break;
           }
+        if (LocaleCompare("concurrent",option+1) == 0)
+          break;
         if (LocaleCompare("contrast",option+1) == 0)
           break;
         if (LocaleCompare("crop",option+1) == 0)
@@ -1007,6 +1013,17 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
             quantize_info->dither_method=(DitherMethod) method;
             break;
           }
+        if (LocaleCompare("duration",option+1) == 0)
+          {
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) (argc-1))
+              ThrowDisplayException(OptionError,"MissingArgument",option);
+            if (IsGeometry(argv[i]) == MagickFalse)
+              ThrowDisplayInvalidArgumentException(option,argv[i]);
+            break;
+          }
         ThrowDisplayException(OptionError,"UnrecognizedOption",option);
       }
       case 'e':
@@ -1761,7 +1778,8 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
           {
             (void) fprintf(stdout,"Version: %s\n",
               GetMagickVersion((unsigned long *) NULL));
-            (void) fprintf(stdout,"Copyright: %s\n\n",GetMagickCopyright());
+            (void) fprintf(stdout,"Copyright: %s\n",GetMagickCopyright());
+            (void) fprintf(stdout,"Features: %s\n\n",GetMagickFeatures());
             break;
           }
         if (LocaleCompare("visual",option+1) == 0)
