@@ -23,7 +23,7 @@
 %                                March 2003                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -72,7 +72,7 @@ struct _PixelIterator
   ExceptionInfo
     *exception;
 
-  ViewInfo
+  CacheView
     *view;
 
   RectangleInfo
@@ -298,7 +298,7 @@ WandExport PixelIterator *NewPixelIterator(MagickWand *wand)
   unsigned long
     depth;
 
-  ViewInfo
+  CacheView
     *view;
 
   depth=MAGICKCORE_QUANTUM_DEPTH;
@@ -310,12 +310,12 @@ WandExport PixelIterator *NewPixelIterator(MagickWand *wand)
   if (image == (Image *) NULL)
     return((PixelIterator *) NULL);
   view=AcquireCacheView(image);
-  if (view == (ViewInfo *) NULL)
+  if (view == (CacheView *) NULL)
     return((PixelIterator *) NULL);
   iterator=(PixelIterator *) AcquireMagickMemory(sizeof(*iterator));
   if (iterator == (PixelIterator *) NULL)
     ThrowWandFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
-      strerror(errno));
+      GetExceptionMessage(errno));
   (void) ResetMagickMemory(iterator,0,sizeof(*iterator));
   iterator->id=AcquireWandId();
   (void) FormatMagickString(iterator->name,MaxTextExtent,"%s-%lu",
@@ -411,7 +411,7 @@ WandExport PixelIterator *NewPixelRegionIterator(MagickWand *wand,const long x,
   unsigned long
     depth;
 
-  ViewInfo
+  CacheView
     *view;
 
   assert(wand != (MagickWand *) NULL);
@@ -425,7 +425,7 @@ WandExport PixelIterator *NewPixelRegionIterator(MagickWand *wand,const long x,
   if (image == (Image *) NULL)
     return((PixelIterator *) NULL);
   view=AcquireCacheView(image);
-  if (view == (ViewInfo *) NULL)
+  if (view == (CacheView *) NULL)
     return((PixelIterator *) NULL);
   iterator=(PixelIterator *) AcquireMagickMemory(sizeof(*iterator));
   if (iterator == (PixelIterator *) NULL)
@@ -504,7 +504,7 @@ WandExport PixelWand **PixelGetCurrentIteratorRow(PixelIterator *iterator,
         iterator->view));
       return((PixelWand **) NULL);
     }
-  indexes=GetCacheViewVirtualIndexes(iterator->view);
+  indexes=GetCacheViewVirtualIndexQueue(iterator->view);
   for (x=0; x < (long) iterator->region.width; x++)
     PixelSetQuantumColor(iterator->pixel_wands[x],pixels+x);
   if (GetCacheViewColorspace(iterator->view) == CMYKColorspace)
@@ -695,7 +695,7 @@ WandExport PixelWand **PixelGetNextIteratorRow(PixelIterator *iterator,
         iterator->view));
       return((PixelWand **) NULL);
     }
-  indexes=GetCacheViewVirtualIndexes(iterator->view);
+  indexes=GetCacheViewVirtualIndexQueue(iterator->view);
   for (x=0; x < (long) iterator->region.width; x++)
     PixelSetQuantumColor(iterator->pixel_wands[x],pixels+x);
   if (GetCacheViewColorspace(iterator->view) == CMYKColorspace)
@@ -773,7 +773,7 @@ WandExport PixelWand **PixelGetPreviousIteratorRow(PixelIterator *iterator,
         iterator->view));
       return((PixelWand **) NULL);
     }
-  indexes=GetCacheViewVirtualIndexes(iterator->view);
+  indexes=GetCacheViewVirtualIndexQueue(iterator->view);
   for (x=0; x < (long) iterator->region.width; x++)
     PixelSetQuantumColor(iterator->pixel_wands[x],pixels+x);
   if (GetCacheViewColorspace(iterator->view) == CMYKColorspace)
@@ -948,14 +948,14 @@ WandExport MagickBooleanType PixelSyncIterator(PixelIterator *iterator)
   ExceptionInfo
     *exception;
 
-  IndexPacket
-    *indexes;
+  register IndexPacket
+    *restrict indexes;
 
   register long
     x;
 
   register PixelPacket
-    *pixels;
+    *restrict pixels;
 
   assert(iterator != (const PixelIterator *) NULL);
   assert(iterator->signature == WandSignature);
@@ -972,7 +972,7 @@ WandExport MagickBooleanType PixelSyncIterator(PixelIterator *iterator)
         iterator->view));
       return(MagickFalse);
     }
-  indexes=GetCacheViewAuthenticIndexes(iterator->view);
+  indexes=GetCacheViewAuthenticIndexQueue(iterator->view);
   for (x=0; x < (long) iterator->region.width; x++)
     PixelGetQuantumColor(iterator->pixel_wands[x],pixels+x);
   if (GetCacheViewColorspace(iterator->view) == CMYKColorspace)
