@@ -51,6 +51,7 @@
 #include "wand/magick-wand-private.h"
 #include "wand/wand.h"
 #include "magick/monitor-private.h"
+#include "magick/thread-private.h"
 /*
  Define declarations.
 */
@@ -73,7 +74,7 @@ struct _PixelView
   MagickWand
     *wand;
 
-  ViewInfo
+  CacheView
     *view;
 
   RectangleInfo
@@ -187,7 +188,8 @@ static PixelWand ***DestroyPixelsThreadSet(PixelWand ***pixel_wands,
   for (i=0; i < (long) number_threads; i++)
     if (pixel_wands[i] != (PixelWand **) NULL)
       pixel_wands[i]=DestroyPixelWands(pixel_wands[i],number_wands);
-  return((PixelWand ***) RelinquishMagickMemory(pixel_wands));
+  pixel_wands=(PixelWand ***) RelinquishAlignedMemory(pixel_wands);
+  return(pixel_wands);
 }
 
 WandExport PixelView *DestroyPixelView(PixelView *pixel_view)
@@ -283,7 +285,7 @@ WandExport MagickBooleanType DuplexTransferPixelViewIterator(
   progress=0;
   exception=destination->exception;
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(dynamic) shared(progress,status)
+  #pragma omp parallel for schedule(static,1) shared(progress,status)
 #endif
   for (y=source->region.y; y < (long) source->region.height; y++)
   {
@@ -291,26 +293,26 @@ WandExport MagickBooleanType DuplexTransferPixelViewIterator(
       sync;
 
     register const IndexPacket
-      *duplex_indexes,
-      *indexes;
+      *restrict duplex_indexes,
+      *restrict indexes;
 
     register const PixelPacket
-      *duplex_pixels,
-      *pixels;
+      *restrict duplex_pixels,
+      *restrict pixels;
 
     register IndexPacket
-      *destination_indexes;
+      *restrict destination_indexes;
 
     register long
       id,
       x;
 
     register PixelPacket
-      *destination_pixels;
+      *restrict destination_pixels;
 
     if (status == MagickFalse)
       continue;
-    id=GetPixelCacheThreadId();
+    id=GetOpenMPThreadId();
     pixels=GetCacheViewVirtualPixels(source->view,source->region.x,y,
       source->region.width,1,source->exception);
     if (pixels == (const PixelPacket *) NULL)
@@ -383,7 +385,7 @@ WandExport MagickBooleanType DuplexTransferPixelViewIterator(
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp critical
+  #pragma omp critical (MagickWand_DuplexTransferPixelViewIterator)
 #endif
         proceed=SetImageProgress(source_image,DuplexTransferPixelViewTag,
           progress++,source->region.height);
@@ -543,7 +545,7 @@ WandExport MagickBooleanType GetPixelViewIterator(PixelView *source,
   status=MagickTrue;
   progress=0;
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(dynamic) shared(progress,status)
+  #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
 #endif
   for (y=source->region.y; y < (long) source->region.height; y++)
   {
@@ -559,7 +561,7 @@ WandExport MagickBooleanType GetPixelViewIterator(PixelView *source,
 
     if (status == MagickFalse)
       continue;
-    id=GetPixelCacheThreadId();
+    id=GetOpenMPThreadId();
     pixels=GetCacheViewVirtualPixels(source->view,source->region.x,y,
       source->region.width,1,source->exception);
     if (pixels == (const PixelPacket *) NULL)
@@ -584,7 +586,7 @@ WandExport MagickBooleanType GetPixelViewIterator(PixelView *source,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp critical
+  #pragma omp critical (MagickWand_GetPixelViewIterator)
 #endif
         proceed=SetImageProgress(source_image,GetPixelViewTag,progress++,
           source->region.height);
@@ -624,7 +626,7 @@ WandExport PixelWand **GetPixelViewPixels(const PixelView *pixel_view)
 
   assert(pixel_view != (PixelView *) NULL);
   assert(pixel_view->signature == WandSignature);
-  id=GetPixelCacheThreadId();
+  id=GetOpenMPThreadId();
   return(pixel_view->pixel_wands[id]);
 }
 
@@ -815,7 +817,7 @@ static PixelWand ***AcquirePixelsThreadSet(const unsigned long number_wands,
   register long
     i;
 
-  pixel_wands=(PixelWand ***) AcquireQuantumMemory(number_threads,
+  pixel_wands=(PixelWand ***) AcquireAlignedMemory(number_threads,
     sizeof(*pixel_wands));
   if (pixel_wands == (PixelWand ***) NULL)
     return((PixelWand ***) NULL);
@@ -839,7 +841,7 @@ WandExport PixelView *NewPixelView(MagickWand *wand)
   pixel_view=(PixelView *) AcquireMagickMemory(sizeof(*pixel_view));
   if (pixel_view == (PixelView *) NULL)
     ThrowWandFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
-      strerror(errno));
+      GetExceptionMessage(errno));
   (void) ResetMagickMemory(pixel_view,0,sizeof(*pixel_view));
   pixel_view->id=AcquireWandId();
   (void) FormatMagickString(pixel_view->name,MaxTextExtent,"%s-%lu",
@@ -849,12 +851,12 @@ WandExport PixelView *NewPixelView(MagickWand *wand)
   pixel_view->view=AcquireCacheView(pixel_view->wand->images);
   pixel_view->region.width=wand->images->columns;
   pixel_view->region.height=wand->images->rows;
-  pixel_view->number_threads=GetPixelCacheMaximumThreads();
+  pixel_view->number_threads=GetOpenMPMaximumThreads();
   pixel_view->pixel_wands=AcquirePixelsThreadSet(pixel_view->region.width,
     pixel_view->number_threads);
   if (pixel_view->pixel_wands == (PixelWand ***) NULL)
     ThrowWandFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
-      strerror(errno));
+      GetExceptionMessage(errno));
   pixel_view->debug=IsEventLogging();
   pixel_view->signature=WandSignature;
   return(pixel_view);
@@ -898,7 +900,7 @@ WandExport PixelView *NewPixelViewRegion(MagickWand *wand,const long x,
   pixel_view=(PixelView *) AcquireMagickMemory(sizeof(*pixel_view));
   if (pixel_view == (PixelView *) NULL)
     ThrowWandFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
-      strerror(errno));
+      GetExceptionMessage(errno));
   (void) ResetMagickMemory(pixel_view,0,sizeof(*pixel_view));
   pixel_view->id=AcquireWandId();
   (void) FormatMagickString(pixel_view->name,MaxTextExtent,"%s-%lu",
@@ -910,12 +912,12 @@ WandExport PixelView *NewPixelViewRegion(MagickWand *wand,const long x,
   pixel_view->region.height=height;
   pixel_view->region.x=x;
   pixel_view->region.y=y;
-  pixel_view->number_threads=GetPixelCacheMaximumThreads();
+  pixel_view->number_threads=GetOpenMPMaximumThreads();
   pixel_view->pixel_wands=AcquirePixelsThreadSet(pixel_view->region.width,
     pixel_view->number_threads);
   if (pixel_view->pixel_wands == (PixelWand ***) NULL)
     ThrowWandFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
-      strerror(errno));
+      GetExceptionMessage(errno));
   pixel_view->debug=IsEventLogging();
   pixel_view->signature=WandSignature;
   return(pixel_view);
@@ -989,7 +991,7 @@ WandExport MagickBooleanType SetPixelViewIterator(PixelView *destination,
   progress=0;
   exception=destination->exception;
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(dynamic) shared(progress,status)
+  #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
 #endif
   for (y=destination->region.y; y < (long) destination->region.height; y++)
   {
@@ -997,18 +999,18 @@ WandExport MagickBooleanType SetPixelViewIterator(PixelView *destination,
       sync;
 
     register IndexPacket
-      *indexes;
+      *restrict indexes;
 
     register long
       id,
       x;
 
     register PixelPacket
-      *pixels;
+      *restrict pixels;
 
     if (status == MagickFalse)
       continue;
-    id=GetPixelCacheThreadId();
+    id=GetOpenMPThreadId();
     pixels=GetCacheViewAuthenticPixels(destination->view,destination->region.x,
       y,destination->region.width,1,exception);
     if (pixels == (PixelPacket *) NULL)
@@ -1039,7 +1041,7 @@ WandExport MagickBooleanType SetPixelViewIterator(PixelView *destination,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp critical
+  #pragma omp critical (MagickWand_SetPixelViewIterator)
 #endif
         proceed=SetImageProgress(destination_image,SetPixelViewTag,progress++,
           destination->region.height);
@@ -1123,7 +1125,7 @@ WandExport MagickBooleanType TransferPixelViewIterator(PixelView *source,
   progress=0;
   exception=destination->exception;
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(dynamic) shared(progress,status)
+  #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
 #endif
   for (y=source->region.y; y < (long) source->region.height; y++)
   {
@@ -1131,24 +1133,24 @@ WandExport MagickBooleanType TransferPixelViewIterator(PixelView *source,
       sync;
 
     register const IndexPacket
-      *indexes;
+      *restrict indexes;
 
     register const PixelPacket
-      *pixels;
+      *restrict pixels;
 
     register IndexPacket
-      *destination_indexes;
+      *restrict destination_indexes;
 
     register long
       id,
       x;
 
     register PixelPacket
-      *destination_pixels;
+      *restrict destination_pixels;
 
     if (status == MagickFalse)
       continue;
-    id=GetPixelCacheThreadId();
+    id=GetOpenMPThreadId();
     pixels=GetCacheViewVirtualPixels(source->view,source->region.x,y,
       source->region.width,1,source->exception);
     if (pixels == (const PixelPacket *) NULL)
@@ -1203,7 +1205,7 @@ WandExport MagickBooleanType TransferPixelViewIterator(PixelView *source,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp critical
+  #pragma omp critical (MagickWand_TransferPixelViewIterator)
 #endif
         proceed=SetImageProgress(source_image,TransferPixelViewTag,progress++,
           source->region.height);
@@ -1281,23 +1283,23 @@ WandExport MagickBooleanType UpdatePixelViewIterator(PixelView *source,
   progress=0;
   exception=source->exception;
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(dynamic) shared(progress,status)
+  #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
 #endif
   for (y=source->region.y; y < (long) source->region.height; y++)
   {
     register IndexPacket
-      *indexes;
+      *restrict indexes;
 
     register long
       id,
       x;
 
     register PixelPacket
-      *pixels;
+      *restrict pixels;
 
     if (status == MagickFalse)
       continue;
-    id=GetPixelCacheThreadId();
+    id=GetOpenMPThreadId();
     pixels=GetCacheViewAuthenticPixels(source->view,source->region.x,y,
       source->region.width,1,exception);
     if (pixels == (PixelPacket *) NULL)
@@ -1331,7 +1333,7 @@ WandExport MagickBooleanType UpdatePixelViewIterator(PixelView *source,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp critical
+  #pragma omp critical (MagickWand_UpdatePixelViewIterator)
 #endif
         proceed=SetImageProgress(source_image,UpdatePixelViewTag,progress++,
           source->region.height);
