@@ -387,7 +387,7 @@ int main(int argc,char **argv)
   MagickWand
     *clone_wand,
     *magick_wand;
- 
+
   PixelIterator
     *iterator;
 
@@ -403,7 +403,7 @@ int main(int argc,char **argv)
   size_t
     length;
 
-  unsigned char 
+  unsigned char
     *profile;
 
   unsigned int
@@ -418,6 +418,9 @@ int main(int argc,char **argv)
     number_wands,
     rows;
 
+  (void) argc;
+  (void) argv;
+  MagickWandGenesis();
   magick_wand=NewMagickWand();
   (void) MagickSetSize(magick_wand,640,480);
   (void) MagickGetSize(magick_wand,&columns,&rows);
@@ -542,11 +545,12 @@ int main(int argc,char **argv)
       };
 
     (void) MagickSetIteratorIndex(magick_wand,2);
-    status=MagickQueueAuthenticPixels(magick_wand,10,10,3,3,"RGB",CharPixel,
+    status=MagickImportImagePixels(magick_wand,10,10,3,3,"RGB",CharPixel,
       primary_colors);
     if (status == MagickFalse)
       ThrowAPIException(magick_wand);
-    status=MagickGetAuthenticPixels(magick_wand,10,10,3,3,"RGB",CharPixel,pixels);
+    status=MagickExportImagePixels(magick_wand,10,10,3,3,"RGB",CharPixel,
+      pixels);
     if (status == MagickFalse)
       ThrowAPIException(magick_wand);
     for (i=0; i < 9; i++)
@@ -682,5 +686,5 @@ int main(int argc,char **argv)
   profile=(unsigned char *) MagickRelinquishMemory(profile);
   magick_wand=DestroyMagickWand(magick_wand);
   (void) fprintf(stdout,"Wand tests pass.\n");
-  return(0); 
+  return(0);
 }
